   SUBROUTINE oas_clm_define(local_size, global_size)
!
!**** *oas_clm_define*  - Define grids etc for OASIS
!
!     Purpose. 
!     --------
!        Define grid and field information for land scheme
!        exchange between atm, land and COUPLER. (OASIS3 software)
!
!**   Interface.
!     ----------
!       *CALL*  *oas_clm_define*
!
!     Input:
!     -----
!
!     Output:
!     ------
!
!     Method:
!     ------
!        
!     Externals:
!     ---------
!       prism_init, prism_init_comp, prism_get_localcomm : prism library
!
!     Modifications.
!     --------------
!       E. Maisonnave : adapted to CLM.3
!                     : adapted to CLM.4

   USE oas_clm_vardef

   USE mct_mod
!EM portage blizzard   use spmdMod     , only : MPI_INTEGER
   USE mod_prism
   USE netcdf

   IMPLICIT NONE

   INTEGER        , intent(in), OPTIONAL :: local_size ! 1D grid local dimension
   INTEGER        , intent(in), OPTIONAL :: global_size ! 1D grid global dimension

   INTEGER         :: igrid     ! ids returned by prism_def_grid
   INTEGER         :: iptid     ! ids returned by prism_set_points
   INTEGER         :: imskid    ! ids returned by prism_set_mask

   INTEGER         :: var_nodims(2)    ! 
   INTEGER         :: ipshape(2)    ! 
   INTEGER         :: ji, jj, jg, jgm1 ! local loop indicees
   INTEGER         :: index ! local loop indicees
   INTEGER         :: my_task, ntask, itype, offset
   INTEGER         :: write_aux_files, nc_grdid, il_varid
   INTEGER         :: total_size

   INTEGER :: igparal(4)     ! shape of arrays passed to PSMILe
   INTEGER ,ALLOCATABLE :: decomp_size(:)

   CHARACTER(len=4)           :: clgrd='clme'

   REAL(KIND=r8)  :: dx, lat_incr
   REAL(KIND=r8), ALLOCATABLE :: zclo(:,:), zcla(:,:)

   ! If missing OASIS auxilary files, create them
   LOGICAL :: pe_definition = .FALSE.

   INTEGER ,parameter :: master_task=0 ! task number of master task

   !! 
   IF ( IOASISDEBUGLVL == 1 ) THEN
     WRITE(6,'(A)')'CLM is setting up OASIS interface'
     CALL flush(6)
   ENDIF

   call mpi_comm_rank(kl_comm, my_task, nerror)
   call mpi_comm_size(kl_comm, ntask, nerror)

   ! We will have to exchange integers via MPI routines
   itype = MPI_INTEGER

   ! PE participating to definition
!EM   IF (present(local_size) ) pe_definition = .TRUE.
! EM bug blizzard ?
   pe_definition = .TRUE.

   !
   ! All declaration through master processor 
   !
   IF ( my_task == master_task ) THEN
      !
      write_aux_files = 1
      !
      ! Check grids.nc file
      ! Warning: if grids.nc exists, other OASIS auxilliary file must exist too
      IF (NF90_OPEN('grids.nc', NF90_NOWRITE, nc_grdid) == NF90_NOERR) THEN
         WRITE(nulout,*) 'CLM: grids.nc already exists'
         IF (NF90_INQ_VARID(nc_grdid,clgrd//'.lon', il_varid) == NF90_NOERR ) write_aux_files = 0
      ENDIF

      IF ( write_aux_files == 1 ) THEN

         write(nulout,*) 'CLM: Define OASIS auxilliary files'
         call flush(nulout)

         CALL prism_start_grids_writing (write_aux_files)
         ALLOCATE( zclo(global_size, 4), stat = nerror )
         IF ( nerror > 0 ) THEN
            CALL prism_abort_proto( ncomp_id, 'oas_clm_define', 'Failure in allocating zclo' )
            RETURN
         ENDIF
         ALLOCATE( zcla(global_size, 4), stat = nerror )
         IF ( nerror > 0 ) THEN
            CALL prism_abort_proto( ncomp_id, 'oas_clm_define', 'Failure in allocating zcla' )
            RETURN
         ENDIF

         ! -----------------------------------------------------------------
         ! ... Define the elements, i.e. specify the corner points for each
         !     volume element. 
         !     We only need to give the 4 horizontal corners
         !     for a volume element plus the vertical position of the upper
         !     and lower face. Nevertheless the volume element has 8 corners.
         ! -----------------------------------------------------------------

         ! -----------------------------------------------------------------
         ! ... Define centers and corners 
         ! -----------------------------------------------------------------
         !  1: lower left corner. 2,: lower right corner.
         !  3: upper right corner. 4,: upper left corner.
         !
         ! Latitudes
         ! Assumes lats are constant on an i line
         !
         !
         lat_incr = 0.
         ji = 1
         do while ( lat_incr < 1e-6 ) 
           lat_incr = ABS ( oasis_lat(1) - oasis_lat(ji) )
           ji = ji + 1
         enddo
  
         zcla(:,1) = oasis_lat(:) + lat_incr * 0.5_r8
         zcla(:,2) = zcla(:,1) 
         zcla(:,3) = oasis_lat(:) - lat_incr * 0.5_r8
         zcla(:,4) = zcla(:,3) 

         ! Longitudes
         !
         !
         dx = oasis_lon(2)-oasis_lon(1)

         zclo(:,1) = oasis_lon(:) - dx * 0.5_r8
         zclo(:,2) = oasis_lon(:) + dx * 0.5_r8 
         zclo(:,3) = zclo(:,2)
         zclo(:,4) = zclo(:,1)

         DO ji = 1, global_size  ! make sure that all centers are nearby corners regarding the longitude...
            IF( oasis_lon(ji) - zclo(ji,1) < -300._r8 )   oasis_lon(ji) = oasis_lon(ji) + 360._r8
            IF( oasis_lon(ji) - zclo(ji,1) >  300._r8 )   oasis_lon(ji) = oasis_lon(ji) - 360._r8
         END DO

         IF ( IOASISDEBUGLVL == 1 ) THEN
            WRITE(nulout,*) ' zclo WEST', zclo(1:109,1)
            WRITE(nulout,*) ' oasis_lon ', oasis_lon(1:109)
            WRITE(nulout,*) ' zclo EAST', zclo(1:109,2)
            WRITE(nulout,*) ' zcla NORTH', zcla(1:1000:109,3)
            WRITE(nulout,*) ' oasis_lat ', oasis_lat(1:1000:109)
            WRITE(nulout,*) ' zcla SOUTH', zcla(1:1000:109,1)
            call flush(nulout)
         ENDIF

         ! -----------------------------------------------------------------
         ! ... Define the mask
         ! -----------------------------------------------------------------
 
         oasis_mask = 1 - oasis_mask

         WRITE(nulout,*) ' Land point number / total number', COUNT(oasis_mask == 0), local_size
         call flush(nulout)

         ! -----------------------------------------------------------------
         ! ... Write info on OASIS auxillary files (if needed)
         ! ----------------------------------------------------------------

!EM OASIS3-MCT non ascendant compatibility
         CALL prism_write_grid (clgrd, global_size, 1, &
                                RESHAPE(oasis_lon,(/global_size,1/)), &
                                RESHAPE(oasis_lat,(/global_size,1/)))

         CALL prism_write_corner (clgrd, global_size, 1, 4, &
                                  RESHAPE(zclo,(/global_size,1,4/)), &
                                  RESHAPE(zcla,(/global_size,1,4/)))

! tbd CALL prism_write_angle (clgrd, global_size, 1, angle)

         CALL prism_write_mask (clgrd, global_size, 1, oasis_mask)

!tbd EM Check areas unit
         CALL prism_write_area (clgrd, global_size, 1, RESHAPE(oasis_areas,(/global_size,1/)))

         CALL prism_terminate_grids_writing()
        
         DEALLOCATE( zclo, zcla)

      ENDIF

   ENDIF

   ! Send global size for fake declaration
   ! necessary if at least one processor is not involved in the coupling
   IF ( my_task == master_task ) THEN
      CALL MPI_BCAST(global_size, 1, itype, 0, kl_comm, nerror)
      total_size=global_size
   ELSE
      CALL MPI_BCAST(total_size, 1, itype, 0, kl_comm, nerror)
   ENDIF

   IF ( pe_definition ) THEN

      ! Declare exchange arrays size 
      grid_size = local_size

      ! -----------------------------------------------------------------
      ! ... Define the partition 
      ! -----------------------------------------------------------------
     
      ALLOCATE(decomp_size(ntask))
      decomp_size=0



      ! Compute global offsets and local extents
      IF ( local_size == global_size ) THEN
         offset = 0
      ELSE
         ! Global offset should be available somewhere ... but calculated here
         CALL MPI_ALLGATHER(local_size, 1, itype, &
                            decomp_size, 1, itype, kl_comm, nerror)
         offset = SUM(decomp_size(1:my_task+1)) - decomp_size(my_task+1)
      ENDIF

      igparal(1) = 3              ! ORANGE style partition
      igparal(2) = 1              ! partitions number
      igparal(3) = offset         ! Global offset
      igparal(4) = local_size     ! Local extent

      CALL prism_def_partition_proto( igrid, igparal, nerror, global_size )
      IF( nerror /= PRISM_Success )   CALL prism_abort_proto (ncomp_id, 'oas_clm_define',   &
            &                                                        'Failure in prism_def_partition' )

   ENDIF

      ! -----------------------------------------------------------------
      ! ... Variable definition
      !  OASIS3-MCT: due to stupid reasons, must be done also by PE
      !  not involved in the coupling
      ! ----------------------------------------------------------------

      ! Default values
      ssnd(:)%laction=.FALSE.  ; srcv(:)%laction=.FALSE.

      ssnd(1)%clname='CLM_TAUX'      !  zonal wind stress
      ssnd(2)%clname='CLM_TAUY'      !  meridional wind stress
      ssnd(3)%clname='CLMLATEN'      !  total latent heat flux (W/m**2)
      ssnd(4)%clname='CLMSENSI'      !  total sensible heat flux (W/m**2)
      ssnd(5)%clname='CLMINFRA'      !  emitted infrared (longwave) radiation (W/m**2)
      ssnd(6)%clname='CLMALBED'      !  albedo

      srcv(1)%clname='CLMTEMPE'
      srcv(2)%clname='CLMUWIND'
      srcv(3)%clname='CLMVWIND'
      srcv(4)%clname='CLMSPWAT'   ! specific water vapor content
      srcv(5)%clname='CLMTHICK'   ! thickness of lowest level (m)
      srcv(6)%clname='CLMPRESS'   ! surface pressure (Pa)
      srcv(7)%clname='CLMDIRSW'   ! direct shortwave downward radiation (W/m2)
      srcv(8)%clname='CLMDIFSW'   ! diffuse shortwave downward radiation (W/m2)
      srcv(9)%clname='CLMLONGW'   ! longwave downward radiation (W/m2)
      srcv(10)%clname='CLMCVRAI'  ! convective rain precipitation      (kg/m2*s)
      srcv(11)%clname='CLMCVSNW'  ! convective snow precipitation      (kg/m2*s)
      srcv(12)%clname='CLMGSRAI'  ! gridscale rain precipitation
      srcv(13)%clname='CLMGSSNW'  ! gridscale snow precipitation
      srcv(14)%clname='CLMGRAUP'  ! gridscale graupel precipitation
      srcv(15)%clname='CLMCVPRE'  ! total convective precipitation
      srcv(16)%clname='CLMGSPRE'  ! total gridscale precipitation

      ! Variable selection (should better be done through namelist information)
      ssnd(1:6)%laction=.TRUE.  ; srcv(1:9)%laction=.TRUE.

      ! Could be done on namelist
      ! Coupling all kind of precipitations  (disabled)
      ! srcv(10:14)%laction=.TRUE.
      ! Coupling only total convective and gridscale precipitations
      srcv(15:16)%laction=.TRUE.


   IF ( pe_definition ) THEN

      var_nodims(1) = 1           ! Dimension number of exchanged arrays
      var_nodims(2) = 1           ! number of bundles (always 1 for OASIS3)

      ipshape(1) = 1             ! minimum index for each dimension of the coupling field array
      ipshape(2) = local_size     ! maximum index for each dimension of the coupling field array

      ! ... Announce send variables. 
      !
      ksnd=0
      DO ji = 1, nmaxfld
         IF ( ssnd(ji)%laction ) THEN 
            CALL prism_def_var_proto(ssnd(ji)%nid, ssnd(ji)%clname, igrid, &
                                     var_nodims, PRISM_Out, ipshape, PRISM_Real, nerror )
            IF ( nerror /= PRISM_Success )   CALL prism_abort_proto( ssnd(ji)%nid, 'oas_clm_define',   &
               &                                               'Failure in prism_def_var for '//TRIM(ssnd(ji)%clname))

            ! If coupling field not in namcouple, do not call oas_clm_put routines
            IF ( ssnd(ji)%nid == OASIS_Var_Uncpl ) ssnd(ji)%laction = .FALSE.

            ksnd = ksnd + 1
         ENDIF
      END DO
      !
      ! ... Announce received variables. 
      !
      DO ji = 1, nmaxfld
         IF ( srcv(ji)%laction ) THEN 
            CALL prism_def_var_proto(srcv(ji)%nid, srcv(ji)%clname, igrid, &
                                     var_nodims, PRISM_In, ipshape, PRISM_Real, nerror )
            IF ( nerror /= PRISM_Success )   CALL prism_abort_proto( srcv(ji)%nid, 'oas_clm_define',   &
               &                                               'Failure in prism_def_var for '//TRIM(srcv(ji)%clname))
         ENDIF
      END DO
      
      !
      ! ... Allocate memory for data exchange and initilize it
      !
      ALLOCATE( exfld(local_size, krcv), stat = nerror )
      IF ( nerror > 0 ) THEN
         CALL prism_abort_proto( ncomp_id, 'oas_cos_define', 'Failure in allocating exfld' )
         RETURN
      ENDIF
     
      exfld = -99999._r8

      !------------------------------------------------------------------
      ! End of definition phase
      !------------------------------------------------------------------
   
      ! must be done by coupled processors only (OASIS3)
      CALL prism_enddef_proto( nerror )
      IF ( nerror /= PRISM_Success )   CALL prism_abort_proto ( ncomp_id, 'oas_clm_define', 'Failure in prism_enddef')

   ! With OASIS3-MCT, PE not participating to coupling must call fake definition routines
   ELSE
      write(nulout,*) ' fake declaration for careless processors ',my_task
      call flush(nulout)
      igparal(:)=0
      ! ... but not so fake. Be careful: global_size is needed
      write(nulout,*) ' total_size ',my_task, total_size
      call flush(nulout)
      CALL prism_def_partition_proto( igrid, igparal, nerror, total_size )
      !
      var_nodims=1 ; ipshape=(/1,total_size/)
      DO ji = 1, nmaxfld
         IF ( ssnd(ji)%laction ) &
           CALL prism_def_var_proto(srcv(1)%nid, ssnd(ji)%clname, igrid, &
                                    var_nodims, PRISM_Out, ipshape, PRISM_Real, nerror )
         IF ( srcv(ji)%laction ) & 
           CALL prism_def_var_proto(srcv(1)%nid, srcv(ji)%clname, igrid, &
                                    var_nodims, PRISM_In,  ipshape, PRISM_Real, nerror )
      ENDDO
      CALL prism_enddef_proto( nerror )
   ENDIF

   END SUBROUTINE oas_clm_define
