set -u

cat << EOF

import numpy as np
from netCDF4 import Dataset
import os
import sys
from nn_interp import nn_interp

init_bathy = "./init_NEMO_bathy.nc"
outside_bathy = "./${isfcpl_outside_bathy}"

input_file = "./${curr_geom_file}"
output_file = "./${new_bathy_file}"

icesheet_gridfile = "./${isfcpl_ice_cdo_gridfile}"
orca_gridfile = "./${isfcpl_oce_cdo_gridfile}"
weight_file = "./${isfcpl_ice_oce_weight_file}"
dist_file = "./${isfcpl_dist_file}"

pid = os.getpid()
tmp_file = "./tmp_pid%d"%pid+".nc"
tmp2_file = "./tmp2_pid%d"%pid+".nc"

float_threshold = .5
dist_discr_buffer = 10
min_height = 40.

fillv = -32767.
rtol = 1.e-3

tmpdat = Dataset(init_bathy, mode='r')
init_bathy = tmpdat.variables['Bathymetry_isf'][:]
init_draft = tmpdat.variables['isf_draft'][:]
tmpdat.close()

tmpdat = Dataset(outside_bathy, mode='r')
outside_bathy = tmpdat.variables['Bathymetry_isf'][:]
outside_draft = tmpdat.variables['isf_draft'][:]
tmpdat.close()

ny_nemo, nx_nemo = np.shape(init_bathy)

tmpdat = Dataset(dist_file, mode='r')
dist_discr = tmpdat.variables['discr_dist_new'][:]
tmpdat.close()

weight_outside_bathy = np.where(dist_discr == 0, 1., \
                             np.where(dist_discr > dist_discr_buffer, 0., \
                                      1. - (dist_discr).astype(float) / float(dist_discr_buffer + 1)))



tmpdat = Dataset(input_file, mode='r')
lat_in = tmpdat.variables['lat'][:]
lon_in = tmpdat.variables['lon'][:]
bathy_in = tmpdat.variables['B'][:]
mask_in = tmpdat.variables['MASK'][:]

is_isf_org = np.abs( mask_in - 3 ) < .5
is_wet_org = np.abs( mask_in - 1 ) > .5
is_ice_org = np.abs( mask_in ) > .5

draft_in = np.where( is_isf_org,
                     tmpdat.variables['draft'][:], \
                     0. )

# surface_in = tmpdat.variables['surface'][:]
# thickness_in = tmpdat.variables['thickness'][:]
tmpdat.close()

ny_ice, nx_ice = np.shape(lat_in)

tmp_out = Dataset(tmp_file, mode='w')

tmp_out.createDimension('y', ny_ice)
tmp_out.createDimension('x', nx_ice)

tmp = tmp_out.createVariable(varname="latitude", datatype='f8', dimensions=['y','x'])
tmp.long_name = "latitude"
tmp.standard_name = "latitude"
tmp.units = "degrees North"
tmp[:] = lat_in

tmp = tmp_out.createVariable(varname="longitude", datatype='f8', dimensions=['y','x'])
tmp.long_name = "longitude"
tmp.standard_name = "longitude"
tmp.units = "degrees East"
tmp[:] = lon_in

tmp = tmp_out.createVariable(varname="is_isf", datatype='f4', dimensions=['y','x'])
tmp.coordinates = "latitude longitude"
tmp[:] = is_isf_org

tmp = tmp_out.createVariable(varname="is_wet", datatype='f4', dimensions=['y','x'])
tmp.coordinates = "latitude longitude"
tmp[:] = is_wet_org

tmp = tmp_out.createVariable(varname="is_ice", datatype='f4', dimensions=['y','x'])
tmp.coordinates = "latitude longitude"
tmp[:] = is_ice_org

tmp = tmp_out.createVariable(varname="bedrock", datatype='f8', dimensions=['y','x'], fill_value = fillv)
tmp.standard_name = "bathymetry"
tmp.units = "m"
tmp.positive = "down"
tmp.coordinates = 'latitude longitude'
tmp[:] = - bathy_in

tmp = tmp_out.createVariable(varname="isf_draft", datatype='f8', dimensions=['y','x'], fill_value=fillv)
tmp.standard_name = "ice_shelf_draft"
tmp.units = "m"
tmp.positive = "down"
tmp.coordinates = 'latitude longitude'
tmp[:] = draft_in


tmp_out.close()

# os.system('module load CDO')
os.system('cdo remap,'+orca_gridfile+','+weight_file+' -setgrid,'+icesheet_gridfile+' '+tmp_file+' '+tmp2_file)
# sys.exit()
# os.system('cdo setmisstonn -setgrid,'+orca_gridfile+' '+tmp2_file+' '+tmp_file)

tmpdat = Dataset(tmp2_file,mode='r')

is_wet_interp = tmpdat.variables['is_wet'][:]
is_ice_interp = tmpdat.variables['is_ice'][:]
is_isf_interp = tmpdat.variables['is_isf'][:]
bedrock_interp = tmpdat.variables['bedrock'][:]
draft_interp = tmpdat.variables['isf_draft'][:]


is_wet_interp = nn_interp(is_wet_interp)
is_ice_interp = nn_interp(is_ice_interp)
is_isf_interp = nn_interp(is_isf_interp)
bedrock_interp = nn_interp(bedrock_interp)
draft_interp = nn_interp(draft_interp)
    


tmpdat.close()

out_nc = Dataset(output_file,mode='w')

out_nc.createDimension('y', ny_nemo)
out_nc.createDimension('x', nx_nemo)

bathy_isf_out = out_nc.createVariable(varname="Bathymetry_isf", datatype='f8', dimensions=['y', 'x'])
bathy_isf_out[:] = weight_outside_bathy * outside_bathy + (1. - weight_outside_bathy) * np.clip( bedrock_interp, a_min = 0., a_max = None)

bathy_noisf_out = out_nc.createVariable(varname="Bathymetry", datatype='f8', dimensions=['y', 'x'])
bathy_noisf_out[:] = weight_outside_bathy * outside_bathy + (1. - weight_outside_bathy) * np.where( is_ice_interp > float_threshold, 0., bathy_isf_out[:] )

draft_out = out_nc.createVariable(varname="isf_draft", datatype='f8', dimensions=['y', 'x'], fill_value=fillv)
draft_tmp = np.ma.array(data = np.where( weight_outside_bathy > rtol, 0., \
                                            np.where(is_ice_interp < float_threshold, 0., \
                                                     np.where( is_wet_interp > float_threshold, \
                                                               # ice and wet: ice shelf
                                                               np.minimum(draft_interp, bathy_isf_out[:]), 
                                                               # ice and dry: grounded ice
                                                               bathy_isf_out[:]))), \
                           mask = False)

is_cyclic = True
n_ovlp = 2

is_border = np.zeros(shape=[ny_nemo, nx_nemo], dtype=bool)

is_border[:2,:] = True
is_border[-2:,:] = True
if(not is_cyclic):
    is_border[:,:2]  = True
    is_border[:,-2:] = True


height_tmp = bathy_isf_out[:] - draft_tmp[:]

is_open_surf = np.logical_and( np.logical_and(bathy_isf_out[:] > rtol, 
                                              draft_tmp[:]     < rtol), \
                               height_tmp >= min_height )

is_open_neigh = np.ndarray(shape=[ny_nemo, nx_nemo, 4], dtype=bool)

is_open_neigh[:,:,0] = np.roll(is_open_surf, shift=+1, axis=-2)
is_open_neigh[:,:,1] = np.roll(is_open_surf, shift=-1, axis=-2)
is_open_neigh[:,:,2] = np.roll(is_open_surf, shift=+1, axis=-1)
is_open_neigh[:,:,3] = np.roll(is_open_surf, shift=-1, axis=-1)

explored = np.zeros(shape=[ny_nemo, nx_nemo], dtype=bool)
candidates = np.transpose(np.nonzero(np.logical_and(is_open_surf, np.logical_not(explored))))

curr_patch_idx = 0

is_crevasse = np.zeros(shape=[ny_nemo, nx_nemo], dtype=bool)

while(np.size(candidates) > 0):

    curr_patch_idx+=1

    is_in_currpatch = np.zeros(shape=[ny_nemo, nx_nemo], dtype=bool)
    is_in_currpatch[candidates[0,0], candidates[0,1]] = True
    
    keep_going = True
    cnt_explore = 0

    while(keep_going):

        n_cp = np.count_nonzero(is_in_currpatch)

        # add x+ neighbors
        is_in_currpatch = np.logical_or(is_in_currpatch, \
                                        np.roll(np.logical_and(is_open_neigh[:,:,0], \
                                                               is_in_currpatch), \
                                                shift=-1, axis=0))

        # add x- neighbors
        is_in_currpatch = np.logical_or(is_in_currpatch, \
                                        np.roll(np.logical_and(is_open_neigh[:,:,1], \
                                                               is_in_currpatch), \
                                                shift=1, axis=0))

        # add y+ neighbors
        is_in_currpatch = np.logical_or(is_in_currpatch, \
                                        np.roll(np.logical_and(is_open_neigh[:,:,2], \
                                                               is_in_currpatch), \
                                                shift=-1, axis=1))

        # add y- neighbors
        is_in_currpatch = np.logical_or(is_in_currpatch, \
                                        np.roll(np.logical_and(is_open_neigh[:,:,3], \
                                                               is_in_currpatch), \
                                                shift=1, axis=1))

        
        n_old = n_cp
        n_cp = np.count_nonzero(is_in_currpatch)

        n_new = n_cp - n_old
        keep_going = (n_new > 0)
        cnt_explore+=1

    if( np.logical_not(np.any( np.logical_and(is_in_currpatch, is_border)))):
        is_crevasse = np.where( is_in_currpatch, True, is_crevasse )
        
    # we have explored the current patch
    explored = np.where(is_in_currpatch, True, explored)

    # the remaining polynya candidates are the ice-free cells which have not been explored
    candidates = np.transpose(np.nonzero(np.logical_and(is_open_surf, np.logical_not(explored))))

print('cnt crevasse', np.count_nonzero(is_crevasse))

# draft_masked = np.ma.array(data = draft_tmp, mask = is_crevasse)
draft_out[:] = np.where(is_crevasse, bathy_isf_out[:], draft_tmp)

column_out = out_nc.createVariable(varname="water_height", datatype='f8', dimensions=['y', 'x'])
column_out[:] = bathy_isf_out[:] - draft_out[:]



out_nc.close()

os.system('rm -rf '+tmp_file+' '+tmp2_file)

EOF
