%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   f.ETISh: Fast Elementary Thermomechanical Ice Sheet model       %
%                  of intermediate complexity                       %
%                                                                   %
%                   Version 1.6 February 2020                       %
%                                                                   %
%                        Frank Pattyn                               %
%                  Laboratoire de Glaciologie                       %
%                Universite libre de Bruxelles                      %
%                      fpattyn@ulb.ac.be                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%-----------------------------------------------------------------
% 2D Finite difference ice sheet/ice shelf model
% SSA-SIA hybrid model
% Velocity field on different staggered grids (Arakawa C)
% Diffusion coefficients/temperature on Arakawa B-grid
% 3d temperature field
% Pseudo-thermomechanical coupling
% Local and non-local isostatic adjustment (ELRA model) with spatially
%   varying flexural rigidity and asthenosphere viscosity
% Power-law sliding / Coulomb friction
% Schoof/Tsai grounding line mechanics with buttressing
% Inversion procedure to determine spatially-varying sliding coefficient
% PICO ocean model for sub-shelf melt calculation
% Calving and hydrofracturing
% Subglacial hydrology
% PDD model for surface melt
%
% Model input: MASK (grounded/floated), MASKo, H, Ho, B, Bo, As, G, Ts, Mb,
%   stdB, v (all input is optional)
%
%
%-----------------------------------
% External control parameters
%-----------------------------------
% restart: (1) restart the model based on last dump-file 'toto' (when
%   runmode=1 or runmode=3)
% inverse: (1) optimization of basal sliding coefficients of power law
%       sliding
% timeslice: (1) output extra variables (snapshot times)
% meltfac: (x) amount of basal melting underneath ice shelves for meltfunc==0
% slidfac: (x) multiplier on basal sliding coefficients (see SeaRISE
%       experiments)
% meltfunc: (1) Melting underneath according to parameterization of Beckmann
%   and Goosse (2003) following Pollard; (2) idem, but linear; (3) PICO model;
%   (0) constant value of melting equal to value of meltfac
% schoof: (1) grounding line mirgation according to Schoof (2007) or
%         (2) according to Tsai et al. (2015)
% shelf: (1) add ice shelves (necessary when buttressing factors need to be
%       calculated (if schoof>0) following SSA
% SSA: (0) SIA for grounded ice sheet
%      (1) SSA model for grounded ice sheet
%      (2) HySSA (SSA+SIA)
% input: name of .mat input file with following variables:
%       MASK (grounded/floating), H, B, As, G and optionally Ts, Mb
% output: name of .mat outputfile
% imax: (x) number of grid points in y-direction
% jmax: (x) number of grid points in x-direction
% kmax: (x) number of vertical layers for temperature calculation
% delta: (x) grid size (equal in both x,y directions
% nsteps: (x) number of iterations in time
% dt: (x) time step (year)
% snapshot: (x) number of snapshots in time dependency and graphical output).
%       For grounding-line monitoring in timeslice, snapshot<250
% MbType: (x) type of SMB distribution/perturbation (default=0,
%       as given in inputfile)
% TsType: (x) type of surface T distribution/perturbation (default=0,
%       as given in inputfile)
% Tcalc: (1) calculate temperature in ice sheet/shelf (2) idem but coupled
%   to flow field (thermomechanical coupling)
% Tinit: (1) Initialize temperature field with steady-state analytical
%       solution
%        (2) Run with analytical solution only
% BasalLayer: (0-1) Fraction of ice column used in calculation of mean
%       temperature of ice sheet (0=only basal temperature; 1=whole column)
% Ao: (x) isotherm constant value of ice sheet A factor in Glen's flow law
% BedAdj: (1) adjustment of bedrock (full); (2) local bed adjustment
% m: (1-3) power-law exponent of Weertman sliding law
% CoulombFric: (1) Coulomb friction (for both SSA and SIA)
%              (2) Combined Weertman-Coulomb friction
% butfac: vector (size of nsteps) with values between 0 and 1: range of
%       full (1) to no (0) buttressing of ice shelves
% DeltaT: vector (size of nsteps) with background temperature forcing
%         zero vector = no forcing
% DeltaSL: vector (size of nsteps) with background sea-level forcing
%         zero vector = no forcing
% shelftune: (0.1 - 1) tuning parameter for ice shelf (close to 0.5)
% gammaT: nominal value for Effective turbulent temperature
%       exchange velocity (used in PICO model and linear/quadratic melt)
%       Different values according to model
% M0: 5-15: nominal value for tuning of Plume and PICOP model
% meltfactor: 0.25-0.4: scaling factor between atmosphere and ocean warming
% subwaterflow: (1) subglacial hydrology model
% calving: (0-4) type of calving law
%------------------------------------------------------------------------

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                   VERSION history                     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% v1.01 (05/2017)
%   Correction that calving also works when schoof=0
%   Weighting factor on Schoof condition that does not allow the GL flux to
%       be used when bed is shallow or above sea level (ncorx and ncory)
%   Reduced sparse matrix in ice sheet solution for initialization
%       (only solving for MASK=1)
%   Intitialization: regularization only applied for marine basins
%       (according to weighting factor in Schoof flux - unrelated)
%   Latitude-Longitude now part of inputfile
%   Optimization of PDD model
%
% v1.1 (08/2017)
%   Correction of possible calving within ice shelves (holes)
%   Implementation of PICO ocean box model for melting under ice shelves
%   Inclusion of ocean temperature To and salinity So as boundary
%       conditions (input data set)
%   Use of effective lithosphere thickness to derive spatially varying
%       flexural rigidity of lithosphere in bedrock deformation module
%   Changes in configuration of options in MbFunc and TsFunc
%   Latitude-longitude removed from input file
%   Addition of 'restart' to allow the model to restart from a previous
%       dumpfile 'toto'
%   Change option 'hydra' in 'runmode' and extended options (linked to
%       restart)
%   Removal of grounding line wedge 'GLwedge'
%   Correction and refinement of PDD model (approach of Janssens and
%       Huybrechts, 2000); inlusion of meltwater percolation and refreezing
%   Inclusion of vertical cliff melting (FMB) following Pollard 2012
%   Less output graphics (only two graphs are shown during/at end of model
%       run)
%   New time-dependent parameters, such as ice volume, grounded ice area,
%       floating ice area, float ice volume, etc.
%   Further correction of the Coulomb friction law (to work for
%       near-plastic cases); in case of Coulomb friction, m is set to 1
%       automatically
%
% v1.2 (01/2018)
%   Bed adjustment invoked every intT iterations (as temperature)
%   Recoding of sparse matrices of SSA velocity field according to
%       Rommelaere and Ritz; improvement on iterative scheme for nonlinear
%       term. Greater stability of velocity field.
%   Renaming of some variables that may be confused with existing Matlab
%       routines.
%   Some parameters in FetishInputParams have now migrated to the
%       FetishModel call: shelftune, gammaT, C, meltfactor and
%       should be defined prior to the model run.
%   Tests on 'exist' are removed from the time loops in order to make the
%       code more efficient
%   PICO model called every intT iterations; interpolated melt calculated
%       inbetween calls
%
% v1.3 (10/2018)
%   Add copy of Bedmap bed and ice thickness (Bo and Ho) as reference for
%       multiple initialization steps. Corrected that MB-elevation feedback and
%       optimization uses this reference surface (Bo+Ho) instead of surface
%       from previous input (important for paleo runs)
%   Further corrections on 0-level corrected for actual sea level
%       (important for paleo runs)
%   Removed staggered grid masks, which optimizes calculation time
%   Correction on 'ocean'/'sea ice' viscosity eta; model now requires a thin
%       layer of sea ice (not ice thickness zero) to function
%   Inclusion of Pollard (2015) calving law and hydrofracturing using
%       surface melt defined from PDD model. In absence of hydrofracturing,
%       CMB only applied at calving front.
%   Ouput of uxs,vxs of SSA solution to be used as initial values for
%       consecutive run
%   Further optimization of grounding line flux and ice thickness solver to
%       speed up model runs using upstream differences for SSA and an
%       over-implicit Crank-Nicholson scheme
%   'forcing' has been replaced by 'timeslice'
%   'ouputF' and 'outputVAF' have been removed from the model call, as the
%       information is anyway stored in 'toto' files
%   Subglacial hydrology implemented based on a LeBrocq/Alley approach
%   Calculation of subglacial melt underneath grounded ice sheet
%   Spatial variations in flexural rigidity (based on effective lithosphere
%       thickness) and spatial variations in relaxation time based on upper
%       mantle viscosity. Kelvin functions are defined for different Db
%       classes for computational efficiency
%   Implementation of Bueler and Brown (2009) scaling of SIA and SSA
%   Effective viscosity 'mu' has been renamed 'eta'
%   Introduction of impossible combination of shelf=1 and SSA=0 (shelf=1
%       can only work with SSA or hybrid model)
%
% v1.4 (06/2019)
%   Simplification of hydrological model for one class based on mean
%       grounded ice thickness
%   Correction on output of uxs,uys (Lars)
%   Corrections on interpolations of sub-shelf melt and hydrology (Lars)
%   Implementation of PICOP model (combined box and plume model for
%       sub-shelf melting) and plume model
%   Correction of initial VAF for initial sea level (instead of SL=0) (Vio)
%   Addition of SLC due to bedrock adjustment (Heiko method - Vio)
%   Major bug in restart corrected (Vio)
%   Inclusion of sea-level fingerprinting based on self-gravitation -
%       defines local sea level instead of global sea level
%   Optimization of tanphi instead of Opt in Coulomb friction law
%   Inclusion of subglacial water flow in Coulomb friction law + various
%       corrections applied
%   New parameter 'diagnostic' to run model without ice-thickness evolution
%
% v1.5 (09/2019)
%   Removal of Bueler velocity calculation (only Winkelmann addition in
%       hybrid model)
%   Homogenized use of parameters in different melt function and melt
%       models
%   Generalization of sliding and friction laws with introduction of
%       effective pressure in Weertman sliding law
%   Optimization of sliding coefficients independent of value of m
%   Subglacial till model (Bueler and Van Pelt, 2015)
%   New call of FetishModel with control file, input and output file
%
% v1.6 (02/2020)
%   New call of FetishModel (using struct)
%   New optimization procedure of sliding coefficients
%   New regularization scheme for optimization of sliding coefficients
%   No sub-shelf melt for 'lakes' (floating areas that are not connected
%       to the ocean) 
%   Constant ice thickness for small islands for inverse==1
%   New faster functions for PICO
% 
% v1.7 LZbPARAMOUR (07/2020)
%   New diagnostic variable VelCheck (1D tracking of velocity changes)
%   New calving option calving=5 for coupling with NEMO (necessary condition
%       for NEMO to not crash: open ocean MASK needs to be constant!):
%           - set Hn to 0 for open ocean (to make sure, shelves can't grow)
%           - minimum thickness of 10m for grid cells that can become open ocean 
%               (ice shelves and grounded ice with B<0) to prevent grid
%               cells from becoming open ocean
%   Implementation of MeltIn (new variable for 2D sub-shelf melt field as input)
%           - MeltIn will be used for all shelves, where MeltIn exists
%               (MeltIn=NaN for not-existing MeltIn)
%           - MeltIn will be extrapolated for shelves where MeltIn exists
%               only partially
%           - MeltIn will be only used for shelves where MeltIn covers more
%               than half of the area
%           - MeltIn has to be defined in fc struct! 
%               (if not needed: fc.MeltIn=NaN(imax,jmax);)
%           - Melt calculated by meltfunc for shelves where MeltIn is not
%               existing
%   New runmodes (2 & 5) (old runmode 2 is now runmode 3) for coupled runs 
%       - creates toto file and stops run at each snapshot
%   Addition of initialization time step time=0 for cnt=1 (because first 
%       timestep is always used to calculate current state, it's not a 
%       timestep forward)
%       --> to make fETISh consistent for very short forward runs (like in PARAMOUR) 
%       - ctr.nsteps now has to be (duration/dt)+1 (e.g. duration should be
%       20000 years, ctr.dt=10 --> ctr.nsteps=2001)
%------------------------------------------------------------------------

function FetishModel2020julyLZbPARAMOUR(ctr,fc,infile,outfile)

% Expiration date of model code
if now>datenum(2023,9,25)
    fprintf('f.ETISh licence expired\nContact fpattyn@ulb.ac.be for help\n');
    return;
else
    fprintf('---f.ETISh v1.6---\n  [%s fpattyn@ulb.ac.be]\n',char(169));
end

%---------------------
% Input parameters
%---------------------

% Impossible combination of parameters
if ctr.SSA==0 && ctr.shelf==1
    fprintf('If shelf=1, then SSA>0\n');
    return;
end

[par]=FetishInputParamsPARAMOUR(ctr.m);

if ctr.SSA==0 && ctr.shelf==0 % central differences for basal velocity in this case
    par.upstream=0;
end

if ctr.CoulombFric==1 || (ctr.m==1 && ctr.p==1 && ctr.subwaterflow==0)
    ctr.Hinv=2*ctr.Hinv;
    ctr.Tinv=2*ctr.Tinv;
end

%---------------------
% Initialization
%---------------------

plotstep=floor(ctr.nsteps/ctr.snapshot);
ctr.snapshot=floor(ctr.nsteps/plotstep)+1;    %LZb: floor because nsteps=(duration/dt)+1; +1 because 1st snapshot for cnt==1
Z=2./(par.n+2)*(par.g*par.rho)^par.n; % SIA isothermal pre-term

%-------------------------------------------
% Numerical parameters and grid definition
%-------------------------------------------
Li=(ctr.imax-1)*ctr.delta/1e3; % length of domain in y
Lj=(ctr.jmax-1)*ctr.delta/1e3; % length of domain in x
dtdx=ctr.dt/(2.*ctr.delta*ctr.delta);
dtdx2=ctr.dt/(2.*ctr.delta);
[X,Y] = meshgrid(0:ctr.delta/1e3:Lj,0:ctr.delta/1e3:Li);
x=0:ctr.delta/1e3:Lj;
y=0:ctr.delta/1e3:Li;

%------------------------------
% Definition of matrix sizes
%------------------------------
MASK=ones(ctr.imax,ctr.jmax);
MASKm=ones(ctr.imax,ctr.jmax);
H=zeros(ctr.imax,ctr.jmax);
Ho=zeros(ctr.imax,ctr.jmax);
B=zeros(ctr.imax,ctr.jmax);
Bo=zeros(ctr.imax,ctr.jmax);
MASKo=ones(ctr.imax,ctr.jmax);
Mb=zeros(ctr.imax,ctr.jmax);
Ts=zeros(ctr.imax,ctr.jmax);
G=zeros(ctr.imax,ctr.jmax)+0.042;
ux=zeros(ctr.imax,ctr.jmax);
uy=zeros(ctr.imax,ctr.jmax);
u=zeros(ctr.imax,ctr.jmax);
VAF=zeros(ctr.nsteps,1);
VA0=zeros(ctr.nsteps,1);
POV=zeros(ctr.nsteps,1);
DENScorr=zeros(ctr.nsteps,1);
SLC=zeros(ctr.nsteps,1);
IntMelt=zeros(ctr.nsteps,1);
Ag=zeros(ctr.nsteps,1);
Af=zeros(ctr.nsteps,1);
IVg=zeros(ctr.nsteps,1);
IVf=zeros(ctr.nsteps,1);
glflux=zeros(ctr.nsteps,1);
dHdt=zeros(ctr.nsteps,1);
time=(0:ctr.dt:(ctr.nsteps-1)*ctr.dt)';    %LZb: start with time=0 and use nsteps=(duration/dt)+1
InvVol=zeros(ctr.nsteps,4);     %LZb: additional diagnostic column
VelCheck=zeros(ctr.nsteps,4);   %LZb: new diagnostic variable
ncor=ones(ctr.imax,ctr.jmax);  %LZ: ncor defined at the beginning
MeltTot=zeros(ctr.imax,ctr.jmax);
Smelt=zeros(ctr.imax,ctr.jmax);
Wd=zeros(ctr.imax,ctr.jmax)+1e-8;
Wtil=Wd;
Bmelt=Wd;
flw=Wd;
if ctr.timeslice==1
    EVmask=zeros(ctr.imax,ctr.jmax,ctr.snapshot);
    EVH=zeros(ctr.imax,ctr.jmax,ctr.snapshot);
    EVB=zeros(ctr.imax,ctr.jmax,ctr.snapshot);
    EVMelt=zeros(ctr.imax,ctr.jmax,ctr.snapshot);
    EVUx=zeros(ctr.imax,ctr.jmax,ctr.snapshot);
    EVUy=zeros(ctr.imax,ctr.jmax,ctr.snapshot);
    EVUd=zeros(ctr.imax,ctr.jmax,ctr.snapshot);
    EVbeta2=zeros(ctr.imax,ctr.jmax,ctr.snapshot);
    EVflux=zeros(ctr.imax,ctr.jmax,ctr.snapshot);
    if ctr.Tcalc>=1
        EVT=zeros(ctr.imax,ctr.jmax,ctr.snapshot);
    end
    if ctr.GeoidCalc==1
        EVSLR=zeros(ctr.imax,ctr.jmax,ctr.snapshot);
    end
    maskcount=0;
end
% initialization of sparse matrix system for velocities
nodeu=linspace(1,ctr.imax*ctr.jmax*2-1,ctr.imax*ctr.jmax)';
nodeu=reshape(nodeu,[ctr.imax ctr.jmax]);
nodev=linspace(2,ctr.imax*ctr.jmax*2,ctr.imax*ctr.jmax)';
nodev=reshape(nodev,[ctr.imax ctr.jmax]);


%----------------------------------------------------------------------
% Read inputdata
% MASK=1: grounded ice sheet
% MASK=0: either ice shelf or open ocean
% MASK=3: ice shelf (Bedmap2 data; Antarctica)
% Read optionally B, H, MASK, G, tanphi, Mb, Ts, stdB, tmp, v, vx, vy
%   Input initialized by zeros(B,H,Mb,Ts,stdB) and ones(MASK)
%----------------------------------------------------------------------
As=ctr.Asin;
if exist([infile,'.mat'],'file')
    load(infile);
end
As=As*ctr.slidfac;
Mb0=Mb; % initial mass balance if read from inputfile
Ts0=Ts; % intial surface temperature if read from inputfile
MASK0=MASK; % MASK at start of run (MASKo is origiginal BedMap mask, if exists)
MASK(MASK==3)=0; % ice shelves in BedMap2 identified with MASK=3
HB=B;
H0=H; % initial ice thickness
B0=B; % initial bed elevation
if exist('tanphi','var')==0
    tanphi=zeros(ctr.imax,ctr.jmax)+tan(par.phi*pi/180); % nominal phi for ice sheet (30 deg)
end
if ctr.inverse~=0   %LZ: new variables needed for initialization 
    Asor=As;
    tanphior=tanphi;
    if exist('stdB','var')
        ncor(stdB>=prctile(stdB(:),99) & MASK==1 & B>1000)=0;    %LZ: definition of new ncor
    end
end

%-------------------------------------
% Define grounded/floating ice sheet
%-------------------------------------
if (ctr.schoof>=1 || ctr.shelf==1) && ctr.inverse==0
    HAF=B-fc.DeltaSL(1)+H*par.rho/par.rhow;
    MASK(HAF<0)=0;
    MASK(HAF>=0)=1;
    HB=max(fc.DeltaSL(1)-par.rho/par.rhow*H,B);
    HB0=HB;
else % keep grounding line fixed
    HB(MASK==0)=fc.DeltaSL(1)-par.rho/par.rhow*H(MASK==0);
    HB(HB<B)=B(HB<B);
    HB0=HB;
end
oldMASK=MASK; % used for interpolation between PICO calls
sn=HB+H;
S0=sn; % initial surface elevation
HBo=Bo; % Original Bedmap data to keep in each model run (Ho, Bo), if exist
HBo(MASK==0)=fc.DeltaSL(1)-par.rho/par.rhow*Ho(MASK==0);
HBo(HBo<Bo)=Bo(HBo<Bo);
sn0=HBo+Ho;
Hn=H;
Bn=B;

%----------------------------------------
% Ocean temperature and salinity
%----------------------------------------
if exist('To','var')==1
    To(isnan(To))=par.Toi;
    So(isnan(So))=par.Soi;
else
    To=zeros(ctr.imax,ctr.jmax)+par.Toi;
    So=zeros(ctr.imax,ctr.jmax)+par.Soi;
end

%----------------------------------------
% Temperature parameter initialization
%----------------------------------------
cntT=0;
if ctr.Tcalc>=1
    if exist('tmp','var')==1
        Tb=tmp(:,:,ctr.kmax)-par.T0;
    else
        tmp=repmat(Ts+par.T0,[1,1,ctr.kmax]);
        Tb=Ts;
    end
    zeta=calculate_zeta(ctr.kmax,0.015);
    wshape=(zeta.^(par.n+2)-1+(par.n+2)*(1-zeta))/(par.n+1); % shape function vertical velocity
    ushape=(par.n+2)*(1-zeta.^(par.n+1))/(par.n+1); % shape function horizontal velocity
    dzc=circshift(zeta,[0 -1])-circshift(zeta,[0 1]);
    dzp=circshift(zeta,[0 -1])-zeta;
    dzm=zeta-circshift(zeta,[0 1]);
    dzc=repmat(reshape(dzc,1,1,ctr.kmax),[ctr.imax,ctr.jmax,1]);
    dzp=repmat(reshape(dzp,1,1,ctr.kmax),[ctr.imax,ctr.jmax,1]);
    dzm=repmat(reshape(dzm,1,1,ctr.kmax),[ctr.imax,ctr.jmax,1]);
end

%------------------------------------
% Initial Volume Above Floatation
%------------------------------------
dSLR=zeros(ctr.imax,ctr.jmax); % spatially varying component of sea level (geoid effect)
SLR=dSLR+fc.DeltaSL(1); % total sea level (background + fingerprint)
SLR0=SLR;
% VAF0=sum((H0(MASK==1)+min(B(MASK==1),fc.DeltaSL(1))*par.rhow/par.rho)*ctr.delta^2);
VAF0=max(0,H0+min(B0-SLR,0)*(par.rhow/par.rho));
% POV0=max(0,fc.DeltaSL(1)-B0); % Potential ocean volume
POV0=max(0,par.SLref-B0); % Potential ocean volume
if ctr.GeoidCalc==1
    frg=round(par.geoidist/ctr.delta);
    Pg0=zeros(ctr.imax,ctr.jmax);
    Pg0(MASK==1)=par.rho*H0(MASK==1)*ctr.delta^2.;
    Pg0(MASK==0)=(-par.rhow)*(B(MASK==0)-SLR(MASK==0))*ctr.delta^2.;
    Pg0=Pg0+par.rhoL*B0*ctr.delta^2.; % addition of bed change (if applicable)
    [gkx,gky]=meshgrid(-par.geoidist:ctr.delta:par.geoidist,-par.geoidist:ctr.delta:par.geoidist);
    dist=sqrt(gkx.^2+gky.^2);
    geoide=par.Re/par.Me*(1./(2*sin(dist/(2*par.Re))));
    geoidmax=par.Re/par.Me*(1./(2*sin(10/(2*par.Re))));
    geoide(geoide>geoidmax)=0;
    geoide(geoide<1.5e-18)=0;
end

%-------------------------------
% Initialization of bed loads
%-------------------------------

if ctr.BedAdj==1
    if exist('Db','var')==1
        Dbexist=1;
        % Varying flexural rigidity (Kevin)
        load0=zeros(ctr.imax,ctr.jmax);
        load0(MASK==1)=par.rho*par.g*H(MASK==1);
        load0(MASK==0)=(-par.rhow*par.g)*(B(MASK==0)-SLR(MASK==0));
        B0=B;
        
        % old method
%         Db0=zeros(ctr.imax,ctr.jmax)+par.FlexRigid;
%         if exist('Db','var')==1
%             % Flexural rigidity as a function of lithospheric conditions. If Db
%             % file exists, then several classes and Kelvin functions are defined
%             Dbi=linspace(min(min(log10(Db)))-1e-2,max(max(log10(Db)))+1e-2,par.Bclass+1);
%             Dbi=10.^Dbi;
%             for i=1:par.Bclass
%                 Db0(Db>=Dbi(i))=(Dbi(i)+Dbi(i+1))/2.;
%                 Dbi(i)=(Dbi(i)+Dbi(i+1))/2.;
%             end
%         else
%             Bclass=1;
%             Dbi=linspace(par.FlexRigid,par.FlexRigid,par.Bclass);
%         end
%         frb=round(par.flexdist/ctr.delta);
%         P=zeros(ctr.imax,ctr.jmax);
%         P(MASK==1)=par.rho*par.g*H(MASK==1)*ctr.delta^2.;
%         P(MASK==0)=(-par.rhow)*par.g*(B(MASK==0)-fc.DeltaSL(1))*ctr.delta^2.;
%         P0=NaN(ctr.imax+2*frb,ctr.jmax+2*frb);
%         P0(frb+1:frb+ctr.imax,frb+1:frb+ctr.jmax)=P;
%         P0(isnan(P0))=P(ctr.imax,ctr.jmax);
%         load0=zeros(ctr.imax,ctr.jmax);
%         Ll=zeros(par.Bclass,1);
%         kei=zeros(2*frb+1,2*frb+1,par.Bclass);
%         for i=1:par.Bclass
%             Ll(i)=(Dbi(i)/par.rhom/par.g)^0.25;
%             [kerx,kery]=meshgrid(-frb*ctr.delta/Ll(i):ctr.delta/Ll(i):frb*ctr.delta/Ll(i), ...
%                 -frb*ctr.delta/Ll(i):ctr.delta/Ll(i):frb*ctr.delta/Ll(i));
%             ker=sqrt((kerx.^2+kery.^2));
%             ker(ker<1e-8)=1e-8;
%             kei(:,:,i)=imag(besselk(0,ker*(1+1i)/sqrt(2)));
%             loadB=-xcorr2(P0,kei(:,:,i))*Ll(i)^2./(2*pi*Dbi(i));
%             loadB=loadB(2*frb+1:ctr.imax+2*frb,2*frb+1:ctr.jmax+2*frb);
%             load0(Db0==Dbi(i))=loadB(Db0==Dbi(i));
%         end    
%         B0=B;
    else
        Dbexist=0;
        % Constant flexural rigidity
        frb=round(par.flexdist/ctr.delta);
        P=zeros(ctr.imax,ctr.jmax);
        P(MASK==1)=par.rho*par.g*H(MASK==1)*ctr.delta^2.;
        P(MASK==0)=(-par.rhow)*par.g*(B(MASK==0)-SLR(MASK==0))*ctr.delta^2.;
        P0=NaN(ctr.imax+2*frb,ctr.jmax+2*frb);
        P0(frb+1:frb+ctr.imax,frb+1:frb+ctr.jmax)=P;
        P0(isnan(P0))=P(ctr.imax,ctr.jmax);
        Ll=(par.FlexRigid/par.rhom/par.g)^0.25;
        [kerx,kery]=meshgrid(-frb*ctr.delta/Ll:ctr.delta/Ll:frb*ctr.delta/Ll, ...
            -frb*ctr.delta/Ll:ctr.delta/Ll:frb*ctr.delta/Ll);
        ker=sqrt((kerx.^2+kery.^2));
        ker(ker<1e-8)=1e-8;
        kei=imag(besselk(0,ker*(1+1i)/sqrt(2)));
        load0=-xcorr2(P0,kei)*Ll^2./(2*pi*par.FlexRigid);
        load0=load0(2*frb+1:ctr.imax+2*frb,2*frb+1:ctr.jmax+2*frb);
        B0=B;
    end
end
if ctr.BedAdj==2
    load0=zeros(ctr.imax,ctr.jmax);
    load0(MASK==1)=par.rho*H(MASK==1)/par.rhom;
    load0(MASK==0)=(-par.rhow)*(B(MASK==0)-SLR(MASK==0))/par.rhom;
    B0=B;
end

%-------------------------------------------------------------------
% Mask and node count for reduced sparse matrix for inversion (only
% grounded ice sheet)
%-------------------------------------------------------------------
[MASKinv,VM,nodes,node]=inverse_mask(MASK,ctr);

%----------------------------------------------------------------------------------
% LZ: create MASK for islands for which the geometry should be kept
% constant during initialization
%----------------------------------------------------------------------------------
if ctr.inverse==1
    MASKisl=bwlabel(MASK);  %LZ: new function for MASKisl
    isl=unique(MASKisl(MASKisl>0));  %LZ: create list of all islands
    isl(:,2)=arrayfun(@(x) sum(MASKisl(:)==x)*ctr.delta^2,isl);   %LZ: calculate size of each island [m^2]
    isl(isl(:,2)<3.2e9,:)=[];  %LZ: find islands that are bigger than threshold (more than 2 pixel for 40km resolution)
                                %LZ: threshold of 3.2e9 m^2 can be changed
    MASKisl(ismember(MASKisl,isl(:,1)))=0;
    MASKisl(MASKisl~=0)=1;
else
    MASKisl=zeros(ctr.imax,ctr.jmax);
end
    
%------------------------------------------
% Test on existance of certain parameters
%------------------------------------------
if ctr.shelf==1 || ctr.schoof>=1
    glMASKexist=1;
else
    glMASKexist=0;
end
if exist('stdB','var')==1
    stdBexist=1;
else
    stdBexist=0;
end
if exist('vx','var')==1
    vxexist=1;
else
    vxexist=0;
end
if exist('uxs','var')==1
    ushelfexist=1;
else
    ushelfexist=0;
    uxs=zeros(ctr.imax,ctr.jmax);
    uys=zeros(ctr.imax,ctr.jmax);
end
if exist('Btau','var')==1
    BTexist=1;
else
    BTexist=0;
end
if any(ismember(fields(ctr),'mismip'))==0
    ctr.mismip=0;
end
if any(ismember(fields(fc),'MeltIn'))==1 && sum(sum(~isnan(fc.MeltIn)))~=0 %LZb: use MeltIn if non nan values exist
    MeltInexist=1;
    %LZb: create scattered Interpolant of fc.MeltIn for extrapolation using nearest neighbor
    MeltIn=scatteredInterpolant(X(~isnan(fc.MeltIn)),Y(~isnan(fc.MeltIn)),fc.MeltIn(~isnan(fc.MeltIn)),'nearest');
else
    MeltInexist=0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                 %
%               ITERATION IN TIME                 %
%                                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tic;
cpu=cputime;
if ctr.runmode<3    %LZb: change because of new runmodes (2 & 5)
    scrsz = get(groot,'ScreenSize');
    figure('Position',[30 scrsz(4)/2.5 scrsz(3)/1.4 scrsz(4)/2.2],'Name',outfile);  %LZb: add name
end
fprintf('... Running ...\n');

%-------------------------------------------------------------
% Restart procedure from previous 'toto' file when timeslice=1
%-------------------------------------------------------------
if ctr.restart==1 && (ctr.runmode~=0 && ctr.runmode~=3) %LZb: change because of new runmodes (2 & 5)
    outputname=[outfile,'_toto'];
    load(outputname); % read dumpfile
    ctr.restart=0; % reset restart
    if cnt==ctr.nsteps  % LZb: check if run already is finished
        fprintf('Run is already finished. No restart possible.');
        return;
    end
    cnt0=cnt+1; % advance one time step since dump
else
    cnt0=1;
end

for cnt=cnt0:ctr.nsteps

%-------------------------------------------------------------
% Print model progression on screen
%-------------------------------------------------------------
    cntT=cntT+1;
    if cnt>1
        fprintf('\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b');
    end
    fprintf('t = %12.2f', time(cnt));
    if cnt==ctr.nsteps
        fprintf('\n... End model run ...\n');
    end    

%------------------------------------------------------
% Copying H and B on old values
%------------------------------------------------------
    if cnt>1
        H=Hn; % assigning new ice thickness to previous ones
        if ctr.BedAdj>0
            B=Bn; % assigning new bed elevation to previous ones
        end
    end


%------------------------------------------------------
% Adjusting MASK based on floating condition
%------------------------------------------------------
    if ctr.GeoidCalc==0
        SLR=zeros(ctr.imax,ctr.jmax)+fc.DeltaSL(cnt);
    else
        SLR=dSLR+fc.DeltaSL(cnt);
    end
    HAF=B-SLR+H*par.rho/par.rhow;
    if (ctr.schoof>=1 || ctr.shelf==1) && ctr.inverse==0
        MASK(HAF<0)=0;
        MASK(HAF>=0)=1;
        HB=max(SLR-par.rho/par.rhow*H,B);
    else
        HB=B;
        HB(MASK==0)=SLR(MASK==0)-par.rho/par.rhow*H(MASK==0);
        HB(HB<B)=B(HB<B);
    end
    sn=H+HB;
    
%------------------------------------------------------
% Surface mass balance and surface temperature
% parametrization and forcing
%------------------------------------------------------
    [Ts,Ts0,PDD]=TsFunc(ctr,par,Ts0,sn0,sn,fc.DeltaSL(cnt),fc.DeltaT(cnt));
    if ctr.MbType>0
        [Mb,Smelt]=MbFunc(ctr,Mb0,Ts0,Ts,sn,PDD,X,Y,Lj,Li,par,fc.DeltaT(cnt));
    else
        Mb=Mb0;
    end

%------------------------------------------------------
% Geometry on staggered grids and grounding-line MASK
%------------------------------------------------------
    [gradm,gradmx,gradmy,Hm,Hmx,Hmy,Bm,Bmx,Bmy,Tsm,Mbm,signx,signy]= ...
        staggered_grid(sn,H,B,Ts,Mb,ctr);
    if (ctr.schoof>=1 || ctr.shelf==1) && ctr.inverse==0
        HAFm=Bm-SLR+Hm*par.rho/par.rhow;
        MASKm(HAFm<0)=0;
        MASKm(HAFm>=0)=1;
    else
        if cnt==1
            HAFm=Bm-SLR+Hm*par.rho/par.rhow;
            MASKm(HAFm<0)=0;
            MASKm(HAFm>=0)=1;
        end
    end
    % Searching GL on H-grid (define MASK & glMASK)    
%     if ctr.schoof>=1 || ctr.shelf==1  %LZ: calculate glMASK always
        [glMASK,MASK]=grounding_line_mask(MASK,H,ctr);
        H(glMASK==6)=par.SeaIceThickness; % sea ice thickness to
                                      % control 'ocean' viscosity (0.1m)
        [glMASKm]=grounding_line_maskm(MASKm);
%     end
    

%-----------------------------------------
% Basal sliding ice-sheet velocities for
% isothermal case and temperature field
%-----------------------------------------
    
    if ctr.SlidAdjust==1 && ctr.Tcalc>0
        if stdBexist==1
            Tr=min(par.TrTemp,-0.02*stdB); % Pollard (2015)
        else
            Tr=par.TrTemp;
        end
        r=max(0,min(1,(Tb+par.pmp*Hm-Tr)./(-Tr)));
        r(MASKm==0)=1;
        r(u>par.u0)=1; % ice streams considered full sliding
        if ctr.schoof>=1 || ctr.shelf==1
            r(glMASKm==2)=1; % grounding line
        end
    else
        r=ones(ctr.imax,ctr.jmax);
    end
    taud=par.rho*par.g*Hm.*sqrt(gradm);
    if ctr.subwaterflow==0
        % Tsai et al (2015)
        pwv=-par.PoreFrac*par.rhow*par.g*Bm;
        pwv(MASKm==0)=par.PoreFrac*par.rho*par.g.*Hm(MASKm==0);
        pwv(Bm>=0)=0;
    else
        pwv=par.PoreFrac*par.rho*par.g*Hm.*(Wd/par.Wdmax); % Bueler and Brown (2009)  
    end
    Po=max(par.rho*par.g*Hm,1e5);
    Neff=max(Po-r.*pwv,par.sigmat*Po);
    if ctr.subwaterflow==2
        [Neff,Wtil]=TillWater(MASKm,Wtil,Bmelt,ctr,Po,par);
    end
    if ctr.subwaterflow==3
        Neff=ones(ctr.imax,ctr.jmax)*par.NeffScale^ctr.p;
        expflw=exp(flw/par.flw0);
    else
        expflw=1;
    end
    if ctr.CoulombFric==1
        tauc=tanphi.*Neff;
        if ctr.q==1
            Asf=ctr.slidfac*min(par.u0./tauc,5e-1);
        end
        ctr.m=1; % Make sure that m in sliding law equals 1 for SIA
             % and temperature calculations
    else
        Asf=par.AsScale*((1-r).*par.AsFroz+As.*r).*expflw./((Neff.^ctr.p)/(par.NeffScale^ctr.p));
    end
    % Asf on u/v grids
    Asfx=0.5*(Asf+circshift(Asf,[1 0]));
    Asfy=0.5*(Asf+circshift(Asf,[0 1]));
    us=Asf.*(taud.^ctr.m); % used for Tcalc (and SIA-model)
    us(MASKm==0)=0;

%----------------------------------------
% Thermomechanical coupling
%----------------------------------------
    A=zeros(ctr.imax,ctr.jmax)+ctr.Ao;
    if ctr.Tcalc==2
        if cnt==1 % initialization only for existing tmp-field
            Tmean=initTmean(tmp,ctr,par,Hm,zeta);
        end
        A(Tmean<=-10)=par.atune*par.a1*exp(-par.Q1./(par.R*(Tmean(Tmean<=-10)+par.T0)));
        A(Tmean>-10)=par.atune*par.a2*exp(-par.Q2./(par.R*(Tmean(Tmean>-10)+par.T0)));
        [Ax,Ay,Axy]=staggeredA(A);
    else
        Ax=A;
        Ay=A;
        Axy=A;
    end
    
%--------------------------------------------------------------------------
% SIA velocity and diffusivities
%--------------------------------------------------------------------------
    d=Z*A.*Hm.^(par.n+2.).*gradm.^((par.n-1.)/2.)+ ...
        Asf*(par.rho*par.g)^ctr.m.*Hm.^(ctr.m+1.).*gradm.^((ctr.m-1.)/2.);
    if ctr.schoof>=1
        ux=Z*Ax.*Hmx.^(par.n+1.).*gradmx.^(par.n/2.)+ ...
            Asfx.*(par.rho*par.g*Hmx).^ctr.m.*gradmx.^(ctr.m/2.);
        uy=Z*Ay.*Hmy.^(par.n+1.).*gradmy.^(par.n/2.)+ ...
            Asfy.*(par.rho*par.g*Hmy).^ctr.m.*gradmy.^(ctr.m/2.);
        ux=min(ux,par.maxspeed);    %LZ: apply maxspeed also for SIA
        uy=min(uy,par.maxspeed);    %LZ: apply maxspeed also for SIA
    end
    ud=Z*A.*Hm.^(par.n+1.).*gradm.^(par.n/2.); % only used for temperature

%---------------------------------------------
% Numerical solution of temperature field
%---------------------------------------------
    if ctr.Tcalc>=1 % on d-grid
        if cntT==1
            if (ctr.Tinit==1 && cnt==1) || ctr.Tinit==2
                [tmp,Tmean]=initTemp(G,taud,us,ud,par,Hm,Mbm, ...
                    zeta,ctr,Tsm,MASKm,To,fc.DeltaT(cnt));
            end
            if ctr.Tinit<2
                [tmp,Tmean]=temperature(tmp,Mbm,Tsm,par,Z,Asfx,Asfy,Hmx,Hmy,gradmx,gradmy, ...
                    ctr,signx,signy,wshape,ushape,ctr.dt*par.intT,ud,us,zeta, ...
                    gradm,Hm,dzc,dzm,dzp,G,taud,A,Ax,Ay,fc.DeltaT,MASKm,To,cnt);
            end
            Bgrad=(G+taud.*us/par.secperyear).*Hm/par.K;
            Tgrad=(tmp(:,:,ctr.kmax)-tmp(:,:,ctr.kmax-1))./dzm(:,:,ctr.kmax);
            Bmelt=min(1,max(1e-8,(Bgrad-Tgrad)*par.K*par.secperyear./(par.rho*par.Latent*Hm)));
            Bmelt(MASKm==0)=0;
        end
        Tb=tmp(:,:,ctr.kmax)-par.T0;
    else
        Bmelt=zeros(ctr.imax,ctr.jmax)+5e-3; % mean value based on Pattyn (2010)
    end    

%-------------------------------------
% Subglacial water flow
%-------------------------------------
    if ctr.subwaterflow==1 || ctr.subwaterflow==3
        if cntT==1
            [flw,Wd]=SubWaterFlux(ctr,par,H,HB,Hm,MASKm,MASK,Bmelt);
            Wd0=Wd;
        else
            if ctr.inverse==0
                Wd=extrapolateWaterFlux(MASK,oldMASK,Wd,Wd0,ctr,par);
            end
        end
        Mb=Mb-Bmelt;
    end

%----------------------------------------------------------
% MASK for ice shelf/SSA (zero if SSA is not calculated)
%----------------------------------------------------------
    HAFmx=Bmx-SLR+Hmx*par.rho/par.rhow; %FP moved this up as needed for schoof
    HAFmy=Bmy-SLR+Hmy*par.rho/par.rhow; %FP
    if ctr.shelf==1 || ctr.SSA>=1
        MASKmx=zeros(ctr.imax,ctr.jmax);
        MASKmy=zeros(ctr.imax,ctr.jmax);
        if ctr.shelf==1
            MASKmx(HAFmx<0)=1; % floating
            MASKmy(HAFmy<0)=1;
        end
        if ctr.SSA>=1
            MASKmx(HAFmx>=0)=1; % grounded
            MASKmy(HAFmy>=0)=1;
        end
    end

%---------------------------
% SSA ice shelf/ice stream
%---------------------------
    if ctr.shelf==1 || ctr.SSA>=1
        taudx=par.rho*par.g*Hmx.*sqrt(gradmx).*signx;
        taudy=par.rho*par.g*Hmy.*sqrt(gradmy).*signy;
        us=velocity_on_grid(uxs,uys);
        if par.ShelfPinning==1 && stdBexist==1
            fg=max(0,1-(HB-B)./stdB); % subgrid pinning points in ice shelf
            beta2=fg.*us.^(1/ctr.m-1).*Asf.^(-1/ctr.m); %FP
        else
            beta2=us.^(1/ctr.m-1).*Asf.^(-1/ctr.m); %FP
        end
        beta2=min(beta2,1e8);
        
        % regrid beta2 on H-grid
        beta2=0.25*(beta2+circshift(beta2,[1 0])+circshift(beta2,[0 1]) ...
            +circshift(beta2,[1 1]));
        beta2(MASK==0)=0;
        betax=0.5*(beta2+circshift(beta2,[0 -1]));
        betay=0.5*(beta2+circshift(beta2,[-1 0]));
        
        if ctr.mismip==1
            betax(:,1)=betax(:,2); % symmetric divide
            betax(1,:)=betax(ctr.imax-1,:);
            betax(ctr.imax,:)=betax(2,:);
            betax(:,ctr.jmax)=0;
            betay(:,1)=betay(:,3);
            betay(1,:)=betay(ctr.imax-2,:);
            betay(ctr.imax-1,:)=betay(2,:);
        end
                
        Hshelf=200; % Mean ice shelf thickness to control viscosity on domain edge
        Tf=.5*par.rho*par.g*Hshelf*(1.-par.rho/par.rhow); % on h-grid
        eta1=0.5*Hshelf.*Tf.^(1.-par.n)./Axy; % Viscosity on edge (or when viscosity cannot be calculated)
        if cnt==1
            su=zeros(ctr.imax*ctr.jmax*2,1);
        end
        for ll=1:par.visciter % iteration over effective viscosity
            [eta,dudx,dvdy]=EffVisc(Axy,uxs,uys,H,par,eta1,MASK,ctr);
            if ctr.shelf==1 || ctr.schoof>0
                eta(glMASK==6)=1e7;
            end
            [uxs1,uys1,su]=StreamStag_opt(nodeu,nodev,su,MASKmx,MASKmy, ...
                H,eta,betax,betay,uxs,uys,taudx,taudy,ctr,par); %FP add ud,vd   
            duxs=sqrt((uxs1-uxs).^2+(uys1-uys).^2);
            duxs(isnan(duxs))=0;
            uxs=uxs1;
            uys=uys1;
            limit=sum(duxs(:))/(ctr.imax*ctr.jmax);
            %---------iterative beta---------
            if cnt<=5   
                us=velocity_on_grid(uxs,uys);
                beta2=us.^(1/ctr.m-1).*Asf.^(-1/ctr.m); %FP
                beta2=min(beta2,1e8);
                % regrid beta2 on H-grid
                beta2=0.25*(beta2+circshift(beta2,[1 0])+circshift(beta2,[0 1]) ...
                    +circshift(beta2,[1 1]));
                beta2(MASK==0)=0;
                betax=0.5*(beta2+circshift(beta2,[0 -1]));
                betay=0.5*(beta2+circshift(beta2,[-1 0]));
            end
            %--------------------------------
            if limit<5e-1 % Limit on convergence
                break;
            end
        end
        signx(uxs<0)=-1;
        signy(uys<0)=-1;
        if ctr.shelf==1
            signx(uxs>=0)=1;
            signy(uys>=0)=1;
        else
            signx(uxs>0)=1;
            signy(uys>0)=1;
        end
        uxs(Hmx==0 & HAFmx>0)=0; % only for grounded ice sheet with H=0
        uys(Hmy==0 & HAFmy>0)=0;
        if ctr.SSA>=1 % SSA for basal sliding - add SIA velocity or not
            us=velocity_on_grid(uxs,uys); % basal velocity magnitude
            if ctr.SSA==1 % SSA
                ux=abs(uxs);
                uy=abs(uys);
                d=zeros(ctr.imax,ctr.jmax);
            else % Hybrid SSA/SIA
                ux=abs(uxs)+Z*Ax.*Hmx.^(par.n+1.).*gradmx.^(par.n/2.); %LZ: old solver
                uy=abs(uys)+Z*Ay.*Hmy.^(par.n+1.).*gradmy.^(par.n/2.); %LZ: old solver
                d=Z*A.*Hm.^(par.n+2).*gradm.^((par.n-1)/2); % only deformational part
%                 % add condition of no shearing in shelves
                ux(HAFmx<=0)=abs(uxs(HAFmx<=0)); %LZ: old solver
                uy(HAFmy<=0)=abs(uys(HAFmy<=0)); %LZ: old solver
            end
            us(MASKm==0)=0;
            u=velocity_on_grid(ux,uy);
        else
            ux=abs(uxs);
            uy=abs(uys);
        end
    end
    ux=min(ux,par.maxspeed);
    uy=min(uy,par.maxspeed);    

%-------------------------------------
% Schoof grounding-line migration
%-------------------------------------
    if ctr.schoof>=1
        % Buttress factors
        if ctr.shelf==1 || (par.RignotInverse==1 && ctr.inverse>=1 && vxexist==1)
            if (par.RignotInverse==1 && ctr.inverse>=1 && ctr.shelf==0 && ...
                    vxexist==1)
                % Buttress factors for initialization with SIA model
                Hshelf=min(max(H,100),1000);
                Tf=.5*par.rho*par.g*Hshelf*(1.-par.rho/par.rhow); % on h-grid
                eta1=0.5*Hshelf.*Tf.^(1.-par.n)./Axy;
                vx(isnan(vx))=ux(isnan(vx));
                vy(isnan(vy))=uy(isnan(vy));
                [eta,dudx,dvdy]=EffVisc(Axy,vx,vy,H,par,eta1,MASK,ctr);
            end
            Txx=2*eta.*(2.*dudx+dvdy)./max(par.SeaIceThickness,H);
            Tyy=2*eta.*(2.*dvdy+dudx)./max(par.SeaIceThickness,H);
            Tf=.5*par.rho*par.g*max(par.SeaIceThickness,H)*(1.-par.rho/par.rhow); % on h-grid

            if ctr.schoof==1 % Schoof
                Thetax=(abs((fc.butfac(cnt)*Txx+(1.-fc.butfac(cnt))*Tf)./Tf)).^(par.n*ctr.m/(ctr.m+1.));
                Thetay=(abs((fc.butfac(cnt)*Tyy+(1.-fc.butfac(cnt))*Tf)./Tf)).^(par.n*ctr.m/(ctr.m+1.));
            else % Tsai
                Thetax=(abs((fc.butfac(cnt)*Txx+(1.-fc.butfac(cnt))*Tf)./Tf)).^(par.n-1.);
                Thetay=(abs((fc.butfac(cnt)*Tyy+(1.-fc.butfac(cnt))*Tf)./Tf)).^(par.n-1.);
            end
            Thetax(MASK==1)=1.;
            Thetay(MASK==1)=1.;
            Thetax(Thetax>1.)=1.;
            Thetay(Thetay>1.)=1.;
            if fc.butfac(cnt)==0 % no buttressing is applied
                Thetax=ones(ctr.imax,ctr.jmax);
                Thetay=ones(ctr.imax,ctr.jmax);
            end
            if ctr.calving>=1 && ctr.shelf==1
                Thetax(glMASK==6)=1;
                Thetay(glMASK==6)=1;
            end
        else
            Thetax=ones(ctr.imax,ctr.jmax);
            Thetay=ones(ctr.imax,ctr.jmax);
        end

        % Applying Schoof BC and heuristic from Pollard
        if ctr.schoof<3
            if ctr.inverse>=1
                glMASKx=glMASK;
                glMASKy=glMASK;
            else
                [glMASKx,glMASKy]=schoof_mask(HAFmx,HAFmy,ctr);
            end
            % Weighting factor for Schoof correction as function bed
            ncorx=max(min(1-1/50*(Bmx+50),1),0);
            ncory=max(min(1-1/50*(Bmy+50),1),0);
            [uxsch,uysch]=grounding_line_flux(ctr,glMASKx,glMASKy,HAFmx,HAFmy,Bmx,Bmy,SLR,par, ...
                Axy,Thetax,Thetay,tanphi,ncorx,ncory,ux,uy,Hmx,Hmy,Asf);
        end
    else
        uxsch=ux;
        uysch=uy;
    end
    % Determine Schoof/Tsai component in velocity
    dux=min(uxsch,1e5)-ux;
    duy=min(uysch,1e5)-uy;
    if ctr.SSA>0
        % Make ux,uy basal velocity + Schoof/Tsai
        %LZ: use same sign for udx/udy as done during solving for vel.!
        udx=Z*Ax.*Hmx.^(par.n+1.).*gradmx.^(par.n/2.);
        udy=Z*Ay.*Hmy.^(par.n+1.).*gradmy.^(par.n/2.);
        udx(HAFmx<0)=0;
        udy(HAFmy<0)=0; 
        ux=min(max(abs(uxs)+dux,0),par.maxspeed); %LZb: old solver
        uy=min(max(abs(uys)+duy,0),par.maxspeed); %LZb: old solver
        % Add sign to velocity
        ux=ux.*signx;
        uy=uy.*signy;
    else % put Schoof/Tsai in d and set ux,uy zero for grounded part
        if ctr.schoof>=1
            dusch=velocity_on_grid(uxsch,uysch);
            d(gradm>1e-20)=dusch(gradm>1e-20).*Hm(gradm>1e-20)./sqrt(gradm(gradm>1e-20));
        end
        ux=zeros(ctr.imax,ctr.jmax);
        uy=zeros(ctr.imax,ctr.jmax);
    end

    % redefining diffusivities and velocities on B-grid after Schoof/Tsai
    if ctr.SSA==1
        u=velocity_on_grid(ux,uy);
    elseif ctr.SSA==2
        u=velocity_on_grid(ux,uy)+ud;
    else
        u(gradm>1e-20)=max(0,d(gradm>1e-20).* ...
            sqrt(gradm(gradm>1e-20))./Hm(gradm>1e-20));
    end
    u=min(u,par.maxspeed);  %LZ: maxspeed for u

%------------------------------------------------------
% Sub-ice shelf melting
%------------------------------------------------------
    if glMASKexist==1
        [ShelfN,numsh,shMASK,MASKlk]=Shelfsetup(MASK,glMASK,H,ctr,par);
        if par.ArcOcean==1
            if cntT==1
                [arcocn,~,~]=OceanArc(MASK,H,MASKlk,ctr,par);
                arcocn0=arcocn;
            else
                arcocn=ExtrapolateArc(MASK,oldMASK,arcocn,arcocn0,ctr);
            end
        end
    end
    if ctr.meltfunc>=1 && glMASKexist==1
        % LZ: new function to define shelf MASK and to number all shelves
        % (lakes are also masked and already omitted in shMASK)
        if ctr.meltfunc==1 || ctr.meltfunc==2
            % Beckman and Goosse (2003)
            T0m=max(To+ctr.meltfactor*fc.DeltaT(cnt),par.lambda1*So+par.lambda2+par.lambda3*HB);
            % Ocean temperature limited by freezing point
            Tfm=par.lambda1*So+par.lambda2+par.lambda3*HB; % Ocean freezing point
            T0m(MASK==1)=0;
            Tfm(MASK==1)=0;
            if ctr.meltfunc==1 % Pollard (quadratic)
                 Melt=ctr.gammaT*(par.rhow*par.cp0/(par.Latent*par.rho))^2 ...
                     *abs(T0m-Tfm).*(T0m-Tfm)*par.secperyear;
            else % linear function
                Melt=ctr.gammaT*par.rhow*par.cp0*(T0m-Tfm)*par.secperyear/(par.Latent*par.rho);
            end
            % Vio: arc ocean (PD16)
            if par.ArcOcean==1
                Melt(Melt>0)=Melt(Melt>0).*max(0,min(1,(arcocn(Melt>0)-20)./20));
            end
            if ctr.meltfac>0
                Melt=Melt*ctr.meltfac;
            end
        end
        if ctr.meltfunc==3 || ctr.meltfunc==4 % PICO/PICOP model
            nu=par.rho*par.Latent/(par.rhow*par.cp0);
            [Bk,Ak,Bmax]=PICOsetup(glMASK,H,ctr,par,ShelfN,numsh,shMASK);
            [T0o,S0o,zb]=OceanVarBox(numsh,To,So,ShelfN,HB);
            T0o=max(T0o+ctr.meltfactor*fc.DeltaT(cnt),par.lambda1*S0o+par.lambda2+ ...
                par.lambda3*zb); % Ocean temperature limited by freezing point
            [T,S]=BoxModel(S0o,T0o,numsh,par,Ak,Bk,ShelfN,HB,nu,ctr);
            if ctr.meltfunc==3 % PICO model
                Melt=ShelfMelt(HB,shMASK,ShelfN,Ak,Bk,S0o,T0o,S,T,numsh,Bmax,ctr,nu,par);
            else % Combined PICO with plume model (PICOP)
                [MASKpicop,MASKb]=PICOPmasks(MASK,MASKlk,H,ctr,par);
                [Zgl]=AdvecGL(HB,B,MASK,MASKpicop,MASKb,uxs,uys,ctr);
                Melt=PICOPmelt(HB,uxs,uys,Zgl,Bk,ShelfN,T,S,par,ctr,Bmax,numsh);
            end
            % Vio: arc ocean (PD16)
            if par.ArcOcean==1
                Melt(Melt>0)=Melt(Melt>0).*max(0,min(1,(arcocn(Melt>0)-20)./20));
            end
            if ctr.meltfac>0
                Melt=Melt*ctr.meltfac; % FP: add meltfac to increase melt
            end
        end
        if ctr.meltfunc==5
            T0o=max(To+ctr.meltfactor*fc.DeltaT(cnt),par.lambda1*So+par.lambda2+ ...
                par.lambda3*HB); % Ocean temperature limited by freezing point
            [MASKpicop,MASKb]=PICOPmasks(MASK,MASKlk,H,ctr,par);
            [Zgl]=AdvecGL(HB,B,MASK,MASKpicop,MASKb,uxs,uys,ctr);
            Melt=PlumeMelt(HB,uxs,uys,Zgl,T0o,So,par,ctr);
            % Vio: arc ocean (PD16)
            if par.ArcOcean==1
                Melt(Melt>0)=Melt(Melt>0).*max(0,min(1,(arcocn(Melt>0)-20)./20));
            end
            if ctr.meltfac>0
                Melt=Melt*ctr.meltfac; % FP: add meltfac to incrase melt
            end
        end
        if ctr.meltfunc==6 % Cornford 2016 melt (function of H)
            Melt=max(min((H-100)*4/7,400),0);
        end
        if ctr.meltfunc==7 % Uniform melting when butfac=1 - value of meltfac
            GeoBut=ones(ctr.imax,ctr.jmax)*fc.butfac(cnt);
            fc.butfac(cnt)=1;
            Melt=ctr.meltfac*GeoBut;
        end
        if ctr.meltfunc==8 % Ice shelf removal (needs butfac=[0])
            Melt=zeros(ctr.imax,ctr.jmax);
            if cnt==1
                H(MASK==0)=par.SeaIceThickness;
                Melt(MASK==0)=ctr.meltfac;
            else % can be modified for regrowth of ice shelves
                H(MASK==0)=par.SeaIceThickness;
                Melt(MASK==0)=ctr.meltfac;
            end
        end
        if ctr.meltfunc==9 % ISMIP6 non-local melt rate parameterization
            icemask_shelves=zeros(ctr.imax,ctr.jmax);
            icemask_shelves(glMASK>2 & glMASK<5)=1;
            Melt=ISMIP6_ocean(par,ctr,HB,icemask_shelves,fc.z,fc.TF(:,:,:)+...
                ctr.meltfactor.*fc.DeltaT(cnt),fc.basinNumber,fc.deltaT_basin);
        end
        Melt(shMASK==0)=0; %LZ: Melt only for real shelves, Melt=0 for 'Lakes'
        if glMASKexist==1
            Melt(glMASK==6)=0;
        end
    else
        Melt=zeros(ctr.imax,ctr.jmax);
        % meltfac = tuning factor in melt rates
        if ctr.shelf==1
            Melt(shMASK==1)=ctr.meltfac; %LZ: Melt only for real shelves, Melt=0 for 'Lakes'
        end
    end
    if MeltInexist==1 %LZb: use MeltIn if non nan values exist
        useMeltIn=shMASK;
        shList=(1:numsh)';
        MeltInCover=arrayfun(@(i) sum(sum(~isnan(fc.MeltIn)&ShelfN==i))/sum(sum(ShelfN==i)),shList);
        useMeltIn(ismember(ShelfN,shList(MeltInCover<0.5)))=0;
        Melt(useMeltIn==1)=fc.MeltIn(useMeltIn==1); %LZb: overwrite Melt by fc.MeltIn where fc.MeltIn should be used
        Melt(useMeltIn==1&isnan(Melt))=MeltIn(X(useMeltIn==1&isnan(Melt)),Y(useMeltIn==1&isnan(Melt))); %LZb: overwrite Melt by extrapolated MeltIn
        if ctr.meltfac>0
            Melt(useMeltIn==1)=Melt(useMeltIn==1)*ctr.meltfac; % FP: add meltfac to incrase melt
        end
    end
    % Integrated value of melting
    dMelt=min(Melt*ctr.dt,H);
    MeltTot=MeltTot+dMelt;
    IntMelt(cnt)=sum(sum(MeltTot))*ctr.delta^2*par.rho/1e12;
    Mb=Mb-Melt;
    
%-----------------------------------------------
% Calving and hydrofracturing (Vio and Sainan)
% After Pollard et al. (2015)
%-----------------------------------------------
    if ctr.calving>=1 && ctr.shelf==1
        [MASKHole,HoleCT]=ShelfHole(MASK,H,par); % Find holes in shelves + contour (where CMB applies)
        div=dudx+dvdy; % flow divergence
        [he,fi]=DefineEdgeThickness(ctr,glMASK,H); % Pollard 2015
%         he=DefineEdgeTickness0(ctr,glMASK,H); % Pollard 2012
        if ctr.calving==1 || ctr.calving==2
            dive=A.*(0.25*par.rho*par.g*he*(1.-par.rho/par.rhow)).^par.n; % error in P15 eq B.2b?
%             dive=A.*(0.25*par.rho*par.g*he).^par.n; % leaving the so-called error gives better calving
            div(fi<1)=dive(fi<1); % divergence for shelf edge grid points
            div(div>dive)=dive(div>dive); % divergence not larger than maximum value
            d_s=2./(par.rho*par.g)*(max(0,div)./A).^(1/par.n); % dry surface crevasse depth
            d_b=2.*par.rho/((par.rhow-par.rho)*(par.rho*par.g))*(max(0,div)./A).^(1/par.n); % basal crevasse depth
            d_a=he.*max(0,log(u/1600))/log(1.2); % additional crevasse deepening
            d_t=he.*max(0,min(1,(150-he)/50)); % thin floating ice
            RPDD=Smelt; % annual surface melt + rainfall available after refreezing
            RPDD(Smelt<1.5)=0; % R calculated as in DP16
            RPDD(Smelt>3)=Smelt(Smelt>3).^2;
            RPDD(Smelt<3 & Smelt>1.5)=4*1.5*(Smelt(Smelt<3 & Smelt>1.5)-1.5);
            d_w=100*RPDD;
            % Calving
            ratio=(d_s+d_b+d_a+d_t+d_w)./he;
            ratio_c=0.75; % Pollard(2015)
            CMB=3000*max(0,min(1,(ratio-ratio_c)./(1-ratio_c))).*he/ctr.delta;
%             CMB(glMASK<4)=0;
            CMB(glMASK<5)=0; % Vio - remark 14/01/2019 - previous is unstable
        end
        if ctr.calving==2 % combination of 1 and 4 (making sure ice shelves don't get bigger)
            CMB(MASK==0 & MASKo==0 & Ho<par.SeaIceThickness)=50; % higher value than initial (LZ)
        end
        if ctr.calving==3
            wc=min(1,he/200.); % Pollard (2012)
            CMB=(1.-wc)*30+wc*3e5.*max(div,0).*he/ctr.delta; % Pollard (2012)
            if par.ArcOcean==1
                CMB=CMB.*max(0,min(1,(arcocn-70)./20)); % Vio: arcocean applied as in PD12
            end
            CMB(glMASK<5)=0;
        end
        if ctr.calving==4 % calving front kept at observed position (if ice is floating)
            CMB=zeros(ctr.imax,ctr.jmax);
            CMB(MASK==0 & MASKo==0 & Ho<par.SeaIceThickness)=50; % higher value than initial (LZ)
        end
        if ctr.calving==5 %LZb: new calving scheme for PARAMOUR (no calving, but constant open ocean mask)
            CMB=zeros(ctr.imax,ctr.jmax);
        end
        CMB(MASKHole==1 | HoleCT==1)=0; % No calving in enclosed shelves holes
        CMB(glMASK>=5 & B<-2700)=50; % Calving rule to prevent unrealistic areas of
                                     % thin ice extending seaward beyond continental
                                     % shelf (Vio)
        if ctr.inverse==0
            Mb=Mb-CMB;
        end
    end

%--------------------------------------------
% Melting at vertical face of calving front
%--------------------------------------------

    if ctr.calving>=1 && ctr.shelf==1
        Dz=SLR-B;
        Dz(MASK==0)=par.rho*he(MASK==0)/par.rhow;
        FMB=max(Dz.*Melt/ctr.delta,0);
        FMB(glMASK~=5)=0;
        if ctr.inverse==0
            Mb=Mb-FMB;
        end
    end

%---------------------------------------------------------
% Construct the finite-difference stiffness matrix 
% and solve continuity equation
%---------------------------------------------------------

    if ctr.diagnostic==0
        if (ctr.schoof>=1 || ctr.shelf==1) && ctr.inverse~=1
            Hn=sparse_solver_new(node,nodes,Mb,H,HB,glMASK,dtdx,dtdx2, ...
                d,ux,uy,ctr,cnt,MASKinv,VM,par);
        else
            Hn=sparse_solver_new(node,nodes,Mb,H,HB,MASK,dtdx,dtdx2, ...
                d,ux,uy,ctr,cnt,MASKinv,VM,par);
        end
        Hn(Hn<0)=0; % limit on minimal ice thickness
        if ctr.inverse>=1
            Hn(MASKisl==1)=H(MASKisl==1); %LZ: set Hn=H for islands
                                          %FP: only for inversion
        end
        dHdt(cnt)=max(max(abs(Hn-H)))*ctr.dt/ctr.delta; % ice-sheet imbalance
        if ctr.SSA==0
            dHdt(cnt)=dHdt(cnt)/ctr.delta;
        end
        if ctr.calving==5 % LZb: PARAMOUR setup: ice shelf or grounded ice gridcells can not become open ocean 
            Hn(MASKo~=0 & B<0 & Hn<11.2)=11.2; % LZb: minimum ice shelf thickness is set to 11.2 meters (corresponding to ice shelf draft)
            Hn(MASKo==0)=0; % LZb: make sure open ocean can't become ice shelf or grounded ice (necessary for PARAMOUR)
        end
    end
    
    % Maximum suction check (Ritz et al., 2015)
    if par.SuctionCheck==1 && ctr.schoof>=1
        gammaSC=(1-par.rho/par.rhow)*par.rho*par.g;
        dH=(Mb-2.*(gammaSC/4.)^par.n*Axy.*H.^(par.n+1.))*ctr.dt;
        fldH=(Hn-H)<dH & dH<0;
        Hn(fldH)=H(fldH)+dH(fldH);
    end


%----------------------
% Bedrock adjustment
%----------------------
    if cntT==1
        if ctr.BedAdj==1
            if Dbexist==1
                % variable flexural rigidity (Kevin)
                loadB=zeros(ctr.imax,ctr.jmax);
                loadB(MASK==1)=par.rho*par.g*H(MASK==1)-load0(MASK==1);
                loadB(MASK==0)=(-par.rhow)*par.g*(B(MASK==0)-SLR(MASK==0))-load0(MASK==0); % Takes into account geoid changes: !!! If GeoidCalc=1 & DeltaSL~=0, need to get rid of DeltaSL signal --> TO IMPLEMENT 
                bload=sparse_solver_bedrock(Db,par,ctr,VM,node,nodes,loadB);
                
                % old model
%                 P=zeros(ctr.imax,ctr.jmax);
%                 P(MASK==1)=par.rho*par.g*Hn(MASK==1)*ctr.delta^2.;
%                 P(MASK==0)=(-par.rhow)*par.g*(B(MASK==0)-fc.DeltaSL(cnt))*ctr.delta^2.;
%                 P0=NaN(ctr.imax+2*frb,ctr.jmax+2*frb);
%                 P0(frb+1:frb+ctr.imax,frb+1:frb+ctr.jmax)=P;
%                 P0(isnan(P0))=P(ctr.imax,ctr.jmax);
%                 bload=zeros(ctr.imax,ctr.jmax);
%                 for i=1:par.Bclass
%                     loadB=-xcorr2(P0,kei(:,:,i))*Ll(i)^2./(2*pi*Dbi(i));
%                     loadB=loadB(2*frb+1:ctr.imax+2*frb,2*frb+1:ctr.jmax+2*frb)-load0;
%                     bload(Db0==Dbi(i))=loadB(Db0==Dbi(i));
%                 end
            else
                % constant flexural rigidity
                P=zeros(ctr.imax,ctr.jmax);
                P(MASK==1)=par.rho*par.g*H(MASK==1)*ctr.delta^2.;
                P(MASK==0)=(-par.rhow)*par.g*(B(MASK==0)-SLR(MASK==0))*ctr.delta^2.;
                P0=NaN(ctr.imax+2*frb,ctr.jmax+2*frb);
                P0(frb+1:frb+ctr.imax,frb+1:frb+ctr.jmax)=P;
                P0(isnan(P0))=P(ctr.imax,ctr.jmax);
                bload=-xcorr2(P0,kei)*Ll^2./(2*pi*par.FlexRigid);
                bload=bload(2*frb+1:ctr.imax+2*frb,2*frb+1:ctr.jmax+2*frb)-load0;
            end
        end
        if ctr.BedAdj==2
            bload=zeros(ctr.imax,ctr.jmax);
            bload(MASK==1)=par.rho*Hn(MASK==1)/par.rhom-load0(MASK==1);
            bload(MASK==0)=(-par.rhow)*(B(MASK==0)-SLR(MASK==0))/par.rhom-load0(MASK==0);
        end
        if ctr.BedAdj>0
            if BTexist==1
                Bn=(B-B0+bload)*ctr.dt*par.intT./(-Btau)+B;
            else
                Bn=(B-B0+bload)*ctr.dt*par.intT/(-par.bedrelax)+B;
            end
        end
    end

%-----------------------------------------------------------------
% Inversion procedure to optimize basal sliding coefficients
%-----------------------------------------------------------------
    if ctr.inverse==1 && rem(cnt*ctr.dt,ctr.Tinv)==0 && cnt>1 && ...
            cnt<ctr.nsteps-round(1000/ctr.dt) %LZ: no optimization within the last 1000 years of the initialization
        %LZ: introduce deltaZold to compare deltaZ between consecutive iterations
        deltaZold=zeros(ctr.imax,ctr.jmax);     %LZ
        if cnt*ctr.dt>ctr.Tinv     %LZ
            deltaZold(MASK==1)=deltaZ(MASK==1);     %LZ
        end      %LZ  
        deltaZ=zeros(ctr.imax,ctr.jmax);
        deltaZ(MASK==1)=max(-1.5,min(1.5,(sn(MASK==1)-sn0(MASK==1))/ctr.Hinv));
        if ctr.CoulombFric>=1
            if ctr.SlidAdjust==1 && ctr.Tcalc>0
                tanphior(r>0 & abs(deltaZ)>=abs(deltaZold))= ...
                    tanphior(r>0 & abs(deltaZ)>=abs(deltaZold)).* ...
                    10.^(-deltaZ(r>0 & abs(deltaZ)>=abs(deltaZold))); %LZ: Bernales  
            else
                tanphior(abs(deltaZ)>=abs(deltaZold))=tanphior(abs(deltaZ)>=abs(deltaZold)).* ...
                    10.^(-deltaZ(abs(deltaZ)>=abs(deltaZold))); %LZ
            end
            tanphior(tanphior<1e-6)=1e-6; % LZ: /100
            tanphior(tanphior>1e4)=1e4;  % LZ: *100
            if par.SmoothInv==1 % regularization of optimization
                tanphi=Smooth2D(tanphior,par.SmoothSpan,ctr.delta,ncor); %LZ: Smooth2D changed!
            else
                tanphi=tanphior;    %LZ
            end
            tanphi(tanphi<1e-5)=1e-5; % LZ
            tanphi(tanphi>1e2)=1e2;  % LZ
        else
            if ctr.SlidAdjust==1 && ctr.Tcalc>0
                Asor(r>0 & abs(deltaZ)>=abs(deltaZold))=Asor(r>0 & abs(deltaZ)>=abs(deltaZold)).* ...
                    10.^deltaZ(r>0 & abs(deltaZ)>=abs(deltaZold)); %LZ
            else
                Asor(abs(deltaZ)>=abs(deltaZold))=Asor(abs(deltaZ)>=abs(deltaZold)).* ...
                    10.^deltaZ(abs(deltaZ)>=abs(deltaZold)); %LZ
            end
            Asor(Asor<par.invmin/10)=par.invmin/10;   % LZ: /100
            Asor(Asor>par.invmax*100)=par.invmax*100;   % LZ: *100
            if par.SmoothInv==1 % regularization of optimization
                As=Smooth2D(Asor,par.SmoothSpan,ctr.delta,ncor); % LZ: optimize not regularized Asor and use regularized As (LZ104) %LZ105 Smooth2D changed!
            else
                As=Asor;    %LZ
            end
            As(As<par.invmin)=par.invmin; % first limits (LZ)
            As(As>par.invmax)=par.invmax;
        end
    end
    InvVol(cnt,1)=sum(sum(abs(sn(MASK==1)-sn0(MASK==1))));
    InvVol(cnt,2)=sum(sum(sn(MASK==1)-sn0(MASK==1)));
    InvVol(cnt,3)=sum(sum(abs(H(MASK==1|MASK0==1)-H0(MASK==1|MASK0==1)))); %LZ: total absolut thickness changes during run
    VelCheck(cnt,1)=nanmean(abs(u(MASK==1)-v(MASK==1)));    %LZb: mean absolut velocity misfit for grounded parts
    VelCheck(cnt,2)=nanmean(u(MASK==1)-v(MASK==1));    %LZb: mean velocity misfit for grounded parts
    if ctr.shelf==1
        VelCheck(cnt,3)=nanmean(abs(u(shMASK==1)-v(shMASK==1)));    %LZb: mean absolut velocity misfit for shelves
        VelCheck(cnt,4)=nanmean(u(shMASK==1)-v(shMASK==1));    %LZb: mean velocity misfit for shelves
        InvVol(cnt,4)=nanmean(H(shMASK==1)-H0(shMASK==1));      %LZb: add mean shelf thickness misfit
    end
    
%-----------------------------------------------
% Volume above floatation and fingerprinting
%-----------------------------------------------
    VAFi=max(0,H+min(B-SLR,0)*(par.rhow/par.rho)); % VAFi influenced by geoid changes if GeoidCalc=1
    % VAF variation (SL equivalent in ocean water)
    VAF(cnt)=sum(sum((VAF0-VAFi)*ctr.delta^2.))*par.rho/(par.Aoc*par.rhow);
    VA0i=max(0,H+min(B-par.SLref,0)*(par.rhow/par.rho));
    % VAreferenceSL variation (SL equivalent in ocean water)
    VA0(cnt)=sum(sum((VAF0-VA0i)*ctr.delta^2.))*par.rho/(par.Aoc*par.rhow);
    POVi=max(0,par.SLref-B);
    % Potential Ocean volume variation (SL equivalent in ocean water)
    POV(cnt)=sum(sum((POV0-POVi)*ctr.delta^2.))/par.Aoc;
    % Density correction for transformation from ice to freshwater and not ocean water (SL equivalent)
    DENScorr(cnt)=sum(sum((H0-H)*ctr.delta^2.))*((par.rho/par.rhow)-(par.rho/par.rhof))/par.Aoc;
    % Sea-level contribution from modelled ice-sheet
    SLC(cnt)=VA0(cnt)+POV(cnt)-DENScorr(cnt);
    if ctr.GeoidCalc==1 && cntT==1
        % use new ice thickess and bed elevation
        HAF=Bn-SLR+Hn*par.rho/par.rhow;
        if (ctr.schoof>=1 || ctr.shelf==1) && ctr.inverse==0
            MASK(HAF<0)=0;
            MASK(HAF>=0)=1;
        end
        Pg1=zeros(ctr.imax,ctr.jmax);
        Pg1(MASK==1)=par.rho*Hn(MASK==1)*ctr.delta^2.;
        Pg1(MASK==0)=(-par.rhow)*(Bn(MASK==0)-SLR(MASK==0)+fc.DeltaSL(cnt))*ctr.delta^2.; % Takes into account previous geoid change but not DeltaSL (external SL forcing) as ocean mass
        Pg1=Pg1+par.rhoL*Bn*ctr.delta^2.; % Takes into account land deformation
        DPg=Pg1-Pg0;
        % Fingerprint
        MaP=zeros(ctr.imax+2*frg,ctr.jmax+2*frg);
        MaP(frg+1:frg+ctr.imax,frg+1:frg+ctr.jmax)=MASK;
        loadH=conv2fft(DPg,geoide); % FFT convolution
        % Determine local sea-level change
        Vd=sum(loadH(MaP==0))*ctr.delta^2; % Volume change in domain without mass addition
        loadG=loadH(frg+1:ctr.imax+frg,frg+1:ctr.jmax+frg);
        dSLR=loadG+SLC(cnt)-Vd/par.Aoc; % previously VAF instead of SLC
        SLR=SLR0+dSLR+fc.DeltaSL(cnt); % SLR = Initial SLR + Geoid perturbation + External SL forcing
    elseif ctr.GeoidCalc==2
        dSLR=zeros(ctr.imax,ctr.jmax)+SLC(cnt); % SLC added uniformly
        SLR=SLR0+dSLR+fc.DeltaSL(cnt);
    end
    flux=totalflux(H,ux,uy,ctr.delta);
    if ctr.timeslice==1
        if cnt==1
            mflux=flux.*ctr.dt;
        else
            mflux=mflux+flux.*ctr.dt;
        end
    end
    IVg(cnt)=sum(H(MASK==1))*ctr.delta^2;
    Ag(cnt)=sum(sum(MASK==1))*ctr.delta^2;
    if glMASKexist==1
        Af(cnt)=(sum(sum(MASK==0))-sum(sum(glMASK==6)))*ctr.delta^2;
        IVf(cnt)=(sum(H(MASK==0))-sum(H(glMASK==6)))*ctr.delta^2;
        glflux(cnt)=sum(flux(glMASK==2));
    end
    
%------------------------------------
% Volume above floatation and plots
%------------------------------------
    if ctr.timeslice==1
        if rem(cnt-1,plotstep)==0    %LZ: cnt-1
            maskcount=maskcount+1;
            EVmask(:,:,maskcount)=MASK;
            EVH(:,:,maskcount)=H;
            EVB(:,:,maskcount)=B;
            EVMelt(:,:,maskcount)=MeltTot;
            EVUx(:,:,maskcount)=ux;
            EVUy(:,:,maskcount)=uy;
            EVUd(:,:,maskcount)=ud;
            EVflux(:,:,maskcount)=mflux;
            if ctr.shelf==1 || ctr.SSA>=1
                EVbeta2(:,:,maskcount)=beta2;
            end
            if (ctr.Tcalc>=1)
                EVT(:,:,maskcount)=min(Tb+par.pmp*Hm,0);
            end
            if ctr.GeoidCalc==1
                EVSLR(:,:,maskcount)=SLR;
            end
        end
    end
    if ctr.runmode<3    %LZb: change because of new runmodes (2 & 5)
        if rem(cnt-1,plotstep)==0   %LZ: cnt-1
            if par.plotH==1
                sn1=sn;
            else
                sn1=sn-S0;
                plim=max(max(abs(sn-S0)));
                plim(plim==0)=1;
            end
            u1=u;
            if glMASKexist==1
                sn1(glMASK==6)=NaN;
                u1(glMASK==6)=NaN;
            else
                sn1(MASK==0)=NaN;
                u1(MASK==0)=NaN;
            end
            subplot(1,2,1);
            if par.plotH==1
                imagescn(x,y,sn1);
            else
                imagescn(x,y,sn1,[-plim plim]);
            end
            axis xy;
            if ctr.mismip==0
                axis equal;
            end
            axis tight;
            colormap(brewermap([],par.plotcolor));
            colorbar;

            subplot(1,2,2);
            imagescn(x,y,log10(u1),[-1 3.5]);
            caxis();
            axis xy;
            if ctr.mismip==0
                axis equal;
            end
            axis tight;
            colormap(brewermap([],par.plotcolor));
            colorbar;
            pause(0.0001);
        end
    end
    cntT(cntT>=par.intT)=0;
    oldMASK=MASK;
    if ctr.runmode==1 || ctr.runmode==4 || ctr.runmode==2 || ctr.runmode==5 % LZb: add coupling runmodes
        if rem(cnt-1,plotstep)==0   %LZ: cnt-1
            outputname=[outfile,'_toto'];
            save(outputname);
            if (ctr.runmode==2 || ctr.runmode==5) && cnt~=ctr.nsteps % LZb: add stop of the run for coupling runmode
                fprintf('\n... End model run ...\n');
                toc;
                fprintf('CPU time: %.2f seconds\n', cputime-cpu);
                return;
            end
        end
    end
end

toc;
fprintf('CPU time: %.2f seconds\n', cputime-cpu);

%------------------------------------
% Save results
%------------------------------------

Mbend=Mb; % final values of Mb
Tsend=Ts; % finl values of Ts;
Mb=Mb0; % reset output Mb to original value, since Mb=Mb-Melt-CMB
Ts=Ts0; % reset output Ts

save(outfile,'H','B','Ho','Bo','MASK','MASKo','As','tanphi','G', ...
    'Ts','Mb','Wd','Wtil');
if ctr.Tcalc>=1
    save(outfile,'tmp','Bmelt','-append');
end
if exist('stdB','var')==1
    save(outfile,'stdB','-append');
end
if exist('vx','var')==1
    save(outfile,'vx','vy','v','-append');
end
if exist('Db','var')==1
    save(outfile,'Db','-append');
end
if exist('To','var')==1
    save(outfile,'To','So','-append');
end
if ctr.SSA>=1 || ctr.shelf==1
    save(outfile,'uxs','uys','-append');
end
if exist('Btau','var')==1
    save(outfile,'Btau','-append');
end
if exist('Bor','var')==1
    save(outfile,'Bor','-append');
end
if exist('lat','var')==1  %LZb: add lat, lon for PARAMOUR NEMO coupling
    save(outfile,'lat','lon','-append');
end
outputname=[outfile,'_toto'];
save(outputname);

%--------------------------
% Plots
%--------------------------
if ctr.runmode<3    %LZb: change because of new runmodes (2 & 5)
    if ctr.inverse>=1
        figure('Name',['InvVol_' outfile]);  %LZb: add name
        plot(time,InvVol(:,1)*ctr.delta^2.*1e-9);
        grid on; hold on;
        plot(time,InvVol(:,2)*ctr.delta^2.*1e-9,'r');
        xlabel('Time (year)');
        ylabel('|H-H_0| (km^3)');
        legend('Abs. Misfit','Misfit');
    end

    figure('Name',['SLR_' outfile]);  %LZb: add name
    plot(time,SLC); grid on; 
    xlabel('Time (year)');
    ylabel('SLR contribution (m)');
    
    if vxexist==1
        figure('Name',['Vel_' outfile]);  %LZb: add name
        loglog(v(MASK==1),u(MASK==1),'.');
        hold on;
        if glMASKexist==1
            loglog(v(glMASK==4),u(glMASK==4),'r.');
        end
        plot([1e-1 4e3],[1e-1 4e3],'k-','linewidth',2);
        axis([1e-1 4e3 1e-1 4e3]);
        grid on;
        xlabel('Observed velocity (m a^{-1})');
        ylabel('Modelled velocity (m a^{-1})');
    end
end

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBROUTINES
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [flux]=totalflux(H,ux,uy,delta)

    ux1=circshift(ux,[0 1]); % ux(i,j-1)
    uy1=circshift(uy,[1 0]); % uy(i-1,j)
    flux=H.*sqrt((0.5*(ux+ux1)).^2+(0.5*(uy+uy1)).^2)*delta;
end


function Tmean=initTmean(tmp,ctr,par,Hm,zeta)

    repH=repmat(Hm+1e-8,[1,1,ctr.kmax]);
    Tp=par.pmp*repH.*repmat(reshape(zeta,1,1,ctr.kmax),[ctr.imax,ctr.jmax,1]);
    Tmean=mean(tmp(:,:,ctr.kmax-ctr.BasalLayer*(ctr.kmax-1):ctr.kmax)+ ...
        Tp(:,:,ctr.kmax-ctr.BasalLayer*(ctr.kmax-1):ctr.kmax),3)-par.T0;
    Tmean(Tmean<-50.)=-50.;
    Tmean(Tmean>0)=0;
    
end


%--------------------------------------
% Temperature field initialization
%--------------------------------------
function [tmp,Tmean]=initTemp(G,taud,us,ud,par,Hm,Mbm, ...
    zeta,ctr,Tsm,MASKm,To,DeltaT)

    W=(zeta.^(par.n+3)-1)/((par.n+1)*(par.n+3))-(par.n+2)*(zeta.^2-1)/(2*(par.n+1))+zeta-1;
    Tgrad=-(G+taud.*(us+par.udfrac*ud)/par.secperyear)/par.K;
    diffus=par.kdif*par.secperyear./(Hm.*max(Mbm,1e-20)); % inverse of Peclet number        
    intW=zeros(ctr.imax,ctr.jmax,ctr.kmax);
    for k=ctr.kmax-1:-1:1 % quadrature (Hindmarsh, 1999)
        intW(:,:,k)=intW(:,:,k+1)+exp(0.5*(W(k)+W(k+1))./diffus)* ...
            (zeta(k)-zeta(k+1));
    end
    repH=repmat(Hm+1e-8,[1,1,ctr.kmax]);
    repz=repmat(reshape(zeta,1,1,ctr.kmax),[ctr.imax,ctr.jmax,1]);
    repmask=repmat(MASKm,[1,1,ctr.kmax]);
    tmp=repmat(Tsm+par.T0,[1,1,ctr.kmax])-repmat(Tgrad,[1,1,ctr.kmax]).* ...
        repH.*(intW-repmat(intW(:,:,1),[1,1,ctr.kmax]));
    Tp=par.pmp*repH.*repz;
    
    % Ice shelf and open ocean
    repTs=repmat(Tsm,[1,1,ctr.kmax]);
    repM=repmat(Mbm,[1,1,ctr.kmax]);
    TBshelf=repmat(min(To+ctr.meltfactor*DeltaT- ...
        0.12e-3*par.rho*Hm/par.rhow,0),[1,1,ctr.kmax]);
    c2=exp(repM.*repH/(par.kdif*par.secperyear));
    c1=exp(repM.*repz.*repH/(par.kdif*par.secperyear));
    shelftmp=((repTs-TBshelf).*c1+TBshelf-repTs.*c2)./(1-c2)+par.T0;
    shelftmp(repH<=1)=TBshelf(repH<=1)+par.T0;
    tmp(repmask==0)=shelftmp(repmask==0);

    % Correction for pmp
    tmp(tmp>par.T0-Tp)=par.T0-Tp(tmp>par.T0-Tp);
    Tmean=mean(tmp(:,:,ctr.kmax-ctr.BasalLayer*(ctr.kmax-1):ctr.kmax)+ ...
        Tp(:,:,ctr.kmax-ctr.BasalLayer*(ctr.kmax-1):ctr.kmax),3)-par.T0;
    Tmean(Tmean<-50.)=-50.;
    Tmean(Tmean>0)=0;
end


%--------------------------------------
% Full englacial temperature calculation
%--------------------------------------
function [tmp,Tmean]=temperature(tmp,Mbm,Tsm,par,Z,Asfx,Asfy,Hmx,Hmy,gradmx,gradmy, ...
    ctr,signx,signy,wshape,ushape,dt,ud,us,zeta,gradm,Hm,dzc,dzm,dzp,G,taud, ...
    A,Ax,Ay,DeltaT,MASKm,To,cnt)
                
    tmp(:,:,1)=Tsm+par.T0;
    
    % horizontal advection (with limits on horizontal velocity)
    uxd=Z*Ax.*Hmx.^(par.n+1.).*gradmx.^(par.n/2.).*signx;
    uyd=Z*Ay.*Hmy.^(par.n+1.).*gradmy.^(par.n/2.).*signy;
    uxb=Asfx.*(par.rho*par.g*Hmx).^ctr.m.*gradmx.^(ctr.m/2.).*signx;
    uyb=Asfy.*(par.rho*par.g*Hmy).^ctr.m.*gradmy.^(ctr.m/2.).*signy;
    uxdt=0.5*(uxd+circshift(uxd,[-1 0]));
    uydt=0.5*(uyd+circshift(uyd,[0 -1]));
    uxbt=0.5*(uxb+circshift(uxb,[-1 0]));
    uybt=0.5*(uyb+circshift(uyb,[0 -1]));
    ut=repmat(uxdt,[1,1,ctr.kmax]).*repmat(reshape(ushape,1,1,ctr.kmax),[ctr.imax,ctr.jmax,1]) ...
        +repmat(uxbt,[1,1,ctr.kmax]);
    vt=repmat(uydt,[1,1,ctr.kmax]).*repmat(reshape(ushape,1,1,ctr.kmax),[ctr.imax,ctr.jmax,1]) ...
        +repmat(uybt,[1,1,ctr.kmax]);
    dTdxm=(circshift(tmp,[0 -1])-tmp)/ctr.delta;
    dTdxp=(tmp-circshift(tmp,[0 1]))/ctr.delta;
    advecx=zeros(ctr.imax,ctr.jmax,ctr.kmax);
    advecx(ut>0)=ut(ut>0).*dTdxp(ut>0)*dt;
    advecx(ut<0)=ut(ut<0).*dTdxm(ut<0)*dt;
    dTdym=(circshift(tmp,[-1 0])-tmp)/ctr.delta;
    dTdyp=(tmp-circshift(tmp,[1 0]))/ctr.delta;
    advecy=zeros(ctr.imax,ctr.jmax,ctr.kmax);
    advecy(vt>0)=vt(vt>0).*dTdyp(vt>0)*dt;
    advecy(vt<0)=vt(vt<0).*dTdym(vt<0)*dt;
    
    % adjusted vertical velocity from mass conservation (Hindmarsh, 1999)
    ws=-Mbm-(ud.*ushape(1)+us).*sqrt(gradm);
    w=repmat(ws,[1,1,ctr.kmax]).*repmat(reshape(wshape,1,1,ctr.kmax),[ctr.imax,ctr.jmax,1]);
    
    % Internal heating
    dudz=repmat(2*A.*taud.^par.n.*Hm,[1,1,ctr.kmax]).*repmat(reshape(zeta.^par.n,1,1,ctr.kmax),[ctr.imax,ctr.jmax,1]);
    fric=par.rho*par.g*par.kdif*dt*repmat(reshape(zeta,1,1,ctr.kmax),[ctr.imax,ctr.jmax,1]) ...
        .*dudz.*repmat(sqrt(gradm),[1,1,ctr.kmax])/par.K;
    
    extraterm=fric-advecx-advecy;
    extraterm(extraterm>10)=10;
    extraterm(extraterm<-10)=-10;

    % Temperature solution
    repH=repmat(Hm+1e-8,[1,1,ctr.kmax]);
    Tp=par.pmp*repH.*repmat(reshape(zeta,1,1,ctr.kmax),[ctr.imax,ctr.jmax,1]);
    atp=(2*par.kdif*par.secperyear./(repH.*dzm)-w)*dt./(repH.*dzc);
    btp=1+2*par.kdif*par.secperyear*dt./((repH.^2).*dzp.*dzm);
    ctp=(2*par.kdif*par.secperyear./(repH.*dzp)+w)*dt./(repH.*dzc);
    ftp=ones(ctr.imax,ctr.jmax,ctr.kmax);
    gtp=zeros(ctr.imax,ctr.jmax,ctr.kmax);
    % basal BC with strain heating (correction with ud)
    gtp(:,:,ctr.kmax)=(G+taud.*us/par.secperyear).*Hm.*dzm(:,:,ctr.kmax)/par.K;
    for k=ctr.kmax-1:-1:2
        ftp(:,:,k)=atp(:,:,k)./(btp(:,:,k)-ctp(:,:,k).*ftp(:,:,k+1));
        gtp(:,:,k)=(tmp(:,:,k)+extraterm(:,:,k)+ ...
            ctp(:,:,k).*gtp(:,:,k+1))./(btp(:,:,k)-ctp(:,:,k).*ftp(:,:,k+1));
    end
    for k=2:ctr.kmax
        tmp(:,:,k)=tmp(:,:,k-1).*ftp(:,:,k)+gtp(:,:,k);
    end
    
    % Ice shelf and open ocean
    repTs=repmat(Tsm,[1,1,ctr.kmax]);
    repM=repmat(Mbm,[1,1,ctr.kmax]);
    repz=repmat(reshape(zeta,1,1,ctr.kmax),[ctr.imax,ctr.jmax,1]);
    repmask=repmat(MASKm,[1,1,ctr.kmax]);
    TBshelf=repmat(min(To+ctr.meltfactor*DeltaT(cnt)- ...
        0.12e-3*par.rho*Hm/par.rhow,0),[1,1,ctr.kmax]);
    c2=exp(repM.*repH/(par.kdif*par.secperyear));
    c1=exp(repM.*repz.*repH/(par.kdif*par.secperyear));
    shelftmp=((repTs-TBshelf).*c1+TBshelf-repTs.*c2)./(1-c2)+par.T0;
    shelftmp(repH<=1)=TBshelf(repH<=1)+par.T0; % open ocean
    tmp(repmask==0)=shelftmp(repmask==0);
    
    % Correction for pmp
    tmp(tmp>par.T0-Tp)=par.T0-Tp(tmp>par.T0-Tp);

    mintemp=min(min(Tsm))+min(DeltaT)+par.T0-5;
    tmp(tmp<mintemp)=mintemp;
    Tmean=mean(tmp(:,:,ctr.kmax-ctr.BasalLayer*(ctr.kmax-1):ctr.kmax)+ ...
        Tp(:,:,ctr.kmax-ctr.BasalLayer*(ctr.kmax-1):ctr.kmax),3)-par.T0;
    Tmean(Tmean<mintemp-par.T0)=mintemp-par.T0;
    Tmean(Tmean>0)=0;
end


%--------------------------------------
% Surface temperature parametrization
%--------------------------------------
function [Ts,Ts0,PDD]=TsFunc(ctr,par,Ts0,S0,sn,DeltaSL,DeltaT)

% 0: no ice-elevation feedback
% 1: ice-elevation feedback
% 2: ice-elevation feedback + PDD model
% 3: no ice-elevation feedback + PDD model
    
    PDD=zeros(ctr.imax,ctr.jmax);
    if ctr.TsType==1 || ctr.TsType==2 % correction for elevation changes
        Ts=Ts0+par.Tlapse*(max(sn,DeltaSL)-S0)+DeltaT;
    end
    if ctr.TsType==0 || ctr.TsType==3 % no correction for elevation changes
        Ts=Ts0+DeltaT;
    end
    if ctr.TsType==2 || ctr.TsType==3 % PDD model (Janssens 2000)
        Ta=min(20+max(sn,0)/300.,30); % seasonal peak-to-peak amplitude in Ts
        t=repmat(reshape(linspace(1,365.25,par.PDDsteps),1,1,par.PDDsteps),[ctr.imax,ctr.jmax,1]);
        Tm=repmat(Ts,[1,1,par.PDDsteps])-0.5*repmat(Ta,[1,1,par.PDDsteps]).*cos(2*pi*t/365.25);
        betaT=0.3989*exp(-1.58*abs(Tm/par.Tsigma).^1.372)+max(0,Tm/par.Tsigma);
        PDD=par.Tsigma*sum(betaT,3)*365.25/par.PDDsteps;
    end
    if ctr.TsType==4 % EISMINT moving margin
        Ts=270.-0.01*sn-par.T0+DeltaT;
        Ts0=Ts;
    end
    if ctr.TsType==5 % McCall glacier
        Ts=-2.08-0.00335*sn;
        Ts(sn>ctr.ELA)=-6.3;
    end
end


%--------------------------------------
% Surface mass balance parametrization
%--------------------------------------
function [Mb,Smelt]=MbFunc(ctr,Mb0,Ts0,Ts,sn,PDD,X,Y,Lj,Li,par,DeltaT)

    Smelt=zeros(ctr.imax,ctr.jmax);
    if ctr.MbType==1 % Pollard 2009 (paleo runs)
        Mb=Mb0.*2.^((Ts-Ts0)/10.);
    end
    if ctr.MbType==2 % Golledge 2015 (future runs, see Frieler)
        Mb=Mb0.*(1+0.053*(Ts-Ts0));
    end
    if ctr.TsType==2 || ctr.TsType==3 % PDD model
        % FP: put rain in as well (Huybrechts and De Wolde)
        RFD=max(-Ts*par.refreeze*par.K/(par.kdif*par.rho*par.Latent),0); % Huybrechts and de Wolde (1999)
        ESM=min(max(Mb0,0),par.snowfac*PDD); % Effective snow melt
        PDDr=max(PDD-ESM/par.snowfac,0); % remaining PDDs after snow melt
        Smelt=par.icefac*PDDr+max(ESM-RFD,0); % combined ice melt and runoff
        Mb=Mb-Smelt;
    end
    if ctr.MbType==3 % EISMINT fixed margin with forcing
        Mb=Mb0+0.02*DeltaT;
    end
    if ctr.MbType==4 || ctr.MbType==5 % EISMINT moving margin
        s=1.e-2;
        Rel=450.;
        if ctr.MbType==5
            Rel=Rel+10*DeltaT;
        end
        dist=sqrt((X-Lj/2.).^2.+(Y-Li/2.).^2.);
        Mb=min(0.5,s*(Rel-dist));
    end
    if ctr.MbType==6 % McCall Glacier
        Mb=ctr.mbgrad*(sn-ctr.ELA); %0.0017
    end
end



%--------------------------------------
% Vertical discretization
%--------------------------------------
function zeta=calculate_zeta(kmax,min_layer)

if (kmax-1)*min_layer > 1. % if number of layers is too high for min_layer
    zeta=0:(1/(kmax-1)):1;
else
    xz=[1 kmax-1 kmax];
    yz=[0 1.-min_layer 1];
    p=polyfit(xz,yz,2);
    nxz=1:1:kmax;
    zeta=p(1)*nxz.^2+p(2)*nxz+p(3);
end
zeta(1)=0;
zeta(kmax)=1;
end


%--------------------------------------
% Smoothing function inversion
%--------------------------------------
function As = Smooth2D(As,SmoothSpan,delta,ncor)
% Smoothing a DEM (2D surface)
DEM = log10(As);

% sigma is the standard deviation of the Gaussian distribution
% span defines the size of the Gaussian filter; has to be odd
sigma=max(SmoothSpan*1e3/delta/2,2);
span=round(sigma*2)*2+1;

DEM = imgaussfilt(DEM,sigma,'FilterSize',span);
DEM = 10.^DEM;

% Smoothing as a function marine basins
As(ncor>0.5)=DEM(ncor>0.5);

end



%--------------------------------------
% Color scheme
%--------------------------------------
function [map,num,typ] = brewermap(N,scheme)
% The complete selection of ColorBrewer colorschemes (RGB colormaps).
%
% (c) 2015 Stephen Cobeldick
%
% ### Function ###
%
% Returns any RGB colormap from the ColorBrewer colorschemes, especially
% intended for mapping and plots with attractive, distinguishable colors.
%
% Syntax (basic):
%  map = brewermap(N,scheme); % Select colormap length, select any colorscheme.
%  brewermap('demo')          % View a figure showing all ColorBrewer colorschemes.
% [...,num,typ] = brewermap(...); % The current colorscheme's number of nodes and type.
%
% Syntax (preselect colorscheme):
%  old = brewermap(scheme);   % Preselect any colorscheme, return the previous scheme.
%  map = brewermap(N);        % Use preselected scheme, select colormap length.
%  map = brewermap;           % Use preselected scheme, same length as current figure's colormap.
%
% This product includes color specifications and designs developed by Cynthia Brewer.
% See the ColorBrewer website for further information about each colorscheme,
% colorblind suitability, licensing, and citations: http://colorbrewer.org/
%
% See also CUBEHELIX RGBPLOT3 RGBPLOT COLORMAP COLORBAR PLOT PLOT3 SURF CONTOURF IMAGE CONTOURCMAP AXES SET JET LBMAP
%
% ### Color Schemes ###
%
% To reverse the colormap sequence simply prefix the string token with '*'.
%
% Each colorscheme is defined by a set of hand-picked RGB values (nodes).
% If <N> is greater than the requested colorscheme's number of nodes then:
%  - Sequential and Diverging schemes are interpolated to give larger colormaps.
%  - Qualitative schemes throw an error.
% Else:
%  - Exact values from the ColorBrewer sequences are returned for all schemes.
%
% # Diverging #
%
% Scheme|'BrBG'|'PRGn'|'PiYG'|'PuOr'|'RdBu'|'RdGy'|'RdYlBu'|'RdYlGn'|'Spectral'|
% ------|------|------|------|------|------|------|--------|--------|----------|
% Nodes |  15  |  15  |  15  |  15  |  15  |  15  |   15   |   15   |     15   |
%
% # Qualitative #
%
% Scheme|'Accent'|'Dark2'|'Paired'|'Pastel1'|'Pastel2'|'Set1'|'Set2'|'Set3'|
% ------|--------|-------|--------|---------|---------|------|------|------|
% Nodes |    8   |   8   |   12   |    9    |    8    |   9  |   8  |  12  |
%
% # Sequential #
%
% Scheme|'Blues'|'BuGn'|'BuPu'|'GnBu'|'Greens'|'Greys'|'OrRd'|'Oranges'|'PuBu'|
% ------|-------|------|------|------|--------|-------|------|---------|------|
% Nodes |   13  |  13  |  13  |  13  |   13   |   13  |  13  |    13   |  13  |
%
% Scheme|'PuBuGn'|'PuRd'|'Purples'|'RdPu'|'Reds'|'YlGn'|'YlGnBu'|'YlOrBr'|'YlOrRd'|
% ------|--------|------|---------|------|------|------|--------|--------|--------|
% Nodes |   13   |  13  |    13   |  13  |  13  |  13  |   13   |   13   |   13   |
%
% ### Examples ###
%
% % Plot a scheme's RGB values:
% rgbplot(brewermap(13,'Blues'))  % standard
% rgbplot(brewermap(13,'*Blues')) % reversed
%
% % View information about a colorscheme:
% [~,num,typ] = brewermap(0,'Paired')
%  num = 12
%  typ = 'Qualitative'
%
% % Multiline plot using matrices:
% N = 6;
% axes('ColorOrder',brewermap(N,'Pastel2'),'NextPlot','replacechildren')
% X = linspace(0,pi*3,1000);
% Y = bsxfun(@(x,n)n*sin(x+2*n*pi/N), X.', 1:N);
% plot(X,Y, 'linewidth',4)
%
% % Multiline plot in a loop:
% N = 6;
% set(0,'DefaultAxesColorOrder',brewermap(N,'Accent'))
% X = linspace(0,pi*3,1000);
% Y = bsxfun(@(x,n)n*sin(x+2*n*pi/N), X.', 1:N);
% for n = 1:N
%     plot(X(:),Y(:,n), 'linewidth',4);
%     hold all
% end
%
% % New colors for the "colormap" example:
% load spine
% image(X)
% colormap(brewermap([],'*YlGnBu'))
%
% % New colors for the "surf" example:
% [X,Y,Z] = peaks(30);
% surfc(X,Y,Z)
% colormap(brewermap([],'RdYlGn'))
% axis([-3,3,-3,3,-10,5])
%
% % New colors for the "contourcmap" example:
% brewermap('*PuOr'); % preselect the colorscheme.
% load topo
% load coast
% figure
% worldmap(topo, topolegend)
% contourfm(topo, topolegend);
% contourcmap('brewermap', 'Colorbar','on', 'Location','horizontal',...
%     'TitleString','Contour Intervals in Meters');
% plotm(lat, long, 'k')
%
% ### Input & Output Arguments ###
%
% Inputs (*=default):
%  N   = NumericScalar, N>=0, an integer to define the colormap length.
%      = *[], use the length of the current figure's colormap (see "colormap").
%      = StringToken, to preselect this ColorBrewer scheme for later use.
%      = 'demo', create a figure showing all of the ColorBrewer schemes.
%  scheme = StringToken, a ColorBrewer scheme to select the colorscheme.
%         = *none, uses the default colorscheme (must be set previously!).
%
% Outputs:
%  map = NumericMatrix, size Nx3, a colormap of RGB values between 0 and 1.
%  num = NumericScalar, the number of nodes defining the ColorBrewer scheme.
%  typ = String, the colorscheme type: 'Diverging'/'Qualitative'/'Sequential'.
%
% [map,num,typ] = brewermap(*N,*scheme)

persistent dcs
%
if nargin==0
    % Use the current figure's colormap length, and the preselected colorscheme.
%     assert(~isempty(dcs),'You need to preselect a colorscheme before trying this!')
    [map,num,typ] = cbSample([],dcs);
elseif nargin==2
    % Use the provided colormap length and the provided colorscheme.
%     assert(isnumeric(N),'First argument must be a numeric scalar, or empty.')
%     assert(ischar(scheme)&&isrow(scheme),'Second argument must be a string.')
    [map,num,typ] = cbSample(N,scheme);
elseif isnumeric(N)
    % Use the provided colormap length, and the preselected colorscheme.
    assert(~isempty(dcs),'You need to preselect a colorscheme before trying this!')
    [map,num,typ] = cbSample(N,dcs);
else% String
    assert(ischar(N)&&isrow(N),'First argument must be a string or numeric.')
    switch N
        case 'demo'
            % Plot all colorschemes in a figure.
            cbDemoFig
        case 'list'
            % Return a list of all available colorschemes.
            map = cbListTok;
            [num,typ] = cellfun(@cbSelect,map,'UniformOutput',false);
            num = cat(1,num{:});
        otherwise
            % Store the preselected colorscheme token.
            map = dcs;
            [num,typ] = cbSelect(N(1+strncmp('*',N,1):end));
            dcs = N;
    end
end
%
end
%----------------------------------------------------------------------END:brewermap
function [map,num,typ] = cbSample(N,tok)
% Pick a colorscheme, downsample/interpolate to the requested colormap length.
%
if isempty(N)
    N = size(get(gcf,'colormap'),1);
else
    assert(isscalar(N)&&isreal(N),'First argument must be a real numeric scalar, or empty.')
end
%
if strncmp('*',tok,1)
    vec = N:-1:1;
    tok = tok(2:end);
else
    vec = 1:+1:N;
end
%
[num,typ,rgb] = cbSelect(tok);
%
if strncmpi(typ,'Q',1)
    % Exact (Qualitative).
    assert(N<=num,'Colorscheme "%s" maximum colormap length: %d. Requested: %d.',tok,num,N)
    map = rgb(vec,:);
elseif N<num
    % Downsample (Diverging / Sequential).
    map = rgb(cbIndex(N,num,typ,vec),:);
elseif N>num
    % Interpolate (Diverging / Sequential).
    xi = (vec-1).' / (N-1);
    map = interp1q((1:num).', rgb, num*xi + xi(end:-1:1));
    %map = interp1((1:num).', rgb, num*xi + xi(end:-1:1), 'pchip');
else
    % Exact (Diverging / Sequential).
    map = rgb(vec,:);
end
%
end
%----------------------------------------------------------------------END:cbSample
function cbDemoFig
% Creates a figure showing all of the ColorBrewer colorschemes.
%
persistent cbh axh
%
seq = cbListTok;
xmx = max(cellfun(@cbSelect,seq));
ymx = numel(seq);
%
if ishghandle(cbh)
    figure(cbh);
    delete(axh);
else
    cbh = figure('HandleVisibility','callback', 'IntegerHandle','off',...
        'NumberTitle','off', 'Name',[mfilename,' Demo'],'Color','white');
end
axh = axes('Parent',cbh, 'Color','none',...
    'XTick',0:xmx, 'YTick',0.5:ymx, 'YTickLabel',seq, 'YDir','reverse');
title(axh,['ColorBrewer Color Schemes (',mfilename,'.m)'], 'Interpreter','none')
xlabel(axh,'Scheme Nodes')
ylabel(axh,'Scheme Name')
%
for y = 1:ymx
    [num,typ,rgb] = cbSelect(seq{y});
    for x = 1:num
        patch([x-1,x-1,x,x],[y-1,y,y,y-1],1, 'FaceColor',rgb(x,:), 'Parent',axh)
    end
    text(xmx+0.1,y-0.5,typ, 'Parent',axh)
end
%
end
%----------------------------------------------------------------------END:cbDemoFig
function idx = cbIndex(N,num,typ,vec)
% Ensure exactly the same colors as in the online ColorBrewer schemes.
%
if strncmp('D',typ,1) && 3<=N && N<=11
    switch N
        case 3
            idx = [5,8,11];
        case 4
            idx = [3,6,10,13];
        case 5
            idx = [3,6,8,10,13];
        case 6
            idx = [2,5,7,9,11,14];
        case 7
            idx = [2,5,7,8,9,11,14];
        case 8
            idx = [2,4,6,7,9,10,12,14];
        case 9
            idx = [2,4,6,7,8,9,10,12,14];
        case 10
            idx = [1,2,4,6,7,9,10,12,14,15];
        case 11
            idx = [1,2,4,6,7,8,9,10,12,14,15];
    end
    idx = idx(vec);
elseif strncmp('S',typ,1) && 3<=N && N<=9
    switch N
        case 3
            idx = [3,6,9];
        case 4
            idx = [2,5,7,10];
        case 5
            idx = [2,5,7,9,11];
        case 6
            idx = [2,4,6,7,9,11];
        case 7
            idx = [2,4,6,7,8,10,12];
        case 8
            idx = [1,3,4,6,7,8,10,12];
        case 9
            idx = [1,3,4,6,7,8,10,11,13];
    end
    idx = idx(vec);
else
    % Evenly spaced downsampling:
    idx = round((1+num) * vec / (1+N));
end
%
end
%----------------------------------------------------------------------END:cbIndex
function seq = cbListTok
% Return a list of all colorschemes, sorted by type and scheme name.
%
seq = {'BrBG';'PRGn';'PiYG';'PuOr';'RdBu';'RdGy';'RdYlBu';'RdYlGn';'Spectral';'Accent';'Dark2';'Paired';'Pastel1';'Pastel2';'Set1';'Set2';'Set3';'Blues';'BuGn';'BuPu';'GnBu';'Greens';'Greys';'OrRd';'Oranges';'PuBu';'PuBuGn';'PuRd';'Purples';'RdPu';'Reds';'YlGn';'YlGnBu';'YlOrBr';'YlOrRd'};
%
end
%----------------------------------------------------------------------END:cbListTok
function [num,typ,rgb] = cbSelect(tok)
% Return the length, type and RGB values of any colorscheme.
%
switch lower(tok) % ColorName
    case 'brbg' % BrBG
        rgb = [84,48,5;140,81,10;166,97,26;191,129,45;216,179,101;223,194,125;246,232,195;245,245,245;199,234,229;128,205,193;90,180,172;53,151,143;1,133,113;1,102,94;0,60,48];
        typ = 'Diverging';
    case 'prgn' % PRGn
        rgb = [64,0,75;118,42,131;123,50,148;153,112,171;175,141,195;194,165,207;231,212,232;247,247,247;217,240,211;166,219,160;127,191,123;90,174,97;0,136,55;27,120,55;0,68,27];
        typ = 'Diverging';
    case 'piyg' % PiYG
        rgb = [142,1,82;197,27,125;208,28,139;222,119,174;233,163,201;241,182,218;253,224,239;247,247,247;230,245,208;184,225,134;161,215,106;127,188,65;77,172,38;77,146,33;39,100,25];
        typ = 'Diverging';
    case 'puor' % PuOr
        rgb = [127,59,8;179,88,6;230,97,1;224,130,20;241,163,64;253,184,99;254,224,182;247,247,247;216,218,235;178,171,210;153,142,195;128,115,172;94,60,153;84,39,136;45,0,75];
        typ = 'Diverging';
    case 'rdbu' % RdBu
        rgb = [103,0,31;178,24,43;202,0,32;214,96,77;239,138,98;244,165,130;253,219,199;247,247,247;209,229,240;146,197,222;103,169,207;67,147,195;5,113,176;33,102,172;5,48,97];
        typ = 'Diverging';
    case 'rdgy' % RdGy
        rgb = [103,0,31;178,24,43;202,0,32;214,96,77;239,138,98;244,165,130;253,219,199;255,255,255;224,224,224;186,186,186;153,153,153;135,135,135;64,64,64;77,77,77;26,26,26];
        typ = 'Diverging';
    case 'rdylbu' % RdYlBu
        rgb = [165,0,38;215,48,39;215,25,28;244,109,67;252,141,89;253,174,97;254,224,144;255,255,191;224,243,248;171,217,233;145,191,219;116,173,209;44,123,182;69,117,180;49,54,149];
        typ = 'Diverging';
    case 'rdylgn' % RdYlGn
        rgb = [165,0,38;215,48,39;215,25,28;244,109,67;252,141,89;253,174,97;254,224,139;255,255,191;217,239,139;166,217,106;145,207,96;102,189,99;26,150,65;26,152,80;0,104,55];
        typ = 'Diverging';
    case 'spectral' % Spectral
        rgb = [158,1,66;213,62,79;215,25,28;244,109,67;252,141,89;253,174,97;254,224,139;255,255,191;230,245,152;171,221,164;153,213,148;102,194,165;43,131,186;50,136,189;94,79,162];
        typ = 'Diverging';
    case 'accent' % Accent
        rgb = [127,201,127;190,174,212;253,192,134;255,255,153;56,108,176;240,2,127;191,91,23;102,102,102];
        typ = 'Qualitative';
    case 'dark2' % Dark2
        rgb = [27,158,119;217,95,2;117,112,179;231,41,138;102,166,30;230,171,2;166,118,29;102,102,102];
        typ = 'Qualitative';
    case 'paired' % Paired
        rgb = [166,206,227;31,120,180;178,223,138;51,160,44;251,154,153;227,26,28;253,191,111;255,127,0;202,178,214;106,61,154;255,255,153;177,89,40];
        typ = 'Qualitative';
    case 'pastel1' % Pastel1
        rgb = [251,180,174;179,205,227;204,235,197;222,203,228;254,217,166;255,255,204;229,216,189;253,218,236;242,242,242];
        typ = 'Qualitative';
    case 'pastel2' % Pastel2
        rgb = [179,226,205;253,205,172;203,213,232;244,202,228;230,245,201;255,242,174;241,226,204;204,204,204];
        typ = 'Qualitative';
    case 'set1' % Set1
        rgb = [228,26,28;55,126,184;77,175,74;152,78,163;255,127,0;255,255,51;166,86,40;247,129,191;153,153,153];
        typ = 'Qualitative';
    case 'set2' % Set2
        rgb = [102,194,165;252,141,98;141,160,203;231,138,195;166,216,84;255,217,47;229,196,148;179,179,179];
        typ = 'Qualitative';
    case 'set3' % Set3
        rgb = [141,211,199;255,255,179;190,186,218;251,128,114;128,177,211;253,180,98;179,222,105;252,205,229;217,217,217;188,128,189;204,235,197;255,237,111];
        typ = 'Qualitative';
    case 'blues' % Blues
        rgb = [247,251,255;239,243,255;222,235,247;198,219,239;189,215,231;158,202,225;107,174,214;66,146,198;49,130,189;33,113,181;8,81,156;8,69,148;8,48,107];
        typ = 'Sequential';
    case 'bugn' % BuGn
        rgb = [247,252,253;237,248,251;229,245,249;204,236,230;178,226,226;153,216,201;102,194,164;65,174,118;44,162,95;35,139,69;0,109,44;0,88,36;0,68,27];
        typ = 'Sequential';
    case 'bupu' % BuPu
        rgb = [247,252,253;237,248,251;224,236,244;191,211,230;179,205,227;158,188,218;140,150,198;140,107,177;136,86,167;136,65,157;129,15,124;110,1,107;77,0,75];
        typ = 'Sequential';
    case 'gnbu' % GnBu
        rgb = [247,252,240;240,249,232;224,243,219;204,235,197;186,228,188;168,221,181;123,204,196;78,179,211;67,162,202;43,140,190;8,104,172;8,88,158;8,64,129];
        typ = 'Sequential';
    case 'greens' % Greens
        rgb = [247,252,245;237,248,233;229,245,224;199,233,192;186,228,179;161,217,155;116,196,118;65,171,93;49,163,84;35,139,69;0,109,44;0,90,50;0,68,27];
        typ = 'Sequential';
    case 'greys' % Greys
        rgb = [255,255,255;247,247,247;240,240,240;217,217,217;204,204,204;189,189,189;150,150,150;115,115,115;99,99,99;82,82,82;37,37,37;37,37,37;0,0,0];
        typ = 'Sequential';
    case 'orrd' % OrRd
        rgb = [255,247,236;254,240,217;254,232,200;253,212,158;253,204,138;253,187,132;252,141,89;239,101,72;227,74,51;215,48,31;179,0,0;153,0,0;127,0,0];
        typ = 'Sequential';
    case 'oranges' % Oranges
        rgb = [255,245,235;254,237,222;254,230,206;253,208,162;253,190,133;253,174,107;253,141,60;241,105,19;230,85,13;217,72,1;166,54,3;140,45,4;127,39,4];
        typ = 'Sequential';
    case 'pubu' % PuBu
        rgb = [255,247,251;241,238,246;236,231,242;208,209,230;189,201,225;166,189,219;116,169,207;54,144,192;43,140,190;5,112,176;4,90,141;3,78,123;2,56,88];
        typ = 'Sequential';
    case 'pubugn' % PuBuGn
        rgb = [255,247,251;246,239,247;236,226,240;208,209,230;189,201,225;166,189,219;103,169,207;54,144,192;28,144,153;2,129,138;1,108,89;1,100,80;1,70,54];
        typ = 'Sequential';
    case 'purd' % PuRd
        rgb = [247,244,249;241,238,246;231,225,239;212,185,218;215,181,216;201,148,199;223,101,176;231,41,138;221,28,119;206,18,86;152,0,67;145,0,63;103,0,31];
        typ = 'Sequential';
    case 'purples' % Purples
        rgb = [252,251,253;242,240,247;239,237,245;218,218,235;203,201,226;188,189,220;158,154,200;128,125,186;117,107,177;106,81,163;84,39,143;74,20,134;63,0,125];
        typ = 'Sequential';
    case 'rdpu' % RdPu
        rgb = [255,247,243;254,235,226;253,224,221;252,197,192;251,180,185;250,159,181;247,104,161;221,52,151;197,27,138;174,1,126;122,1,119;122,1,119;73,0,106];
        typ = 'Sequential';
    case 'reds' % Reds
        rgb = [255,245,240;254,229,217;254,224,210;252,187,161;252,174,145;252,146,114;251,106,74;239,59,44;222,45,38;203,24,29;165,15,21;153,0,13;103,0,13];
        typ = 'Sequential';
    case 'ylgn' % YlGn
        rgb = [255,255,229;255,255,204;247,252,185;217,240,163;194,230,153;173,221,142;120,198,121;65,171,93;49,163,84;35,132,67;0,104,55;0,90,50;0,69,41];
        typ = 'Sequential';
    case 'ylgnbu' % YlGnBu
        rgb = [255,255,217;255,255,204;237,248,177;199,233,180;161,218,180;127,205,187;65,182,196;29,145,192;44,127,184;34,94,168;37,52,148;12,44,132;8,29,88];
        typ = 'Sequential';
    case 'ylorbr' % YlOrBr
        rgb = [255,255,229;255,255,212;255,247,188;254,227,145;254,217,142;254,196,79;254,153,41;236,112,20;217,95,14;204,76,2;153,52,4;140,45,4;102,37,6];
        typ = 'Sequential';
    case 'ylorrd' % YlOrRd
        rgb = [255,255,204;255,255,178;255,237,160;254,217,118;254,204,92;254,178,76;253,141,60;252,78,42;240,59,32;227,26,28;189,0,38;177,0,38;128,0,38];
        typ = 'Sequential';
    otherwise
        error('Colorscheme "%s" is not supported. Check the token tables.', tok)
end
num = size(rgb,1);
rgb = rgb./255;
%
end
%----------------------------------------------------------------------END:cbSelect
% Copyright (c) 2015 Stephen Cobeldick
% Copyright (c) 2002 Cynthia Brewer, Mark Harrower, and The Pennsylvania State University.
%
% Licensed under the Apache License, Version 2.0 (the "License");
% you may not use this file except in compliance with the License.
% You may obtain a copy of the License at
%
% http://www.apache.org/licenses/LICENSE-2.0
%
% Unless required by applicable law or agreed to in writing, software
% distributed under the License is distributed on an "AS IS" BASIS,
% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
% See the License for the specific language governing permissions and limitations under the License.
%----------------------------------------------------------------------END:license


function h = imagescn(varargin) 
% imagescn behaves just like imagesc, but makes NaNs transparent, sets
% axis to xy (aka ydirection normal) if xdata and ydata are included, and has a little more 
% error checking than imagesc. 
% 
% Syntax 
% 
%  imagescn(C) 
%  imagescn(x,y,C) 
%  imagescn(x,y,C,clims) 
%  imagescn('PropertyName',PropertyValue,...) 
%  h = imagescn(...) 
% 
% Description 
% 
% imagescn(C) displays the data in array C as an image that uses the full range of colors in the colormap. 
% Each element of C specifies the color for 1 pixel of the image. The resulting image is an m-by-n grid of 
% pixels where m is the number of columns and n is the number of rows in C. The row and column indices of 
% the elements determine the centers of the corresponding pixels. NaN values in C appear transparent. 
% 
% imagescn(x,y,C) specifies x and y locations of the centers of the pixels in C. If x and y are two-element
% vectors, the outside rows and columns of C are centered on the values in x and y. Mimicking imagesc, if 
% x or y are vectors with more than two elements, only the first and last elements of of the vectors are 
% considered, and spacing is automatically scaled as if you entered two-element arrays. The imagescn function
% takes this one step further, and allows you to enter x and y as 2D grids the same size as C. If x and y
% are included, the imagescn function automatically sets axes to cartesian xy rather than the (reverse) ij axes. 
% 
% imagescn(x,y,C,clims) specifies the data values that map to the first and last elements of the colormap. 
% Specify clims as a two-element vector of the form [cmin cmax], where values less than or equal to cmin 
% map to the first color in the colormap and values greater than or equal to cmax map to the last color in 
% the colormap.
% 
% imagescn('PropertyName',PropertyValue,...) specifies image properties as name-value pairs. 
% 
% h = imagescn(...) retrns a handle of the object created. 
% 
% Differences between imagesc, imagescn, and pcolor
% The imagescn function plots data with imagesc, but after plotting, sets NaN pixels to an 
% alpha value of 0. The imagesc function allows input coordinates x and y to be grids, which 
% are assumed to be evenly-spaced and monotonic as if created by meshgrid. If x and y data 
% are included when calling imagescn, y axis direction is changed from reverse to normal. 
% 
% The imagescn function is faster than pcolor. Pcolor (nonsensically) deletes an outside row 
% and column of data, and pcolor also refuses to plot data points closest to any NaN holes. 
% The imagescn function does not delete any data.  However, you may still sometimes wish to 
% use pcolor if x,y coordinates are not evenly spaced or if you want interpolated shading. 
% 
% Examples 
% For examples, type 
% 
%  cdt imagescn
% 
% Author Info 
% 
% This function was written by Chad A. Greene of the University of 
% Texas Institute for Geophysics (UTIG), January 2017. 
% http://www.chadagreene.com 
% 
% See also imagesc, image, and pcolor.
% The imagesc function does not have error checking regarding number of elements
% in xdata, ydata versus number of elements in the input image, so I'm gonna add
% some error checking: 

% Check inputs: 

xydata = false; 
if nargin>2
   if all([isnumeric(varargin{1}) isnumeric(varargin{2})]) 
      % This is an assumption that should typically be safe to make: 
      xydata = true; 
      
      % Determine if input coordinates are meshgrid type and if so, convert to vector: 
      if isequal(size(varargin{1}),size(varargin{2}),size(varargin{3}))
         X = varargin{1}; 
         Y = varargin{2}; 
         
         varargin{1} = [X(1,1) X(end,end)]; 
         varargin{2} = [Y(1,1) Y(end,end)]; 
      end
   end
end

% Plot

% Plot imagesc: 
h = imagesc(varargin{:}); 

% Make NaNs transparent: 
cd = get(h,'CData'); 
set(h,'alphadata',isfinite(cd)); 

if xydata
   axis xy
end

if nargout==0
   clear h
end

end


function [MASKinv,VM,nodes,node]=inverse_mask(MASK,ctr)

% Creates mask and nodes for a fixed grounded ice domain in order to solve
% the ice-sheet equation more efficiently. Only used for Pollard inversion
% on the grounded ice sheet (inverse=1); otherwise, all grid cells are
% taken into account

    MASKinv=MASK;
    if ctr.inverse==1
        MASK1=circshift(MASK,[0 -1]);
        MASK2=circshift(MASK,[0 1]);
        MASK3=circshift(MASK,[-1 0]);
        MASK4=circshift(MASK,[1 0]);
        % adjacent floating grid cell
        MASKinv(MASK==0 & (MASK1==1 | MASK2==1 | MASK3==1 | MASK4==1))=1;
        % FP: adjacent-adjacent floating grid cell (upstream differences)
        MASK1=circshift(MASKinv,[0 -1]);
        MASK2=circshift(MASKinv,[0 1]);
        MASK3=circshift(MASKinv,[-1 0]);
        MASK4=circshift(MASKinv,[1 0]);
        MASKinv(MASK==0 & (MASK1==1 | MASK2==1 | MASK3==1 | MASK4==1))=1;
        VM=reshape(MASKinv,ctr.imax*ctr.jmax,1);
        nodes=sum(sum(MASKinv));
        node=zeros(ctr.imax,ctr.jmax);
        node(MASKinv>0)=linspace(1,nodes,nodes);
    else
        nodes=ctr.imax*ctr.jmax;
        node=linspace(1,nodes,nodes)';
        node=reshape(node,[ctr.imax ctr.jmax]);
        VM=ones(nodes,1);
    end
end



function [gradm,gradmx,gradmy,Hm,Hmx,Hmy,Bm,Bmx,Bmy,Tsm,Mbm,signx,signy]= ...
        staggered_grid(sn,H,B,Ts,Mb,ctr)
       
    sn1=circshift(sn,[-1 0]); % sn(i+1,j)
    sn2=circshift(sn,[0 -1]); % sn(i,j+1)
    sn3=circshift(sn,[-1 -1]); % sn(i+1,j+1)
    
    Hm=(H+circshift(H,[-1 0])+circshift(H,[0 -1])+circshift(H,[-1 -1]))/4.; 
    Bm=(B+circshift(B,[-1 0])+circshift(B,[0 -1])+circshift(B,[-1 -1]))/4.; 
    Tsm=(Ts+circshift(Ts,[-1 0])+circshift(Ts,[0 -1])+circshift(Ts,[-1 -1]))/4.; 
    Mbm=(Mb+circshift(Mb,[-1 0])+circshift(Mb,[0 -1])+circshift(Mb,[-1 -1]))/4.; 
    gradm=((sn2+sn3-sn-sn1)/(2*ctr.delta)).^2+((sn1+sn3-sn-sn2)/(2*ctr.delta)).^2;
   
    gradmx=(sn2-sn)/ctr.delta;
    Hmx=(H+circshift(H,[0 -1]))/2.;
    Bmx=(B+circshift(B,[0 -1]))/2.;
    gradmy=(sn1-sn)/ctr.delta;
    Hmy=(H+circshift(H,[-1 0]))/2.;
    Bmy=(B+circshift(B,[-1 0]))/2.;
    
    signx=ones(ctr.imax,ctr.jmax);
    signy=ones(ctr.imax,ctr.jmax);
    signx(gradmx>0)=-1;
    signy(gradmy>0)=-1;
    gradmx=gradmx.^2;
    gradmy=gradmy.^2;

end


function [glMASK,MASK]=grounding_line_mask(MASK,H,ctr)

glMASK=MASK;
MASK1=circshift(MASK,[0 -1]); % MASK(i,j+1)
MASK2=circshift(MASK,[0 1]); % MASK(i,j-1)
MASK3=circshift(MASK,[-1 0]); % MASK(i+1,j)
MASK4=circshift(MASK,[1 0]); % MASK(i-1,j)

glMASK(MASK==1 & (MASK1==0 | MASK2==0 | MASK3==0 | MASK4==0))=2;

MASK1=circshift(glMASK,[0 -1]); % glMASK(i,j+1)
MASK2=circshift(glMASK,[0 1]); % glMASK(i,j-1)
MASK3=circshift(glMASK,[-1 0]); % glMASK(i+1,j)
MASK4=circshift(glMASK,[1 0]); % glMASK(i-1,j)

% adjacent floating grid cell (=3)
glMASK(MASK==0 & (MASK1==2 | MASK2==2 | MASK3==2 | MASK4==2))=3;

if ctr.shelf==1
    glMASK(glMASK==0)=4; % ice shelf (=4)
    if ctr.calving>=1
        glMASK(glMASK==4 & H<5)=6;  % sea ice/ocean (=6) FP: changed this 3/9/18 to remove icebergs
        MASK1=circshift(glMASK,[0 -1]); % glMASK(i,j+1)
        MASK2=circshift(glMASK,[0 1]); % glMASK(i,j-1)
        MASK3=circshift(glMASK,[-1 0]); % glMASK(i+1,j)
        MASK4=circshift(glMASK,[1 0]); % glMASK(i-1,j)
        glMASK((glMASK==3 | glMASK==4) & (MASK1==6 | MASK2==6 | ...
            MASK3==6 | MASK4==6))=5; % Calving front (=5)
    end
end
if ctr.mismip==1
    glMASK(:,1)=1;
end

end


function [glMASKm]=grounding_line_maskm(MASKm)

glMASKm=MASKm;
MASK1=circshift(MASKm,[0 -1]); % MASK(i,j+1)
MASK2=circshift(MASKm,[0 1]); % MASK(i,j-1)
MASK3=circshift(MASKm,[-1 0]); % MASK(i+1,j)
MASK4=circshift(MASKm,[1 0]); % MASK(i-1,j)

glMASKm(MASKm==1 & (MASK1==0 | MASK2==0 | MASK3==0 | MASK4==0))=2;

end


function [glMASKx,glMASKy]=schoof_mask(HAFmx,HAFmy,ctr)

% Masks used for the calculation of Schoof/Tsai grounding line fluxes

MASK=ones(ctr.imax,ctr.jmax);
MASK(HAFmx<0)=0;
MASK(:,ctr.jmax)=0;
glMASKx=MASK;
MASK1=circshift(MASK,[0 -1]); % MASK(i,j+1)
MASK2=circshift(MASK,[0 1]); % MASK(i,j-1)
MASK3=circshift(MASK,[-1 0]); % MASK(i+1,j)
MASK4=circshift(MASK,[1 0]); % MASK(i-1,j)

glMASKx(MASK==1 & (MASK1==0 | MASK2==0 | MASK3==0 | MASK4==0))=2;

MASK1=circshift(glMASKx,[0 -1]); % glMASK(i,j+1)
MASK2=circshift(glMASKx,[0 1]); % glMASK(i,j-1)
MASK3=circshift(glMASKx,[-1 0]); % glMASK(i+1,j)
MASK4=circshift(glMASKx,[1 0]); % glMASK(i-1,j)

% adjacent floating grid cell (=3)
glMASKx(MASK==0 & (MASK1==2 | MASK2==2 | MASK3==2 | MASK4==2))=3;
glMASKx(glMASKx==3 & HAFmx>0)=0;

MASK=ones(ctr.imax,ctr.jmax);
MASK(HAFmy<0)=0;
MASK(ctr.imax,:)=0;
glMASKy=MASK;
MASK1=circshift(MASK,[0 -1]); % MASK(i,j+1)
MASK2=circshift(MASK,[0 1]); % MASK(i,j-1)
MASK3=circshift(MASK,[-1 0]); % MASK(i+1,j)
MASK4=circshift(MASK,[1 0]); % MASK(i-1,j)

glMASKy(MASK==1 & (MASK1==0 | MASK2==0 | MASK3==0 | MASK4==0))=2;

MASK1=circshift(glMASKy,[0 -1]); % glMASK(i,j+1)
MASK2=circshift(glMASKy,[0 1]); % glMASK(i,j-1)
MASK3=circshift(glMASKy,[-1 0]); % glMASK(i+1,j)
MASK4=circshift(glMASKy,[1 0]); % glMASK(i-1,j)

% adjacent floating grid cell (=3)
glMASKy(MASK==0 & (MASK1==2 | MASK2==2 | MASK3==2 | MASK4==2))=3;
glMASKy(glMASKy==3 & HAFmy>=0)=0;

end


function [Ax,Ay,Axy]=staggeredA(A)
    
% Flow parmater A on different staggered grids. Originally, A is determined
% on the diffusivity grid (d-grid)

    A1=circshift(A,[0 1]); % A(i,j-1)
    A2=circshift(A,[1 0]); % A(i-1,j)
    A3=circshift(A,[1 1]); % A(i-1,j-1)
    
    Ax=(A+A2)/2.; % A on u-grid
    Ay=(A+A1)/2.; % A on v-grid
    Axy=(A+A1+A2+A3)/4.; % A on H-grid

end


function u=velocity_on_grid(ux,uy)

% Determine magnitude of velocities based on velocities on the staggered u
% and v-grids. Only used for visualization purposes

    ux1=circshift(ux,[-1 0]); % ux(i+1,j)
    uy1=circshift(uy,[0 -1]); % uy(i,j+1)
    u=sqrt((0.5*(ux+ux1)).^2+(0.5*(uy+uy1)).^2);
    
end


function [uxsch,uysch]=grounding_line_flux(ctr,glMASKx,glMASKy,HAFmx,HAFmy,Bmx,Bmy,SLR,par, ...
    Axy,Thetax,Thetay,tanphi,ncorx,ncory,ux,uy,Hmx,Hmy,Asf)
    
% Grounding line flux calculation according to Schoof and Tsai following
% the method of Pollard and DeConto (2012)
    
% Modification: apply condition at last grounded AND first floating grid
% point + only apply at both grid points when Schoof flux is lower than SSA
% flux

    uxsch=ux;
    uysch=uy;
    
    if ctr.schoof==1
        q0=(Axy*(par.rho*par.g)^(par.n+1)*(1-par.rho/par.rhow)^par.n.*Asf.^(1/ctr.m)/4^par.n).^(ctr.m/(ctr.m+1));
        qe0=ctr.m*(par.n+2)/(ctr.m+1);
    end

    % conditions for (i, j+1)
    qgx=zeros(ctr.imax,ctr.jmax);
    Mgl=zeros(ctr.imax,ctr.jmax);
    glMASK1=circshift(glMASKx,[0 -1]); % glMASK(i,j+1)
    HAF1=circshift(HAFmx,[0 -1]); % HAF(i,j+1)
    B1=circshift(Bmx,[0 -1]); % B(i,j+1)
    Thetax1=circshift(Thetax,[0 -1]); % Thetax(i,j+1)
    Mgl(glMASKx==2 & glMASK1==3)=1;
    fracgx=(-HAFmx)./(HAF1-HAFmx);
    hbgx=(1.-fracgx).*Bmx+fracgx.*B1;
    hgx=abs(SLR-hbgx)*par.rhow/par.rho;
    if ctr.schoof==1
        ugx=q0.*hgx.^qe0.*Thetax1;
    else
        ugx=par.Qo*8*Axy*(par.rho*par.g)^par.n*(1.-par.rho/par.rhow)^(par.n-1) ...
            .*Thetax1./(tanphi*4^par.n).*hgx.^(par.n+1);
    end
    qgx(Mgl==1)=ugx(Mgl==1).*hgx(Mgl==1);
    % GL retreat
    uxsch(Mgl==1)=ux(Mgl==1)+(qgx(Mgl==1)./max(Hmx(Mgl==1),0.01)-ux(Mgl==1)).*ncorx(Mgl==1);
    % GL advance
    if ctr.inverse==0
        Mgl1=circshift(Mgl,[0 1]); % condition for i,j+1
        qgx1=circshift(qgx,[0 1]);
        ncx=circshift(ncorx,[0 1]);
        uxsch(Mgl1==1)=ux(Mgl1==1)+(qgx1(Mgl1==1)./max(Hmx(Mgl1==1),0.01)-ux(Mgl1==1)).*ncx(Mgl1==1);
    end
    
    
    % conditions for (i, j-1)
    qgx=zeros(ctr.imax,ctr.jmax);
    Mgl=zeros(ctr.imax,ctr.jmax);
    glMASK1=circshift(glMASKx,[0 1]); % glMASK(i,j-1)
    HAF1=circshift(HAFmx,[0 1]); % HAF(i,j-1)
    B1=circshift(Bmx,[0 1]); % B(i,j-1)
    Thetax1=circshift(Thetax,[0 1]); % Thetax(i,j-1)
    Mgl(glMASKx==2 & glMASK1==3)=1;
    fracgx=(-HAF1)./(HAFmx-HAF1);
    hbgx=(1.-fracgx).*B1+fracgx.*Bmx;
    hgx=abs((SLR-hbgx)*par.rhow/par.rho);
    if ctr.schoof==1
        ugx=q0.*hgx.^qe0.*Thetax1;
    else
        ugx=par.Qo*8*Axy*(par.rho*par.g)^par.n*(1.-par.rho/par.rhow)^(par.n-1) ...
            .*Thetax1./(tanphi*4^par.n).*hgx.^(par.n+1);
    end
    qgx(Mgl==1)=ugx(Mgl==1).*hgx(Mgl==1);
    % GL retreat
    uxsch(Mgl==1)=ux(Mgl==1)+(qgx(Mgl==1)./max(Hmx(Mgl==1),0.01)-ux(Mgl==1)).*ncorx(Mgl==1);
    % GL advance
    if ctr.inverse==0
        Mgl1=circshift(Mgl,[0 -1]); % condition for i,j-1
        qgx1=circshift(qgx,[0 -1]);
        ncx=circshift(ncorx,[0 -1]);
        uxsch(Mgl1==1)=ux(Mgl1==1)+(qgx1(Mgl1==1)./max(Hmx(Mgl1==1),0.01)-ux(Mgl1==1)).*ncx(Mgl1==1);
    end
    
    
    % conditions for (i+1, j)
    qgy=zeros(ctr.imax,ctr.jmax);
    Mgl=zeros(ctr.imax,ctr.jmax);
    glMASK1=circshift(glMASKy,[-1 0]); % glMASK(i+1,j)
    HAF1=circshift(HAFmy,[-1 0]); % HAF(i+1,j)
    B1=circshift(Bmy,[-1 0]); % B(i+1,j)
    Thetay1=circshift(Thetay,[-1 0]); % Thetay(i+1,j)
    Mgl(glMASKy==2 & glMASK1==3)=1;
    fracgy=(-HAFmy)./(HAF1-HAFmy);
    hbgy=(1.-fracgy).*Bmy+fracgy.*B1;
    hgy=abs((SLR-hbgy)*par.rhow/par.rho); 
    if ctr.schoof==1
        ugy=q0.*hgy.^qe0.*Thetay1;
    else
        ugy=par.Qo*8*Axy*(par.rho*par.g)^par.n*(1.-par.rho/par.rhow)^(par.n-1) ...
            .*Thetay1./(tanphi*4^par.n).*hgy.^(par.n+1);
    end
    qgy(Mgl==1)=ugy(Mgl==1).*hgy(Mgl==1);
    % GL retreat
    uysch(Mgl==1)=uy(Mgl==1)+(qgy(Mgl==1)./max(Hmy(Mgl==1),0.01)-uy(Mgl==1)).*ncory(Mgl==1);
    % GL advance
    if ctr.inverse==0
        Mgl1=circshift(Mgl,[1 0]); % condition for i+1,j
        qgy1=circshift(qgy,[1 0]);
        ncy=circshift(ncory,[1 0]);
        uysch(Mgl1==1)=uy(Mgl1==1)+(qgy1(Mgl1==1)./max(Hmy(Mgl1==1),0.01)-uy(Mgl1==1)).*ncy(Mgl1==1);
    end


    % conditions for (i-1, j)
    qgy=zeros(ctr.imax,ctr.jmax);
    Mgl=zeros(ctr.imax,ctr.jmax);
    glMASK1=circshift(glMASKy,[1 0]); % glMASK(i-1,j)
    HAF1=circshift(HAFmy,[1 0]); % HAF(i-1,j)
    B1=circshift(Bmy,[1 0]); % B(i-1,j)
    Thetay1=circshift(Thetay,[1 0]); % Thetay(i-1,j)
    Mgl(glMASKy==2 & glMASK1==3)=1;
    fracgy=(-HAF1)./(HAFmy-HAF1);
    hbgy=(1.-fracgy).*B1+fracgy.*Bmy;
    hgy=abs((SLR-hbgy)*par.rhow/par.rho);
    if ctr.schoof==1
        ugy=q0.*hgy.^qe0.*Thetay1;
    else
        ugy=par.Qo*8*Axy*(par.rho*par.g)^par.n*(1.-par.rho/par.rhow)^(par.n-1) ...
            .*Thetay1./(tanphi*4^par.n).*hgy.^(par.n+1);
    end
    qgy(Mgl==1)=ugy(Mgl==1).*hgy(Mgl==1);
    % GL retreat
    uysch(Mgl==1)=uy(Mgl==1)+(qgy(Mgl==1)./max(Hmy(Mgl==1),0.01)-uy(Mgl==1)).*ncory(Mgl==1);
    % GL advance
    if ctr.inverse==0
        Mgl1=circshift(Mgl,[-1 0]); % condition for i-1,j
        qgy1=circshift(qgy,[-1 0]);
        ncy=circshift(ncory,[-1 0]);
        uysch(Mgl1==1)=uy(Mgl1==1)+(qgy1(Mgl1==1)./max(Hmy(Mgl1==1),0.01)-uy(Mgl1==1)).*ncy(Mgl1==1);
    end
end


%--------------------------------------
% Solving SSA equation
%--------------------------------------
function [u,v,s]=StreamStag_opt(nodeu,nodev,s0,MASKmx,MASKmy, ...
    H,eta,betax,betay,u,v,taudx,taudy,ctr,par)
%LZ: old solution without deformational part
limit=1e-5;

% sparse matrix solution for two two-dimensional ice shelf velocity field
% with kinematic boundary conditions
% u-field on u-grid, viscosity on h-grid
% u-velocities: quadrant U and Uv of solution matrix
% V-velocities: quadrant V and Vu of solution matrix
% Subsequent interleaving of u and v velocities (Quiquet et al., 2018) to
% improve stability and speed of the algorithm

R0=zeros(ctr.imax,ctr.jmax);
U0=zeros(ctr.imax,ctr.jmax); % u(i,j)
U1=zeros(ctr.imax,ctr.jmax); % u(i,j+1)
U2=zeros(ctr.imax,ctr.jmax); % u(i,j-1)
U3=zeros(ctr.imax,ctr.jmax); % u(i+1,j)
U4=zeros(ctr.imax,ctr.jmax); % u(i-1,j)
U5=zeros(ctr.imax,ctr.jmax); % u(i+1,j+1)
U6=zeros(ctr.imax,ctr.jmax); % u(i+1,j-1)
U7=zeros(ctr.imax,ctr.jmax); % u(i-1,j+1)
U8=zeros(ctr.imax,ctr.jmax); % u(i-1,j-1)
U9=zeros(ctr.imax,ctr.jmax); % periodic BC i=1
U10=zeros(ctr.imax,ctr.jmax); % periodic BC i=imax
Uv0=zeros(ctr.imax,ctr.jmax); % v(i,j)
Uv1=zeros(ctr.imax,ctr.jmax); % v(i,j+1)
Uv2=zeros(ctr.imax,ctr.jmax); % v(i-1,j)
Uv3=zeros(ctr.imax,ctr.jmax); % v(i-1,j+1)

eta1=circshift(eta,[0 -1]); % eta(i,j+1)
H1=circshift(H,[0 -1]); % H(i,j+1)

MASKb=zeros(ctr.imax,ctr.jmax); % domain mask for SSA
MASKb(2:ctr.imax-1,2:ctr.jmax-2)=1;
MASKb(MASKmx==0)=0;

U0(MASKb==1)=-5.*(eta1(MASKb==1)+eta(MASKb==1))/(ctr.delta^2)- ...
    betax(MASKb==1);
U1(MASKb==1)=2.*(eta1(MASKb==1)+eta(MASKb==1))/(ctr.delta^2.);
U2(MASKb==1)=2.*(eta1(MASKb==1)+eta(MASKb==1))/(ctr.delta^2.);
U3(MASKb==1)=0.5*(eta1(MASKb==1)+eta(MASKb==1))/(ctr.delta^2.);
U4(MASKb==1)=0.5*(eta1(MASKb==1)+eta(MASKb==1))/(ctr.delta^2.);
Uv0(MASKb==1)=-1.5*(eta(MASKb==1)+eta1(MASKb==1))/(ctr.delta^2);
Uv1(MASKb==1)=1.5*(eta(MASKb==1)+eta1(MASKb==1))/(ctr.delta^2);
Uv2(MASKb==1)=1.5*(eta(MASKb==1)+eta1(MASKb==1))/(ctr.delta^2);
Uv3(MASKb==1)=-1.5*(eta(MASKb==1)+eta1(MASKb==1))/(ctr.delta^2);
R0(MASKb==1)=-taudx(MASKb==1);

MASKb=zeros(ctr.imax,ctr.jmax); % domain mask for non-SSA
MASKb(2:ctr.imax-1,2:ctr.jmax-2)=1;
MASKb(MASKmx>0)=0;

U0(MASKb==1)=1;
R0(MASKb==1)=u(MASKb==1);

% boundary conditions

if ctr.shelf==1 && ctr.mismip==0
    % j=1; contact with ocean (upwinding in x)
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(2:ctr.imax-1,1)=1;
    U0(MASKb==1)=-4.*eta1(MASKb==1)/ctr.delta;
    U1(MASKb==1)=4.*eta1(MASKb==1)/ctr.delta;
    Uv1(MASKb==1)=2.*eta(MASKb==1)/ctr.delta;
    Uv3(MASKb==1)=-2.*eta(MASKb==1)/ctr.delta;
    R0(MASKb==1)=0.5*par.rho*par.g*H1(MASKb==1).^2.*(1.-par.rho/par.rhow);

    % j=jmax-1; contact with ocean
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(2:ctr.imax-1,ctr.jmax-1)=1;
    U0(MASKb==1)=4.*eta(MASKb==1)/ctr.delta;
    U2(MASKb==1)=-4.*eta(MASKb==1)/ctr.delta;
    Uv0(MASKb==1)=2.*eta(MASKb==1)/ctr.delta;
    Uv2(MASKb==1)=-2.*eta(MASKb==1)/ctr.delta;
    R0(MASKb==1)=0.5*par.rho*par.g*H(MASKb==1).^2.*(1.-par.rho/par.rhow);

    % i=1; contact with ocean (v-direction)
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(1,2:ctr.jmax-2)=1;
    U0(MASKb==1)=-1/ctr.delta;
    U3(MASKb==1)=1./ctr.delta;
    Uv0(MASKb==1)=-1./ctr.delta;
    Uv1(MASKb==1)=1./ctr.delta;
    R0(MASKb==1)=0;

    % i=imax; contact with ocean (v-direction)
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(ctr.imax,2:ctr.jmax-2)=1;
    U0(MASKb==1)=1./ctr.delta;
    U4(MASKb==1)=-1./ctr.delta;
    Uv2(MASKb==1)=-1./ctr.delta;
    Uv3(MASKb==1)=1./ctr.delta;
    R0(MASKb==1)=0;
    
    % j=jmax;
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(:,ctr.jmax)=1;
    U0(MASKb==1)=1;
    R0(MASKb==1)=NaN;
    
    % Model corners
    U0(1,1)=1;
    U1(1,1)=-1;
    U3(1,1)=-1;
    U5(1,1)=1;
    R0(1,1)=0;
    U0(1,ctr.jmax-1)=1;
    U2(1,ctr.jmax-1)=-1;
    U3(1,ctr.jmax-1)=-1;
    U6(1,ctr.jmax-1)=1;
    R0(1,ctr.jmax-1)=0;
    U0(ctr.imax,1)=1;
    U1(ctr.imax,1)=-1;
    U4(ctr.imax,1)=-1;
    U7(ctr.imax,1)=1;
    R0(ctr.imax,1)=0;
    U0(ctr.imax,ctr.jmax-1)=1;
    U2(ctr.imax,ctr.jmax-1)=-1;
    U4(ctr.imax,ctr.jmax-1)=-1;
    U8(ctr.imax,ctr.jmax-1)=1;
    R0(ctr.imax,ctr.jmax-1)=0;
    
elseif ctr.shelf==1 && ctr.mismip==1
    
    % j=1: ice divide (symmetric)
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(1:ctr.imax,1)=1;
    U0(MASKb==1)=1;
    U1(MASKb==1)=1;
    R0(MASKb==1)=0;
    
    % j=jmax-1; contact with ocean
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(2:ctr.imax-1,ctr.jmax-1)=1;
    U0(MASKb==1)=4.*eta(MASKb==1)/ctr.delta;
    U2(MASKb==1)=-4.*eta(MASKb==1)/ctr.delta;
    Uv0(MASKb==1)=2.*eta(MASKb==1)/ctr.delta;
    Uv2(MASKb==1)=-2.*eta(MASKb==1)/ctr.delta;
    R0(MASKb==1)=0.5*par.rho*par.g*H(MASKb==1).^2.*(1.-par.rho/par.rhow);
    
    % i=1: periodic boundary condition
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(1,2:ctr.jmax)=1;
    U0(MASKb==1)=1;
    U9(MASKb==1)=-1;
    R0(MASKb==1)=0;
    
    % i=imax: periodic boundary condition
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(ctr.imax,2:ctr.jmax)=1;
    U0(MASKb==1)=1;
    U10(MASKb==1)=-1;
    R0(MASKb==1)=0;
    
    % j=jmax;
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(:,ctr.jmax)=1;
    U0(MASKb==1)=1;
    R0(MASKb==1)=NaN;
    
else
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(1,1:ctr.jmax-1)=1;
    MASKb(ctr.imax,1:ctr.jmax-1)=1;
    MASKb(:,1)=1;
    MASKb(:,ctr.jmax-1)=1;
    U0(MASKb==1)=1;
    R0(MASKb==1)=0;
    
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(:,ctr.jmax)=1;
    U0(MASKb==1)=1;
    R0(MASKb==1)=NaN;
end

% v-velocities: quadrant V and Vu of solution matrix

S0=zeros(ctr.imax,ctr.jmax);
V0=zeros(ctr.imax,ctr.jmax); % v(i,j)
V1=zeros(ctr.imax,ctr.jmax); % v(i+1,j)
V2=zeros(ctr.imax,ctr.jmax); % v(i-1,j)
V3=zeros(ctr.imax,ctr.jmax); % v(i,j+1)
V4=zeros(ctr.imax,ctr.jmax); % v(i,j-1)
V5=zeros(ctr.imax,ctr.jmax); % v(i+1,j+1)
V6=zeros(ctr.imax,ctr.jmax); % v(i-1,j+1)
V7=zeros(ctr.imax,ctr.jmax); % v(i+1,j-1)
V8=zeros(ctr.imax,ctr.jmax); % v(i-1,j-1)
V9=zeros(ctr.imax,ctr.jmax); % Periodic BC on i=1
V10=zeros(ctr.imax,ctr.jmax); % Periodic BC on i=imax-1
V11=zeros(ctr.imax,ctr.jmax); % Symmetric ice divide v(i,j+3)
Vu0=zeros(ctr.imax,ctr.jmax); % u(i,j)
Vu1=zeros(ctr.imax,ctr.jmax); % u(i+1,j)
Vu2=zeros(ctr.imax,ctr.jmax); % u(i,j-1)
Vu3=zeros(ctr.imax,ctr.jmax); % u(i+1,j-1)

eta1=circshift(eta,[-1 0]); % eta(i+1,j)
H1=circshift(H,[-1 0]); % H(i+1,j)

MASKb=zeros(ctr.imax,ctr.jmax); % domain mask for SSA
MASKb(2:ctr.imax-2,2:ctr.jmax-1)=1;
MASKb(MASKmy==0)=0;

V0(MASKb==1)=-5.*(eta1(MASKb==1)+eta(MASKb==1))/(ctr.delta^2)- ...
    betay(MASKb==1);
V1(MASKb==1)=2.*(eta1(MASKb==1)+eta(MASKb==1))/(ctr.delta^2.);
V2(MASKb==1)=2.*(eta1(MASKb==1)+eta(MASKb==1))/(ctr.delta^2.);
V3(MASKb==1)=0.5*(eta1(MASKb==1)+eta(MASKb==1))/(ctr.delta^2.);
V4(MASKb==1)=0.5*(eta1(MASKb==1)+eta(MASKb==1))/(ctr.delta^2.);
Vu0(MASKb==1)=-1.5*(eta(MASKb==1)+eta1(MASKb==1))/(ctr.delta^2);
Vu1(MASKb==1)=1.5*(eta(MASKb==1)+eta1(MASKb==1))/(ctr.delta^2);
Vu2(MASKb==1)=1.5*(eta(MASKb==1)+eta1(MASKb==1))/(ctr.delta^2);
Vu3(MASKb==1)=-1.5*(eta(MASKb==1)+eta1(MASKb==1))/(ctr.delta^2);
S0(MASKb==1)=-taudy(MASKb==1);

MASKb=zeros(ctr.imax,ctr.jmax); % domain mask for non-SSA
MASKb(2:ctr.imax-2,2:ctr.jmax-1)=1;
MASKb(MASKmy>0)=0;
V0(MASKb==1)=1;
S0(MASKb==1)=v(MASKb==1);

% boundary conditions

if ctr.shelf==1 && ctr.mismip==0
    % i=1; contact with ocean
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(1,2:ctr.jmax-1)=1;
    V0(MASKb==1)=-4.*eta1(MASKb==1)/ctr.delta;
    V1(MASKb==1)=4.*eta1(MASKb==1)/ctr.delta;
    Vu1(MASKb==1)=2.*eta(MASKb==1)/ctr.delta;
    Vu3(MASKb==1)=-2.*eta(MASKb==1)/ctr.delta;
    S0(MASKb==1)=0.5*par.rho*par.g*H1(MASKb==1).^2.*(1.-par.rho/par.rhow);

    % i=imax-1; contact with ocean
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(ctr.imax-1,2:ctr.jmax-1)=1;
    V0(MASKb==1)=4.*eta(MASKb==1)/ctr.delta;
    V2(MASKb==1)=-4.*eta(MASKb==1)/ctr.delta;
    Vu0(MASKb==1)=2.*eta(MASKb==1)/ctr.delta;
    Vu2(MASKb==1)=-2.*eta(MASKb==1)/ctr.delta;
    S0(MASKb==1)=0.5*par.rho*par.g*H(MASKb==1).^2.*(1.-par.rho/par.rhow);

    % j=1; contact with ocean (u-direction)
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(2:ctr.imax-2,1)=1;
    V0(MASKb==1)=-1/ctr.delta;
    V3(MASKb==1)=1./ctr.delta;
    Vu0(MASKb==1)=-1./ctr.delta;
    Vu1(MASKb==1)=1./ctr.delta;
    S0(MASKb==1)=0;

    % j=jmax; contact with ocean (u-direction)
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(2:ctr.imax-2,ctr.jmax)=1;
    V0(MASKb==1)=1./ctr.delta;
    V4(MASKb==1)=-1./ctr.delta;
    Vu2(MASKb==1)=-1./ctr.delta;
    Vu3(MASKb==1)=1./ctr.delta;
    S0(MASKb==1)=0;
    
    % Model corners
    V0(1,1)=1;
    V1(1,1)=-1;
    V3(1,1)=-1;
    V5(1,1)=1;
    S0(1,1)=0;
    V0(ctr.imax-1,1)=1;
    V2(ctr.imax-1,1)=-1;
    V3(ctr.imax-1,1)=-1;
    V6(ctr.imax-1,1)=1;
    S0(ctr.imax-1,1)=0;
    V0(1,ctr.jmax)=1;
    V1(1,ctr.jmax)=-1;
    V4(1,ctr.jmax)=-1;
    V7(1,ctr.jmax)=1;
    S0(1,ctr.jmax)=0;
    V0(ctr.imax-1,ctr.jmax)=1;
    V2(ctr.imax-1,ctr.jmax)=-1;
    V4(ctr.imax-1,ctr.jmax)=-1;
    V8(ctr.imax-1,ctr.jmax)=1;
    S0(ctr.imax-1,ctr.jmax)=0;
    
    % i=imax;
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(ctr.imax,:)=1;
    V0(MASKb==1)=1;
    S0(MASKb==1)=NaN;
    
elseif ctr.shelf==1 && ctr.mismip==1
    
    % j=1: ice divide (symmetric)
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(1:ctr.imax-1,1)=1;
    V0(MASKb==1)=1;
    V11(MASKb==1)=-1;
    S0(MASKb==1)=0;
    
    % j=jmax; contact with ocean (u-direction)
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(2:ctr.imax-2,ctr.jmax)=1;
    V0(MASKb==1)=1./ctr.delta;
    V4(MASKb==1)=-1./ctr.delta;
    Vu2(MASKb==1)=-1./ctr.delta;
    Vu3(MASKb==1)=1./ctr.delta;
    S0(MASKb==1)=0;
    
    % i=1: periodic boundary condition
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(1,2:ctr.jmax)=1;
    V0(MASKb==1)=1;
    V9(MASKb==1)=1;
    S0(MASKb==1)=0;
    
    % i=imax-1: periodic boundary condition
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(ctr.imax-1,2:ctr.jmax)=1;
    V0(MASKb==1)=1;
    V10(MASKb==1)=1;
    S0(MASKb==1)=0;
    
    % i=imax;
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(ctr.imax,:)=1;
    V0(MASKb==1)=1;
    S0(MASKb==1)=NaN;
    
else
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(1:ctr.imax-1,1)=1;
    MASKb(1:ctr.imax-1,ctr.jmax)=1;
    MASKb(1,:)=1;
    MASKb(ctr.imax-1,:)=1;
    V0(MASKb==1)=1;
    S0(MASKb==1)=0;
    
    MASKb=zeros(ctr.imax,ctr.jmax); % boundary mask
    MASKb(ctr.imax,:)=1;
    V0(MASKb==1)=1;
    S0(MASKb==1)=NaN;
end

nodes=ctr.imax*ctr.jmax;
V=[reshape(U0,nodes,1)
    reshape(V0,nodes,1)
    U1(U1~=0)
    U2(U2~=0)
    U3(U3~=0)
    U4(U4~=0)
    U5(U5~=0)
    U6(U6~=0)
    U7(U7~=0)
    U8(U8~=0)
    U9(U9~=0)
    U10(U10~=0)
    Uv0(Uv0~=0)
    Uv1(Uv1~=0)
    Uv2(Uv2~=0)
    Uv3(Uv3~=0)
    V1(V1~=0)
    V2(V2~=0)
    V3(V3~=0)
    V4(V4~=0)
    V5(V5~=0)
    V6(V6~=0)
    V7(V7~=0)
    V8(V8~=0)
    V9(V9~=0)
    V10(V10~=0)
    V11(V11~=0)
    Vu0(Vu0~=0)
    Vu1(Vu1~=0)
    Vu2(Vu2~=0)
    Vu3(Vu3~=0)
  ];

row=[reshape(nodeu,nodes,1)
    reshape(nodev,nodes,1)
    nodeu(U1~=0)
    nodeu(U2~=0)
    nodeu(U3~=0)
    nodeu(U4~=0)
    nodeu(U5~=0)
    nodeu(U6~=0)
    nodeu(U7~=0)
    nodeu(U8~=0)
    nodeu(U9~=0)
    nodeu(U10~=0)
    nodeu(Uv0~=0)
    nodeu(Uv1~=0)
    nodeu(Uv2~=0)
    nodeu(Uv3~=0)
    nodev(V1~=0)
    nodev(V2~=0)
    nodev(V3~=0)
    nodev(V4~=0)
    nodev(V5~=0)
    nodev(V6~=0)
    nodev(V7~=0)
    nodev(V8~=0)
    nodev(V9~=0)
    nodev(V10~=0)
    nodev(V11~=0)
    nodev(Vu0~=0)
    nodev(Vu1~=0)
    nodev(Vu2~=0)
    nodev(Vu3~=0)
  ];
nodeU1=circshift(nodeu,[0 -1]); %i,j+1
nodeU2=circshift(nodeu,[0 1]); %i,j-1
nodeU3=circshift(nodeu,[-1 0]); %i+1,j
nodeU4=circshift(nodeu,[1 0]); %i-1,j
nodeU5=circshift(nodeu,[-1 -1]); %i+1,j+1
nodeU6=circshift(nodeu,[-1 1]); %i+1,j-1
nodeU7=circshift(nodeu,[1 -1]); %i-1,j+1
nodeU8=circshift(nodeu,[1 1]); %i-1,j-1
nodeU9=circshift(nodeu,[2 0]); % periodic BC i=1
nodeU10=circshift(nodeu,[-2 0]); %periodic BC i=imax
nodeUv1=circshift(nodev,[0 -1]); %i,j+1
nodeUv2=circshift(nodev,[1 0]); %i-1,j
nodeUv3=circshift(nodev,[1 -1]); %i-1,j+1

nodeV1=circshift(nodev,[-1 0]); %i+1,j
nodeV2=circshift(nodev,[1 0]); %i-1,j
nodeV3=circshift(nodev,[0 -1]); %i,j+1
nodeV4=circshift(nodev,[0 1]); %i,j-1
nodeV5=circshift(nodev,[-1 -1]); %i+1,j+1
nodeV6=circshift(nodev,[1 -1]); %i-1,j+1
nodeV7=circshift(nodev,[-1 1]); %i+1,j-1
nodeV8=circshift(nodev,[1 1]); %i-1,j-1
nodeV9=circshift(nodev,[3 0]); % periodic BC i=1
nodeV10=circshift(nodev,[-3 0]); %periodic BC i=imax
nodeV11=circshift(nodev,[0 -2]); % ice divide
nodeVu1=circshift(nodeu,[-1 0]); %i+1,j
nodeVu2=circshift(nodeu,[0 1]); %i,j-1
nodeVu3=circshift(nodeu,[-1 1]); %i+1,j-1

col=[reshape(nodeu,nodes,1)
    reshape(nodev,nodes,1)
    nodeU1(U1~=0)
    nodeU2(U2~=0)
    nodeU3(U3~=0)
    nodeU4(U4~=0)
    nodeU5(U5~=0)
    nodeU6(U6~=0)
    nodeU7(U7~=0)
    nodeU8(U8~=0)
    nodeU9(U9~=0)
    nodeU10(U10~=0)
    nodev(Uv0~=0)
    nodeUv1(Uv1~=0)
    nodeUv2(Uv2~=0)
    nodeUv3(Uv3~=0)
    nodeV1(V1~=0)
    nodeV2(V2~=0)
    nodeV3(V3~=0)
    nodeV4(V4~=0)
    nodeV5(V5~=0)
    nodeV6(V6~=0)
    nodeV7(V7~=0)
    nodeV8(V8~=0)
    nodeV9(V9~=0)
    nodeV10(V10~=0)
    nodeV11(V11~=0)
    nodeu(Vu0~=0)
    nodeVu1(Vu1~=0)
    nodeVu2(Vu2~=0)
    nodeVu3(Vu3~=0)
  ];

% R=[reshape(R0,nodes,1)
%     reshape(S0,nodes,1)];
R(nodeu)=R0;
R(nodev)=S0;
R=R';
R(isnan(R))=0;

% construct sparse matrix
A=sparse(row,col,V);
    
% Cholesky factor and solve
tol=1e-3;
maxit=50;
D=diag(diag(A));
C1=tril(A);
C2=D\triu(A);
[s,flag]=bicgstab(A,R,tol,maxit,C1,C2,s0);
if flag>0
    s=A\R;
end
u=s(nodeu);
v=s(nodev);

end


function [he,fi]=DefineEdgeThickness(ctr,glMASK,H)
    
% Define ice thickness at the shelf edges used for calving.
% Based on Pollard et al (2015) and Pollard and DeConto (2012)
% Use of maximum thickness of surrounding points and not mean H

    IC=ones(ctr.imax,ctr.jmax);
    IC(glMASK>=5)=0; % only cells grounded/floated and not adjacent to ocean
    IC1=circshift(IC,[-1 0]);
    IC2=circshift(IC,[1 0]);
    IC3=circshift(IC,[0 1]);
    IC4=circshift(IC,[0 -1]);
    H1=circshift(H,[-1 0]);
    H2=circshift(H,[1 0]);
    H3=circshift(H,[0 1]);
    H4=circshift(H,[0 -1]);
    w1=1-min(1,H./(H1*exp(-ctr.delta/1e5)))*(1-exp(-ctr.delta/1e5));
    w2=1-min(1,H./(H2*exp(-ctr.delta/1e5)))*(1-exp(-ctr.delta/1e5));
    w3=1-min(1,H./(H3*exp(-ctr.delta/1e5)))*(1-exp(-ctr.delta/1e5));
    w4=1-min(1,H./(H4*exp(-ctr.delta/1e5)))*(1-exp(-ctr.delta/1e5));
    he=(H1.*w1.*IC1+H2.*w2.*IC2+H3.*w3.*IC3+H4.*w4.*IC4)./ ...
        (IC1+IC2+IC3+IC4);
    he(isnan(he))=H(isnan(he)); % no neighbouring ice-cells (iceberg)
    fi=min(1,H./he);
    fi(glMASK~=5)=1;
    fi(glMASK==6)=0;
end



function [eta,dudx,dvdy]=EffVisc(Axy,uxs,uys,H,par,eta1,MASK,ctr)

% Effective viscosity of the SSA solution. On the borders of the domain, a
% fixed value is determined based on a fixed ice thickness (Hshelf) that
% determines eta1

    dudx=(uxs-circshift(uxs,[0 1]))/ctr.delta;
    dvdy=(uys-circshift(uys,[1 0]))/ctr.delta;
    dudy=(circshift(uxs,[-1 1])+circshift(uxs,[-1 0])- ...
        circshift(uxs,[1 1])-circshift(uxs,[1 0]))/(4*ctr.delta);
    dvdx=(circshift(uys,[0 -1])+circshift(uys,[1 -1])- ...
        circshift(uys,[0 1])-circshift(uys,[1 1]))/(4*ctr.delta);
    eta=0.5*H.*Axy.^(-1./par.n).*(dudx.^2+dvdy.^2+dudx.*dvdy+ ...
        0.25*(dudy+dvdx).^2).^((1-par.n)/(2*par.n));
    eta(eta<=0)=NaN;
%     eta(isnan(eta))=eta1(isnan(eta));
    eta(MASK==0)=eta(MASK==0)./ctr.shelftune(MASK==0);
    eta(isnan(eta))=1e7;
    
    % NEED TO CHECK THE VISCOSITY ON EDGES OF ICE SHELF/SEA ICE!!!
    % NOW TAKEN AS A CONSTANT VALUE FOR STABILITY AS MOSTLY SEA ICE
    
    if ctr.mismip==1
        eta(:,1)=eta(:,3); % ice divide - symmetric
        eta(1,:)=eta(ctr.imax-1,:); % periodic BC
        eta(ctr.imax,:)=eta(2,:); % periodic BC
        MASKb=ones(ctr.imax,ctr.jmax); % use constant eta on edges of ice shelf
        MASKb(1:ctr.imax,1:ctr.jmax-2)=0;
        eta(MASKb==1 & MASK==0)=eta1(MASKb==1 & MASK==0);
    else
        MASKb=ones(ctr.imax,ctr.jmax); % use constant eta on edges of ice shelf
        MASKb(3:ctr.imax-2,3:ctr.jmax-2)=0;
        eta(MASKb==1 & MASK==0)=eta1(MASKb==1 & MASK==0);
    end
    eta=min(max(eta,1e5),1e15);
end

function [T0o,S0o,zb]=OceanVarBox(numsh,To,So,ShelfN,HB)

    T0o=zeros(numsh,1); % mean box T0
    S0o=zeros(numsh,1); % mean box S0
    zb=zeros(numsh,1); % mean box HB

    for k=1:numsh
        T0o(k)=max(To(ShelfN==k));
        S0o(k)=mean(So(ShelfN==k));
        zb(k)=mean(HB(ShelfN==k));
    end
end

function [Melt]=ShelfMelt(HB,shMASK,ShelfN,Ak,Bk,S0o,T0o,S,T, ...
    numsh,Bmax,ctr,nu,par)

% Extrapolate melt underneath each ice shelf based on the results for each
% ocean box
    
    % Results for local shelf system
    Melt=zeros(ctr.imax,ctr.jmax);
    HB(shMASK==0)=0;
    
    % Box I
    g1=Ak*ctr.gammaT;
    for i=1:numsh
        Tref=min(par.lambda1*S0o(i)+par.lambda2+par.lambda3*HB-T0o(i),0); % make sure Tref<=0
        dif=g1./(par.C*par.rhoref*(par.betao*S0o(i)/nu-par.alphao));
        x=-0.5*dif+0.5*sqrt(dif.^2-4*dif.*Tref);
        Tsh=T0o(i)-x; % T1 should be lower than T0
        Ssh=S0o(i)*(1-x/nu);
        m=-ctr.gammaT*(par.lambda1*Ssh+par.lambda2+par.lambda3*HB-Tsh)/nu*par.secperyear;
        Melt(Bk==1 & ShelfN==i)=m(Bk==1 & ShelfN==i);
    end
    
    % Box II to nbox
    for i=1:numsh
        q=par.C*par.rhoref*(par.betao*(S0o(i)-S(i,1))-par.alphao*(T0o(i)-T(i,1)));
        for k=2:Bmax(i)
            Tref=par.lambda1*S(i,k-1)+par.lambda2+par.lambda3*HB-T(i,k-1);
            x=-g1.*Tref./(q+g1*(1-par.lambda1*S(i,k-1)/nu));
            Tsh=T(i,k-1)-x;
            Ssh=S(i,k-1)*(1-x/nu);
            m=-ctr.gammaT*(par.lambda1*Ssh+par.lambda2+par.lambda3*HB-Tsh)/nu*par.secperyear;
            Melt(Bk==k & ShelfN==i)=m(Bk==k & ShelfN==i);
        end
    end
    
end

function [T,S]=BoxModel(S0o,T0o,numsh,par,Ak,Bk,ShelfN,HB,nu,ctr)

    % Initialization of boxes and ice shelves
    % numsh = num ice shelves; nbox = num boxes
    HBk=zeros(numsh,par.nbox); % mean ice thickness
    Ab=zeros(numsh,1); % surface of each box (all boxes within a shelf have equal size)
    T=zeros(numsh,par.nbox); % mean box temperature
    S=zeros(numsh,par.nbox); % mean box salinity
    for i=1:numsh
        Ab(i)=mean(Ak(ShelfN==i));
        for k=1:par.nbox
            HBk(i,k)=mean(HB(ShelfN==i & Bk==k));
        end
    end
    HBk(isnan(HBk))=0; % some shelves don't have all boxes - set HBk=0

    % Box I
    Tref=min(par.lambda1*S0o+par.lambda2+par.lambda3*HBk(:,1)-T0o,0); % make sure Tref<=0
    g1=Ab*ctr.gammaT;
    dif=g1./(par.C*par.rhoref*(par.betao*S0o/nu-par.alphao));
    x=(-0.5)*dif+0.5*sqrt(dif.^2-4*dif.*Tref);
    T(:,1)=T0o-x; % T1 should be lower than T0 in case of melting at GL
    S(:,1)=S0o.*(1-x/nu);

    % Box II to n
    q=par.C*par.rhoref*(par.betao*(S0o-S(:,1))-par.alphao*(T0o-T(:,1)));
    for i=2:par.nbox
        Tref=par.lambda1*S(:,i-1)+par.lambda2+par.lambda3*HBk(:,i)-T(:,i-1);
        x=-g1.*Tref./(q+g1.*(1-par.lambda1*S(:,i-1)/nu));
        T(:,i)=T(:,i-1)-x;
        S(:,i)=S(:,i-1).*(1-x/nu);
    end
end

% LZ: function PICOsetup changed to seperate shelf numbering from PICOsetup
function [Bk,Ak,Bmax]=PICOsetup(glMASK,H,ctr,par,ShelfN,numsh,shMASK)

% Setup of the PICO model to determine each ice shelf (ShelfN), its area
% (Ak) and the ocean boxes underneath each shelf (Bk)

    % Add first ocean point in glMASK
    MASK1=circshift(glMASK,[0 -1]); % glMASK(i,j+1)
    MASK2=circshift(glMASK,[0 1]); % glMASK(i,j-1)
    MASK3=circshift(glMASK,[-1 0]); % glMASK(i+1,j)
    MASK4=circshift(glMASK,[1 0]); % glMASK(i-1,j)
    glMASK(glMASK==6 & (MASK1==5 | MASK2==5 | ...
        MASK3==5 | MASK4==5))=7; % First floating point (=7)
        
    % relative distance to GL: gMASK
    gMASK=ones(ctr.imax,ctr.jmax)-2;
    gMASK(glMASK==2)=0; % start at GL position
    gMASK(glMASK==3 | glMASK==4 | glMASK==5 | glMASK==7)=-2;
    for i=0:1000
        sg=sum(sum(gMASK==-2));
        if i>0 && (sg-sg0)==0
            break;
        end
        MASK1=circshift(gMASK,[0 -1]); % glMASK(i,j+1)
        MASK2=circshift(gMASK,[0 1]); % glMASK(i,j-1)
        MASK3=circshift(gMASK,[-1 0]); % glMASK(i+1,j)
        MASK4=circshift(gMASK,[1 0]); % glMASK(i-1,j)
        gMASK(gMASK==-2 & (MASK1==i | MASK2==i | MASK3==i | MASK4==i))=i+1;
        sg0=sg;
    end

    % relative distance from front: fMASK
    fMASK=ones(ctr.imax,ctr.jmax)-2;
    fMASK(glMASK==5)=0; % start at front position
    fMASK(glMASK==2 | glMASK==3 | glMASK==4)=-2;
    for i=0:1000
        sg=sum(sum(fMASK==-2));
        if i>0 && (sg-sg0)==0
            break;
        end
        MASK1=circshift(fMASK,[0 -1]); % glMASK(i,j+1)
        MASK2=circshift(fMASK,[0 1]); % glMASK(i,j-1)
        MASK3=circshift(fMASK,[-1 0]); % glMASK(i+1,j)
        MASK4=circshift(fMASK,[1 0]); % glMASK(i-1,j)
        fMASK(fMASK==-2 & (MASK1==i | MASK2==i | MASK3==i | MASK4==i))=i+1;
        sg0=sg;
    end

    gMASK(gMASK<0 | H<5*par.SeaIceThickness)=0;
    fMASK(fMASK<0 | H<5*par.SeaIceThickness)=0;
    gMASK=gMASK*ctr.delta;
    fMASK=fMASK*ctr.delta;

    % define boxes and relative distance r
    dmax=max(max(gMASK));
    rd=gMASK./(gMASK+fMASK); % relative distance between GL and front

    % Count number of ice shelves
    ShelfN=bwlabel(shMASK);
    
    % define number of boxes for different ice shelves
    nB=1+round(sqrt(gMASK/dmax)*(par.nbox-1));
    nD=zeros(ctr.imax,ctr.jmax);
    for i=1:numsh
        nD(ShelfN==i)=max(nB(ShelfN==i));
    end

    Bk=zeros(ctr.imax,ctr.jmax);
    for k=1:par.nbox
        LL=1-sqrt(abs((nD-k+1)./nD));
        UL=1-sqrt(abs((nD-k)./nD));
        Bk(rd>=LL & rd<=UL)=k;
        Bk(Bk>nD)=nD(Bk>nD);
    end
    Bk(glMASK==3)=1; % make sure that boxes near GL are always Box 1
    Bmax=zeros(numsh,1);
    for i=1:numsh
        Bmax(i)=max(Bk(ShelfN==i));
    end
    % Calculate surface of each box (Ak)
    Ak=shMASK; % size of each box within particular ice shelf
    for i=1:numsh
        Ak(ShelfN==i)=Ak(ShelfN==i)*sum(sum(ShelfN==i));
    end
    Bk(Ak==1)=1;
    Ak(nD>0)=Ak(nD>0)*ctr.delta^2./nD(nD>0);
end


% LZ: new function Shelfsetup to seperate shelf numbering from PICOsetup
function [ShelfN,numsh,shMASK,MASKlk]=Shelfsetup(MASK,glMASK,H,ctr,par)

    %LZ: find all 'lakes'
    [~,MASKlk]=bwboundaries(MASK,4);
    MASKlk(MASK==1)=0;
    MASKlk(MASKlk~=0)=1;

    shMASK=zeros(ctr.imax,ctr.jmax); % Mask of ice shelf
    shMASK(MASKlk==0 & (glMASK==3 | glMASK==4 | glMASK==5))=1;
    shMASK(H<5*par.SeaIceThickness)=0;
    
    %LZ: new function for calculation of ShelfN
    ShelfN=bwlabel(shMASK);
    numsh=max(ShelfN(:));

end


function Melt=PlumeMelt(HB,vx,vy,Zgl,To,So,par,ctr)
    
    % local basal slope alfa
    s1=circshift(HB,[-1 0]); % i+1,j
    s2=circshift(HB,[1 0]); % i-1,j
    s3=circshift(HB,[0 -1]); % i,j+1
    s4=circshift(HB,[0 1]); % i,j-1
    MV=ones(ctr.imax,ctr.jmax); % u>=1, v>=1
    MV(vx>=0 & vy<0)=2;
    MV(vx<0 & vy>=0)=3;
    MV(vx<0 & vy<0)=4;
    sina=sin(sqrt(((s4-HB)/ctr.delta).^2+((s2-HB)/ctr.delta).^2));
    sina(MV==2)=sin(sqrt(((s4(MV==2)-HB(MV==2))/ctr.delta).^2+ ...
        ((s1(MV==2)-HB(MV==2))/ctr.delta).^2));
    sina(MV==3)=sin(sqrt(((s3(MV==3)-HB(MV==3))/ctr.delta).^2+ ...
        ((s2(MV==3)-HB(MV==3))/ctr.delta).^2));
    sina(MV==4)=sin(sqrt(((s3(MV==4)-HB(MV==4))/ctr.delta).^2+ ...
        ((s1(MV==4)-HB(MV==4))/ctr.delta).^2));
    sina=min(sina,0.05); % limit on slope of ice shelf
%     sina=sin(sqrt(((s1-s2)/(2*ctr.delta)).^2+((s3-s4)/(2*ctr.delta)).^2));

    Tfgl=par.lambda1*So+par.lambda2+par.lambda3*Zgl;
    CdGamTS=par.CdGamT*(par.gamma1+(par.gamma2*(To-Tfgl)/par.lambda3).*(par.Eo*sina./ ...
        (par.CdGamTS0+par.Eo*sina)));
    galfa=((sina./(par.Cd+par.Eo*sina)).^(0.5)).*((CdGamTS./(CdGamTS+par.Eo*sina)).^(0.5)).* ...
        (par.Eo*sina./(CdGamTS+par.Eo*sina));
    len=(To-Tfgl)/par.lambda3.*(par.x0*CdGamTS+par.Eo*sina)./(par.x0*(CdGamTS+par.Eo*sina));
    Xhat=(HB-Zgl)./len;
    Xhat=min(Xhat,1);
    Melt=ctr.M0*MscaledPoly(Xhat,par.pcof).*galfa.*(To-Tfgl).^2;
    
end


function Melt=PICOPmelt(HB,vx,vy,Zgl,Bk,ShelfN,T,S,par,ctr,Bmax,numsh)
    
    % local basal slope alfa
    s1=circshift(HB,[-1 0]); % i+1,j
    s2=circshift(HB,[1 0]); % i-1,j
    s3=circshift(HB,[0 -1]); % i,j+1
    s4=circshift(HB,[0 1]); % i,j-1
    MV=ones(ctr.imax,ctr.jmax); % u>=1, v>=1
    MV(vx>=0 & vy<0)=2;
    MV(vx<0 & vy>=0)=3;
    MV(vx<0 & vy<0)=4;
    sina=sin(sqrt(((s4-HB)/ctr.delta).^2+((s2-HB)/ctr.delta).^2));
    sina(MV==2)=sin(sqrt(((s4(MV==2)-HB(MV==2))/ctr.delta).^2+ ...
        ((s1(MV==2)-HB(MV==2))/ctr.delta).^2));
    sina(MV==3)=sin(sqrt(((s3(MV==3)-HB(MV==3))/ctr.delta).^2+ ...
        ((s2(MV==3)-HB(MV==3))/ctr.delta).^2));
    sina(MV==4)=sin(sqrt(((s3(MV==4)-HB(MV==4))/ctr.delta).^2+ ...
        ((s1(MV==4)-HB(MV==4))/ctr.delta).^2));
    sina=min(sina,0.05); % limit on slope of ice shelf
%     sina=sin(sqrt(((s1-s2)/(2*ctr.delta)).^2+((s3-s4)/(2*ctr.delta)).^2));

    % Define Ta, Sa for all ice shelves based on Box model
    Ta=zeros(ctr.imax,ctr.jmax)+par.Toi;
    Sa=zeros(ctr.imax,ctr.jmax)+par.Soi;
    for i=1:numsh
        for k=1:Bmax(i)
            Ta(Bk==k & ShelfN==i)=T(i,k);
            Sa(Bk==k & ShelfN==i)=S(i,k);
        end
    end
    % Limit on Ta
    Ta=max(Ta,par.lambda1*Sa+par.lambda2+par.lambda3*HB);
    
    Tfgl=par.lambda1*Sa+par.lambda2+par.lambda3*Zgl;
    CdGamTS=par.CdGamT*(par.gamma1+(par.gamma2*(Ta-Tfgl)/par.lambda3).*(par.Eo*sina./ ...
        (par.CdGamTS0+par.Eo*sina)));
    galfa=((sina./(par.Cd+par.Eo*sina)).^(0.5)).*((CdGamTS./(CdGamTS+par.Eo*sina)).^(0.5)).* ...
        (par.Eo*sina./(CdGamTS+par.Eo*sina));
    len=(Ta-Tfgl)/par.lambda3.*(par.x0*CdGamTS+par.Eo*sina)./(par.x0*(CdGamTS+par.Eo*sina));
    Xhat=(HB-Zgl)./len;
    Xhat=min(Xhat,1);
    Melt=ctr.M0*MscaledPoly(Xhat,par.pcof).*galfa.*(Ta-Tfgl).^2;
    
end

    
function [m]=MscaledPoly(x,p)

    m=zeros(size(x));
    for i=1:12
        m=m+p(i)*x.^(i-1);
    end
    
end


function [Zgl]=AdvecGL(HB,B,MASK,MASKpicop,MASKb,vx,vy,ctr)

% Advecting the grounding line thickness following Pelle et al. (2018)
% Employs upstream difference scheme

    epsilon=1e-5; % diffusion coefficient for stabilizing advection scheme

    VM=reshape(MASKpicop,ctr.imax*ctr.jmax,1);
    nodes=sum(sum(MASKpicop));
    node=zeros(ctr.imax,ctr.jmax);
    node(MASKpicop>0)=linspace(1,nodes,nodes);

    Zgl=HB;
    Zgl=min(Zgl,0);
    Zgl(MASK==0)=0;

    dtdx2=epsilon/(ctr.delta*ctr.delta);
    u1=circshift(vx,[0 1]); % vx(i,j-1)
    v1=circshift(vy,[1 0]); % vy(i-1,j)

    % Velocity sign masks
    MV=ones(ctr.imax,ctr.jmax); % u>=1, v>=1
    MV(vx>=0 & vy<0)=2;
    MV(vx<0 & vy>=0)=3;
    MV(vx<0 & vy<0)=4;
    % conditions for (u,v)>=0 (applied to whole matrix)
    V0=4*dtdx2+u1/ctr.delta+v1/ctr.delta; % i,j
    V1=zeros(ctr.imax,ctr.jmax)-dtdx2; % i,j+1
    V2=-u1/ctr.delta-dtdx2; % i,j-1
    V3=zeros(ctr.imax,ctr.jmax)-dtdx2; % i+1,j
    V4=-v1/ctr.delta-dtdx2; % i-1,j
    % conditions for u>=0 and v<0
    V0(MV==2)=4*dtdx2+u1(MV==2)/ctr.delta-vy(MV==2)/ctr.delta; % i,j
    V3(MV==2)=vy(MV==2)/ctr.delta-dtdx2; % i+1,j
    V4(MV==2)=-dtdx2; % i-1,j
    % conditions for u<0 and v>=0
    V0(MV==3)=4*dtdx2-vx(MV==3)/ctr.delta+v1(MV==3)/ctr.delta; % i,j
    V1(MV==3)=vx(MV==3)/ctr.delta-dtdx2; % i,j+1
    V2(MV==3)=-dtdx2; % i,j-1
    % conditions for u<0 and v<0
    V0(MV==4)=4*dtdx2-vx(MV==4)/ctr.delta-vy(MV==4)/ctr.delta;
    V1(MV==4)=vx(MV==4)/ctr.delta-dtdx2; % i,j+1
    V2(MV==4)=-dtdx2; % i,j-1
    V3(MV==4)=vy(MV==4)/ctr.delta-dtdx2; % i+1,j
    V4(MV==4)=-dtdx2; % i-1,j

    V0(MASKpicop==0)=0;
    V1(MASKpicop==0)=0;
    V2(MASKpicop==0)=0;
    V3(MASKpicop==0)=0;
    V4(MASKpicop==0)=0;

    R0=zeros(ctr.imax,ctr.jmax);

    V0(MASKb==1)=1;
    V1(MASKb==1)=0;
    V2(MASKb==1)=0;
    V3(MASKb==1)=0;
    V4(MASKb==1)=0;
    R0(MASKb==1)=Zgl(MASKb==1);

    V=[reshape(V0(VM==1),nodes,1)
        V1(V1~=0)
        V2(V2~=0)
        V3(V3~=0)
        V4(V4~=0)];

    row=[reshape(node(VM==1),nodes,1)
        node(V1~=0)
        node(V2~=0)
        node(V3~=0)
        node(V4~=0)];

    nodeV1=circshift(node,[0 -1]); % i,j+1
    nodeV2=circshift(node,[0 1]); % i,j-1
    nodeV3=circshift(node,[-1 0]); % i+1,j
    nodeV4=circshift(node,[1 0]); % i-1,j

    col=[reshape(node(VM==1),nodes,1)
    nodeV1(V1~=0)
    nodeV2(V2~=0)
    nodeV3(V3~=0)
    nodeV4(V4~=0)];

    R=reshape(R0(VM==1),nodes,1);

    % construct sparse matrix
    A=sparse(row,col,V);
    % Cholesky factor and solve
    s=A\R;
    Zgl(node>0)=s(node(node>0));
    Zgl(Zgl>HB)=HB(Zgl>HB);
    Zgl(Zgl<B)=B(Zgl<B);
    Zgl=min(Zgl,0);
end


function [MASKpicop,MASKb]=PICOPmasks(MASK,MASKlk,H,ctr,par)

% Defines the MASK for PICOP domain (ice shelf + grounding line + first
% ocean grid point) and the boundary (grounding line and first ocean
% grid point)

    MASKpicop=zeros(ctr.imax,ctr.jmax); % initialize
    MASKb=zeros(ctr.imax,ctr.jmax); % ice shelf boundary points
    MASK=MASK+MASKlk; % FP: make sure that lakes are removed
    % grounding line (=2)
    MASK1=circshift(MASK,[0 -1]);
    MASK2=circshift(MASK,[0 1]);
    MASK3=circshift(MASK,[-1 0]);
    MASK4=circshift(MASK,[1 0]);
    MASKpicop(MASK==1 & (MASK1==0 | MASK2==0 | MASK3==0 | MASK4==0))=1;
    MASKb(MASK==1 & (MASK1==0 | MASK2==0 | MASK3==0 | MASK4==0))=1;
    % ice shelves (=1)
    MASKpicop(MASK==0 & H>par.SeaIceThickness)=1;
    % adjacent floating grid cell
    MASK1=circshift(MASKpicop,[0 -1]);
    MASK2=circshift(MASKpicop,[0 1]);
    MASK3=circshift(MASKpicop,[-1 0]);
    MASK4=circshift(MASKpicop,[1 0]);
    MASKb(MASKpicop==0 & MASK==0 & (MASK1==1 | MASK2==1 | MASK3==1 | MASK4==1))=1;
    MASKpicop(MASKpicop==0 & MASK==0 & (MASK1==1 | MASK2==1 | MASK3==1 | MASK4==1))=1;

end


function [flw,Wd]=SubWaterFlux(ctr,par,H,HB,Hm,MASKm,MASK,Bmelt)

% Steady-state subglacial water flow model
% Method due to LeBrocq (GDS Warner model)
% Simplified implementation for ice thickness classes as a function of
% the distance over hich hydraulic gradient coupling takes place

flux=zeros(ctr.imax,ctr.jmax)-1;
pot=par.rho*par.g*H+par.rhof*par.g*HB;
pot(MASK==0)=0;
[pot]=potential_filling(pot,ctr);
% Correction of ice thickness due to filling of 'pot'
H=(pot-par.rhof*par.g*HB)/(par.rho*par.g);
% Hydraulic potential gradients on staggered d-grid
gdsx=-par.rho*par.g*(circshift(H,[0 -1])+circshift(H,[-1 -1])-H- ...
    circshift(H,[-1 0]))/(2*ctr.delta)-par.rhof*par.g*(circshift(HB,[0 -1])+ ...
    circshift(HB,[-1 -1])-HB-circshift(HB,[-1 0]))/(2*ctr.delta);
gdsy=-par.rho*par.g*(circshift(H,[-1 0])+circshift(H,[-1 -1])-H- ...
    circshift(H,[0 -1]))/(2*ctr.delta)-par.rhof*par.g*(circshift(HB,[-1 0])+ ...
    circshift(HB,[-1 -1])-HB-circshift(HB,[0 -1]))/(2*ctr.delta);

% define multiplier for GDS
Hlev=mean(Hm(MASK==1));
scale=Hlev*par.longcoupwater*2.; % scale of GDS window
width=2.*scale;
scale(width<=ctr.delta)=ctr.delta/2+1;
maxlevel=2*round(width/ctr.delta-0.5)+1;
frb=(maxlevel-1)/2;
[ew,ns]=meshgrid(1:maxlevel,1:maxlevel); % size of GDS window

kegdsx0=zeros(ctr.imax+2*frb,ctr.jmax+2*frb);
kegdsy0=zeros(ctr.imax+2*frb,ctr.jmax+2*frb);
kegdsx0(frb+1:frb+ctr.imax,frb+1:frb+ctr.jmax)=gdsx;
kegdsy0(frb+1:frb+ctr.imax,frb+1:frb+ctr.jmax)=gdsy;

% Define mulipliers as a function of ice thickness (scale) and convolute
dist=sqrt((ctr.delta*(ew-frb-1)).^2+(ctr.delta*(ns-frb-1)).^2)/scale;
multiplier=max(0.,1.-dist/2.);
multiplier=multiplier/sum(sum((multiplier)));
kegx=xcorr2(kegdsx0,multiplier);
kegy=xcorr2(kegdsy0,multiplier);
kegdsx=kegx(2*frb+1:ctr.imax+2*frb,2*frb+1:ctr.jmax+2*frb);
kegdsy=kegy(2*frb+1:ctr.imax+2*frb,2*frb+1:ctr.jmax+2*frb);

% Recursive algorithm
gdsmag=abs(kegdsx)+abs(kegdsy);
for i=1:ctr.imax-1
    for j=1:ctr.jmax-1
        if MASKm(i,j)==1
            flux=dparea_war_gds(i,j,MASKm,Bmelt, ...
                kegdsx,kegdsy,par,ctr,flux,gdsmag,1);
        end
    end
end

corfac=zeros(ctr.imax,ctr.jmax)+1;
denom=sqrt(kegdsx.^2+kegdsy.^2);
corfac(gdsmag>0)=gdsmag(gdsmag>0)./denom(gdsmag>0);
flw=min(max(flux./corfac/ctr.delta,0),1e5);
flw(MASKm==0)=0;
Wd=min(par.Wdmax,max(par.Wdmin,(12*par.waterviscosity*flw/mean(gdsmag(MASKm==1))).^(1/3)));
end


function [pot]=potential_filling(pot,ctr)

% Hollow filling for potential gradient

for k=1:10
    pool=zeros(ctr.imax,ctr.jmax);
    p1=circshift(pot,[0 -1]);
    p2=circshift(pot,[0 1]);
    p3=circshift(pot,[-1 0]);
    p4=circshift(pot,[1 0]);
    pool(pot<p1 & pot<p2 & pot<p3 & pot<p4)=1;
    pot(pool==1)=(p1(pool==1)+p2(pool==1)+p3(pool==1)+p4(pool==1))/4;
end

end


function flux=dparea_war_gds(ewr,nsr,MASK,Bmelt,kegdsx,kegdsy, ...
    par,ctr,flux,gdsmag,funcnt)

    if MASK(ewr,nsr)>=1 && flux(ewr,nsr)<0.
        flux(ewr,nsr)=Bmelt(ewr,nsr)*ctr.delta^2;
        count=1;
        for j=nsr-1:nsr+1
            for i=ewr-1:ewr+1
                pp=zeros(9,1);
                if count~=5
                    if i>=1 && i<ctr.imax && j>=1 && j<ctr.jmax
                        if kegdsy(i,j)>0.
                            pp(6)=kegdsy(i,j)/gdsmag(i,j);
                        elseif kegdsy(i,j)<0.
                            pp(4)=abs(kegdsy(i,j))/gdsmag(i,j);
                        end
                        if kegdsx(i,j)<0.
                            pp(2)=abs(kegdsx(i,j))/gdsmag(i,j);
                        elseif kegdsx(i,j)>0.
                            pp(8)=kegdsx(i,j)/gdsmag(i,j);
                        end
                        if pp(par.dirpp_war(count))>0 && funcnt<=5e4
                            funcnt=funcnt+1;
                            flux=dparea_war_gds(i,j,MASK,Bmelt,kegdsx,kegdsy, ...
                                par,ctr,flux,gdsmag,funcnt);
                            flux(ewr,nsr)=flux(ewr,nsr)+ ...
                                pp(par.dirpp_war(count))*flux(i,j);
                        end
                    end
                end
                count=count+1;
            end
        end
    end
end

function Wd=extrapolateWaterFlux(MASK,oldMASK,Wd,Wd0,ctr,par)

% Interpolate Wd between consecutive calls (to improve
% calculation speed

    dMASK=MASK-oldMASK; % difference between two consecutive
                        % masks (grounded - floated)
    WD=zeros(ctr.imax,ctr.jmax,8);
    OldWD=Wd;
    OldWD(MASK==0)=NaN;
    WD(:,:,1)=circshift(OldWD,[-1 1]);
    WD(:,:,2)=circshift(OldWD,[0 1]);
    WD(:,:,3)=circshift(OldWD,[1 1]);
    WD(:,:,4)=circshift(OldWD,[1 0]);
    WD(:,:,5)=circshift(OldWD,[-1 0]);
    WD(:,:,6)=circshift(OldWD,[-1 -1]);
    WD(:,:,7)=circshift(OldWD,[0 -1]);
    WD(:,:,8)=circshift(OldWD,[1 -1]);
    NewWD=nanmean(WD,3); % Take mean of neighbours for which Wd exists
    NewWD(isnan(NewWD))=par.Wdmin;
    Wd(dMASK==1)=NewWD(dMASK==1); % Apply depth for points becoming grounded
    % Apply flw depths for points that were previously determined
    %   with the subglacial water flow model
    Wd(dMASK==1 & Wd0~=par.Wdmin)=Wd0(dMASK==1 & Wd0~=par.Wdmin);
    Wd(dMASK==-1)=par.Wdmin; % set minimum Wd value for points becoming floated

end


function [Ntil,Wtil]=TillWater(MASKm,Wtil,Bmelt,ctr,Po,par)

Wtil=max(par.Wdmin,min(Wtil+(Bmelt-par.Cdr)*ctr.dt,par.Wmax));
s=Wtil/par.Wmax;
s(MASKm==0)=1;
Ntil=par.N0*((par.sigmat*Po/par.N0).^s).*10.^((par.e0/par.Cc).*(1.-s));
Ntil=max(min(Po,Ntil),par.sigmat*Po);

end


%--------------------------------------
% Solving ice sheet equation
%--------------------------------------
function H=sparse_solver_new(node,nodes,Mb,H,HB,MASK,dtdx,dtdx2, ...
    d,u,v,ctr,cnt,MASKinv,VM,par)

d=min(d,1e20); % FP: limit due to NaNs in initialization for high resolution

d1=circshift(d,[0 1]); % d(i,j-1)
d2=circshift(d,[1 0]); % d(i-1,j)
d3=circshift(d,[1 1]); % d(i-1,j-1)
um1=circshift(u,[0 1]); % u(i,j-1)
vm1=circshift(v,[1 0]); % v(i-1,j)
if par.upstream==1
    up1=circshift(u,[0 -1]); % u(i,j+1)
    vp1=circshift(v,[-1 0]); % v(i+1,j)
    um2=circshift(u,[0 2]); % u(i,j-2)
    vm2=circshift(v,[2 0]); % v(i-2,j)
end

dipx=dtdx*(d+d2);
dimx=dtdx*(d1+d3);
dipy=dtdx*(d+d1);
dimy=dtdx*(d2+d3);
    
if par.upstream==1
    % conditions for diffusion scheme (init)
    V0=2*(d+d1+d2+d3)*dtdx; % i,j
    V1=-dipx; % i,j+1
    V2=-dimx; % i,j-1
    V3=-dipy; % i+1,j
    V4=-dimy; % i-1,j
    
    % Velocity sign masks
    MV=ones(ctr.imax,ctr.jmax); % u>=1, v>=1
    MV(u>=0 & v<0)=2;
    MV(u<0 & v>=0)=3;
    MV(u<0 & v<0)=4;
    
    % conditions for (u,v)>=0 (applied to whole matrix)
    V0a=2*u+um1+2*v+vm1; % i,j
    V1a=zeros(ctr.imax,ctr.jmax); % i,j+1
    V2a=(-3*um1-um2); % i,j-1
    V3a=zeros(ctr.imax,ctr.jmax); % i+1,j
    V4a=(-3*vm1-vm2); % i-1,j
    V5=um2; % i,j-2
    V6=vm2; % i-2,j
    V7=zeros(ctr.imax,ctr.jmax); % i,j+2
    V8=zeros(ctr.imax,ctr.jmax); % i+2,j
    
    % conditions for u>=0 and v<0
    V0a(MV==2)=2*u(MV==2)+um1(MV==2)-v(MV==2)-2*vm1(MV==2); % i,j
    V3a(MV==2)=3*v(MV==2)+vp1(MV==2); % i+1,j
    V4a(MV==2)=0; % i-1,j
    V6(MV==2)=0; % i-2,j
    V8(MV==2)=(-vp1(MV==2)); % i+2,j
    % conditions for u<0 and v>=0
    V0a(MV==3)=(-u(MV==3)-2*um1(MV==3)+2*v(MV==3)+vm1(MV==3)); % i,j
    V1a(MV==3)=3*u(MV==3)+up1(MV==3); % i,j+1
    V2a(MV==3)=0; % i,j-1
    V5(MV==3)=0; % i,j-2
    V7(MV==3)=(-up1(MV==3)); % i,j+2
    % conditions for u<0 and v<0
    V0a(MV==4)=(-u(MV==4)-2*um1(MV==4)-v(MV==4)-2*vm1(MV==4));
    V1a(MV==4)=3*u(MV==4)+up1(MV==4); % i,j+1
    V2a(MV==4)=0; % i,j-1
    V3a(MV==4)=3*v(MV==4)+vp1(MV==4); % i+1,j
    V4a(MV==4)=0; % i-1,j
    V5(MV==4)=0; % i,j-2
    V6(MV==4)=0; % i-2,j
    V7(MV==4)=(-up1(MV==4)); % i,j+2
    V8(MV==4)=(-vp1(MV==4)); % i+2,j
    
    % Filling V-matrix
    V0=V0+V0a*dtdx2;%.*(1.-alfa);
    V1=V1+V1a*dtdx2;%.*(1.-alfa);
    V2=V2+V2a*dtdx2;%.*(1.-alfa);
    V3=V3+V3a*dtdx2;%.*(1.-alfa);
    V4=V4+V4a*dtdx2;%.*(1.-alfa);
    V5=V5*dtdx2;%.*(1.-alfa);
    V6=V6*dtdx2;%.*(1.-alfa);
    V7=V7*dtdx2;%.*(1.-alfa);
    V8=V8*dtdx2;%.*(1.-alfa);
else
    V0=2*(d+d1+d2+d3)*dtdx+dtdx2*(u-um1+v-vm1); % i,j
    V1=-dipx+dtdx2*u; % i,j+1
    V2=-dimx-dtdx2*um1; % i,j-1
    V3=-dipy+dtdx2*v; % i+1,j
    V4=-dimy-dtdx2*vm1; % i-1,j
end

R0=Mb*ctr.dt+H-H*(1-par.omega).*V0-circshift(H,[0 -1])*(1-par.omega).*V1- ...
    circshift(H,[0 1])*(1-par.omega).*V2-circshift(H,[-1 0])*(1-par.omega).*V3- ...
    circshift(H,[1 0])*(1-par.omega).*V4+ ...
    dipx.*(circshift(HB,[0 -1])-HB)-dimx.*(HB-circshift(HB,[0 1])) ...
    +dipy.*(circshift(HB,[-1 0])-HB)-dimy.*(HB-circshift(HB,[1 0]));
if par.upstream==1
    R0=R0-circshift(H,[0 2])*(1-par.omega).*V5-circshift(H,[2 0])*(1-par.omega).*V6- ...
        circshift(H,[0 -2])*(1-par.omega).*V7-circshift(H,[-2 0])*(1-par.omega).*V8;
end

if ctr.inverse==1
    V0(MASKinv==1 & MASK==0)=0;
    V1(MASK==0)=0;
    V2(MASK==0)=0;
    V3(MASK==0)=0;
    V4(MASK==0)=0;
    if par.upstream==1
        V5(MASK==0)=0;
        V6(MASK==0)=0;
        V7(MASK==0)=0;
        V8(MASK==0)=0;
    end
    if ctr.schoof>=1 && ctr.inverse~=1 % FP original inverse==0
        R0(MASKinv==1 & MASK==0)=0;
    else
        R0(MASKinv==1 & MASK==0)=H(MASKinv==1 & MASK==0);
    end
else
    V0(MASK==0)=0;
    V1(MASK==0)=0;
    V2(MASK==0)=0;
    V3(MASK==0)=0;
    V4(MASK==0)=0;
    if par.upstream==1
        V5(MASK==0)=0;
        V6(MASK==0)=0;
        V7(MASK==0)=0;
        V8(MASK==0)=0;
    end
    if ctr.schoof>=1 && ctr.inverse==0
        R0(MASK==0)=0;
    else
        R0(MASK==0)=H(MASK==0);
    end

    % boundaries
    V9=zeros(ctr.imax,ctr.jmax); % ice divide or ocean
    V10=zeros(ctr.imax,ctr.jmax); % i=1 periodic boundary or ocean
    V11=zeros(ctr.imax,ctr.jmax); % i=imax periodic boundary or ocean
    V12=zeros(ctr.imax,ctr.jmax); % j=jmax ocean contact
    
    MASKb=zeros(ctr.imax,ctr.jmax);
    MASKb(:,1)=1; % symmetric divide or ocean
    V0(MASKb==1)=0;
    V9(MASKb==1)=-1;
    R0(MASKb==1)=0;

    MASKb=zeros(ctr.imax,ctr.jmax);
    MASKb(1,2:ctr.jmax-1)=1; % periodic BC at i=1 or ocean
    V0(MASKb==1)=0;
    V10(MASKb==1)=-1;
    R0(MASKb==1)=0;

    MASKb=zeros(ctr.imax,ctr.jmax);
    MASKb(ctr.imax,2:ctr.jmax-1)=1; % periodic BC at i=imax or ocean
    V0(MASKb==1)=0;
    V11(MASKb==1)=-1;
    R0(MASKb==1)=0;

    MASKb=zeros(ctr.imax,ctr.jmax);
    MASKb(1:ctr.imax,ctr.jmax)=1; % ocean
    V0(MASKb==1)=0;
    V12(MASKb==1)=-1;
    R0(MASKb==1)=0;

    MASKb=zeros(ctr.imax,ctr.jmax);
    MASKb(1,:)=1;
    MASKb(ctr.imax,:)=1;
    MASKb(:,1)=1;
    MASKb(:,ctr.jmax)=1;
    V1(MASKb==1)=0;
    V2(MASKb==1)=0;
    V3(MASKb==1)=0;
    V4(MASKb==1)=0;
    if par.upstream==1
        V5(MASKb==1)=0;
        V6(MASKb==1)=0;
        V7(MASKb==1)=0;
        V8(MASKb==1)=0;
    end
end

if par.upstream==1
    V=[reshape(V0(VM==1)*par.omega+1,nodes,1)
        V1(V1~=0)*par.omega
        V2(V2~=0)*par.omega
        V3(V3~=0)*par.omega
        V4(V4~=0)*par.omega
        V5(V5~=0)*par.omega
        V6(V6~=0)*par.omega
        V7(V7~=0)*par.omega
        V8(V8~=0)*par.omega
        V9(V9~=0)
        V10(V10~=0)
        V11(V11~=0)
        V12(V12~=0)];

    row=[reshape(node(VM==1),nodes,1)
        node(V1~=0)
        node(V2~=0)
        node(V3~=0)
        node(V4~=0)
        node(V5~=0)
        node(V6~=0)
        node(V7~=0)
        node(V8~=0)
        node(V9~=0)
        node(V10~=0)
        node(V11~=0)
        node(V12~=0)];
else
    V=[reshape(V0(VM==1)*par.omega+1,nodes,1)
        V1(V1~=0)*par.omega
        V2(V2~=0)*par.omega
        V3(V3~=0)*par.omega
        V4(V4~=0)*par.omega];

    row=[reshape(node(VM==1),nodes,1)
        node(V1~=0)
        node(V2~=0)
        node(V3~=0)
        node(V4~=0)];
end

nodeV1=circshift(node,[0 -1]); % i,j+1
nodeV2=circshift(node,[0 1]); % i,j-1
nodeV3=circshift(node,[-1 0]); % i+1,j
nodeV4=circshift(node,[1 0]); % i-1,j
if par.upstream==1
    nodeV5=circshift(node,[0 2]); % i,j-2
    nodeV6=circshift(node,[2 0]); % i-2,j
    nodeV7=circshift(node,[0 -2]); % i,j+2
    nodeV8=circshift(node,[-2 0]); % i+2,j
end
if ctr.mismip==1
    nodeV9=circshift(node,[0 -2]); % i,j+2 - divide
    nodeV10=circshift(node,[2 0]); % imax-1,j - PBC at i=1
    nodeV11=circshift(node,[-2 0]); % 2,j - PBC at i=imax
    nodeV12=circshift(node,[0 1]); % i,jmax-1 - ocean
else
    nodeV9=circshift(node,[0 -1]); % i,j+1 - ocean
    nodeV10=circshift(node,[-1 0]); % 2,j - ocean
    nodeV11=circshift(node,[1 0]); % 2,j - ocean
    nodeV12=circshift(node,[0 1]); % i,jmax-1 - ocean
end

if par.upstream==1
    col=[reshape(node(VM==1),nodes,1)
        nodeV1(V1~=0)
        nodeV2(V2~=0)
        nodeV3(V3~=0)
        nodeV4(V4~=0)
        nodeV5(V5~=0)
        nodeV6(V6~=0)
        nodeV7(V7~=0)
        nodeV8(V8~=0)
        nodeV9(V9~=0)
        nodeV10(V10~=0)
        nodeV11(V11~=0)
        nodeV12(V12~=0)];
else
    col=[reshape(node(VM==1),nodes,1)
        nodeV1(V1~=0)
        nodeV2(V2~=0)
        nodeV3(V3~=0)
        nodeV4(V4~=0)];
end

R=reshape(R0(VM==1),nodes,1);

% construct sparse matrix
A=sparse(row,col,V);
% Cholesky factor and solve
if ctr.inverse==1
    s=A\R;
else
    tol=1e-7; % 1e-5    %LZ: tol changed (before 1e-5) 
    maxit=20; % 15
    D=diag(diag(A));
    C1=tril(A);
    C2=D\triu(A);
    [s,flag]=pcg(A,R,tol,maxit,C1,C2);
    if flag>0 || cnt==1
        s=A\R;
    end
end

H(node>0)=s(node(node>0));

end


function [arcocn,distocn,distgl]=OceanArc(MASK,H,MASKlk,ctr,par)

arcocn=zeros(ctr.imax,ctr.jmax);
distocn=zeros(ctr.imax,ctr.jmax)+1000e3;
distgl=zeros(ctr.imax,ctr.jmax)+10000e3;
narc=72; % number of directions (every 5 degree)
angarc=zeros(1,narc);
tanarc=zeros(1,narc);
ifi=zeros(1,narc);
idir=zeros(1,narc);
MASK(MASK==0&H>par.SeaIceThickness)=3;  %mark shelves in MASK
MASK1=circshift(MASK,[0 -1]); % MASK(i,j+1)
MASK2=circshift(MASK,[0 1]); % MASK(i,j-1)
MASK3=circshift(MASK,[-1 0]); % MASK(i+1,j)
MASK4=circshift(MASK,[1 0]); % MASK(i-1,j)
MASK5=circshift(MASK,[-1 -1]); % MASK(i+1,j+1)
MASK6=circshift(MASK,[-1 1]); % MASK(i+1,j-1)
MASK7=circshift(MASK,[1 -1]); % MASK(i-1,j+1)
MASK8=circshift(MASK,[1 1]); % MASK(i-1,j-1)
ifdo=zeros(ctr.imax,ctr.jmax);
ifdo(MASK~=1&MASKlk==0&(MASK1~=0|MASK2~=0|MASK3~=0|MASK4~=0|MASK5~=0|MASK6~=0|MASK7~=0|MASK8~=0))=1;
[jo,io] = meshgrid(1:ctr.imax,1:ctr.jmax);
distocn(MASK==0)=0;

for mm=1:narc
    angarc(mm)=-pi+(mm-0.5)*(2*pi)/narc;
    tanarc(mm)=tan(angarc(mm));
    if abs(angarc(mm))>=0.75*pi
        ifi(mm)=1;
        idir(mm)=-1;
    elseif angarc(mm)>=-0.75*pi && angarc(mm)<=-0.25*pi
        ifi(mm)=0;
        idir(mm)=-1;
    elseif abs(angarc(mm))<=0.25*pi
        ifi(mm)=1;
        idir(mm)=1;
    else
        ifi(mm)=0;
        idir(mm)=1;
    end
end

[arcocn(ifdo==1),distocn(ifdo==1),distgl(ifdo==1)]=arrayfun(@(io,jo) calcOceanArc(io,jo,MASK,ctr.imax,ctr.jmax,ctr.delta,narc,tanarc,ifi,idir),io(ifdo==1),jo(ifdo==1));

arcocn(MASK~=1&ifdo==0&MASKlk==0)=360;
distocn(MASK==0)=0;
distgl(MASK~=1&ifdo==0)=0;
end


function [arcocn,distocn,distgl]=calcOceanArc(io,jo,MASK,imax,jmax,delta,narc,tanarc,ifi,idir)
    
    arcocn=0;
    distocn=1e6;
    distgl=1e7;
    for mm=1:narc
        idirmm=idir(mm);
        tanarcmm=tanarc(mm);
        calcdist=0;
        if ifi(mm)==1
            ii=io;
            if idirmm==-1
                ttmax=io-1;
            else
                ttmax=imax-io-1;
            end
            for tt=1:ttmax
                ii=ii+idirmm;
                jj=round((ii-io)*tanarcmm)+jo;
                if jj<1 || jj>jmax
                    arcocn=arcocn+360./narc;
                    break
                end
                if MASK(ii,jj)==1
                    zdist = delta * sqrt((ii-io)^2+(jj-jo)^2);
                    distgl=min(distgl,zdist);
                    break
                elseif MASK(ii,jj)==0 && calcdist==0
                    zdist = delta * sqrt((ii-io)^2+(jj-jo)^2);
                    distocn=min(distocn,zdist);
                    calcdist=1;
                end
            end
        else
            jj=jo;
            if idirmm==-1
                ttmax=jo-1;
            else
                ttmax=jmax-jo-1;
            end
            for tt=1:ttmax
                jj=jj+idirmm;
                ii=round((jj-jo)*tanarcmm)+io;
                if ii<1 || ii>imax
                    arcocn=arcocn+360./narc;
                    break
                end
                if MASK(ii,jj)==1
                    zdist = delta * sqrt((ii-io)^2+(jj-jo)^2);
                    distgl=min(distgl,zdist);
                    break
                elseif MASK(ii,jj)==0 && calcdist==0
                    zdist = delta * sqrt((ii-io)^2+(jj-jo)^2);
                    distocn=min(distocn,zdist);
                    calcdist=1;
                end
            end
        end
        if tt==ttmax
            arcocn=arcocn+360./narc;
        end
    end
end


function Arc=ExtrapolateArc(MASK,oldMASK,Arc,Arc0,ctr)

% Interpolate Arc between consecutive calls (to improve
% calculation speed)

    dMASK=MASK-oldMASK; % difference between two consecutive
                        % masks (grounded - floated)
    Me=zeros(ctr.imax,ctr.jmax,8);
    OldArc=Arc;
    OldArc(MASK==1)=NaN;
    Me(:,:,1)=circshift(OldArc,[-1 1]);
    Me(:,:,2)=circshift(OldArc,[0 1]);
    Me(:,:,3)=circshift(OldArc,[1 1]);
    Me(:,:,4)=circshift(OldArc,[1 0]);
    Me(:,:,5)=circshift(OldArc,[-1 0]);
    Me(:,:,6)=circshift(OldArc,[-1 -1]);
    Me(:,:,7)=circshift(OldArc,[0 -1]);
    Me(:,:,8)=circshift(OldArc,[1 -1]);
    NewArc=nanmean(Me,3); % Take mean of neighbours for which Arc exists
    NewArc(isnan(NewArc))=0;
    Arc(dMASK==-1)=NewArc(dMASK==-1); % Apply Arc for points becoming floated
    % Apply Arc for points that were previously determined
    Arc(dMASK==-1 & Arc0~=0)=Arc0(dMASK==-1 & Arc0~=0);
    Arc(dMASK==1)=0; % remove melt for points becoming grounded
end


%----------------------------------------------------------------------
% Solving thin-plate equation with spatially-varying flexural rigidity
%----------------------------------------------------------------------

function bload=sparse_solver_bedrock(Db,par,ctr,VM,node,nodes,loadB)

Db1=circshift(Db,[0 -1]); % d(i,j+1)
Db2=circshift(Db,[0  1]); % d(i,j-1)
Db3=circshift(Db,[-1 0]); % d(i+1,j)
Db4=circshift(Db,[1 0]); % d(i-1,j)
Db5=circshift(Db,[-1 -1]); % d(i+1,j+1)
Db6=circshift(Db,[1  -1]); % d(i-1,j+1)
Db7=circshift(Db,[-1 1]); % d(i+1,j-1)
Db8=circshift(Db,[1 1]); % d(i-1,j-1)

MASKb=zeros(ctr.imax,ctr.jmax);
MASKb(1:2,:)=1;
MASKb(ctr.imax-1:ctr.imax,:)=1;
MASKb(:,1:2)=1;
MASKb(:,ctr.jmax-1:ctr.jmax)=1;

nabla2Db = (-4*Db+Db1+Db2+Db3+Db4);
dDbx = (Db1-Db2);
dDby = (Db3-Db4);
dDbx2 = (-2*Db+Db1+Db2);
dDbxy = (Db5-Db6-Db7+Db8)/4;
dDby2 = (-2*Db+Db3+Db4);

V0 = (20*Db-4*nabla2Db-(1-par.nuB)*(-2*dDbx2-2*dDby2))/ctr.delta^4+par.rhom*par.g;
V1 = (-8*Db-dDbx-dDbx+nabla2Db-(1-par.nuB)*dDby2)/ctr.delta^4;
V2 = (-8*Db+dDbx+dDbx+nabla2Db-(1-par.nuB)*dDby2)/ctr.delta^4;
V3 = (-8*Db-dDby-dDby+nabla2Db-(1-par.nuB)*dDbx2)/ctr.delta^4; 
V4 = (-8*Db+dDby+dDby+nabla2Db-(1-par.nuB)*dDbx2)/ctr.delta^4;         
V5 = (2*Db+0.5*dDbx+0.5*dDby-0.5*(1-par.nuB)*(-dDbxy))/ctr.delta^4;
V6 = (2*Db+0.5*dDbx-0.5*dDby-0.5*(1-par.nuB)*(dDbxy))/ctr.delta^4;
V7 = (2*Db-0.5*dDbx+0.5*dDby-0.5*(1-par.nuB)*(dDbxy))/ctr.delta^4;
V8 = (2*Db-0.5*dDbx-0.5*dDby-0.5*(1-par.nuB)*(-dDbxy))/ctr.delta^4;
V9 = (Db+0.5*dDbx)/ctr.delta^4;
V10 = (Db-0.5*dDbx)/ctr.delta^4;
V11 = (Db+0.5*dDby)/ctr.delta^4;
V12 = (Db-0.5*dDby)/ctr.delta^4;

R0 = loadB;

V0(MASKb==1)=1;
V1(MASKb==1)=0;
V2(MASKb==1)=0;
V3(MASKb==1)=0;
V4(MASKb==1)=0;
V5(MASKb==1)=0;
V6(MASKb==1)=0;
V7(MASKb==1)=0;
V8(MASKb==1)=0;
V9(MASKb==1)=0;
V9(MASKb==1)=0;
V10(MASKb==1)=0;
V11(MASKb==1)=0;
V12(MASKb==1)=0;
R0(MASKb==1)=0;

V=[reshape(V0(VM==1),nodes,1)
    V1(V1~=0)
    V2(V2~=0)
    V3(V3~=0)
    V4(V4~=0)
    V5(V5~=0)
    V6(V6~=0)
    V7(V7~=0)
    V8(V8~=0)
    V9(V9~=0)
    V10(V10~=0)
    V11(V11~=0)
    V12(V12~=0)
    ];

row=[reshape(node(VM==1),nodes,1)
    node(V1~=0)
    node(V2~=0)
    node(V3~=0)
    node(V4~=0)
    node(V5~=0)
    node(V6~=0)
    node(V7~=0)
    node(V8~=0)
    node(V9~=0)
    node(V10~=0)
    node(V11~=0)
    node(V12~=0)
    ];

nodeV1=circshift(node,[0 -1]); % i,j+1
nodeV2=circshift(node,[0 1]); % i,j-1
nodeV3=circshift(node,[-1 0]); % i+1,j
nodeV4=circshift(node,[1 0]); % i-1,j
nodeV5=circshift(node,[-1 -1]); % i+1,j+1
nodeV6=circshift(node,[1 -1]); % i-1,j+1
nodeV7=circshift(node,[-1 1]); % i+1,j-1
nodeV8=circshift(node,[1 1]); % i-1,j-1
nodeV9=circshift(node,[0 -2]); % i,j+2
nodeV10=circshift(node,[0 2]); % i,j-2
nodeV11=circshift(node,[-2 0]); % i+2,j
nodeV12=circshift(node,[2 0]); % i-2,j

col=[reshape(node(VM==1),nodes,1)
    nodeV1(V1~=0)
    nodeV2(V2~=0)
    nodeV3(V3~=0)
    nodeV4(V4~=0)
    nodeV5(V5~=0)
    nodeV6(V6~=0)
    nodeV7(V7~=0)
    nodeV8(V8~=0)
    nodeV9(V9~=0)
    nodeV10(V10~=0)
    nodeV11(V11~=0)
    nodeV12(V12~=0)
    ];

R=reshape(R0(VM==1),nodes,1);

% construct sparse matrix

A=sparse(row,col,V);

% solve

s=A\R;
bload = zeros(ctr.imax,ctr.jmax);
bload(node>0)=s(node(node>0));

end



% Sainan: ISMIP6 ocean parameterization equation (1)
function Melt=ISMIP6_ocean(par,ctr,ice_draft,icemask_shelves,z,thermal_forcing,basinNumber,deltaT_basin)
    
    cste = (par.rhow*par.cp0/(par.rho*par.Latent)).^2;  % in K^(-2)

    Nbasin=max(max(basinNumber))+1;

    TF_draft=zeros(ctr.imax,ctr.jmax);
    Melt=zeros(ctr.imax,ctr.jmax);
    mean_TF=zeros(Nbasin,1);

    TF_up=thermal_forcing(:,:,1);
    TF_draft(ice_draft>=z(1))=TF_up(ice_draft>=z(1));
    TF_down=thermal_forcing(:,:,end);
    TF_draft(ice_draft<=z(end))=TF_down(ice_draft<=z(end));
    for k=length(z):-1:2
        TF_up=thermal_forcing(:,:,k);
        TF_down=thermal_forcing(:,:,k-1);
        TF_draft(ice_draft>=z(k) & ice_draft<=z(k-1))=...
            ((z(k)-ice_draft(ice_draft>=z(k) & ice_draft<=z(k-1))).*TF_down(ice_draft>=z(k) & ice_draft<=z(k-1))...
            +(ice_draft(ice_draft>=z(k) & ice_draft<=z(k-1))-z(k-1)).*TF_up(ice_draft>=z(k) & ice_draft<=z(k-1)))...
            ./(z(k)-z(k-1));
    end
    TF_draft(icemask_shelves<5.e-1 | ice_draft>0.)=-9999.9;
    basinNumber=basinNumber+1;
    for i=1:Nbasin
        mean_TF(i)=mean(TF_draft(basinNumber==i & icemask_shelves>5.e-1 & ice_draft<=0.));
    end

    % Step3 calculation of melting rate
    % melt rate in m/yr (meters of pure water per year):
    % [*rhofw_SI/rhoi_SI to get it in meters of ice per year]

    for i=1:Nbasin
        Melt(basinNumber==i)=par.gamma0.*cste.*(TF_draft(basinNumber==i)+deltaT_basin(basinNumber==i))...
            .*abs(mean_TF(i)+deltaT_basin(basinNumber==i));
    end
    Melt(TF_draft<=-5)=0;
    Melt=Melt*par.rhof/par.rho;
end


function [MASKHole,HoleCT]=ShelfHole(MASK,H,par)
    MASK(MASK==0 & H>par.SeaIceThickness)=3;
    [~,MASKHole]=bwboundaries(MASK>0,4);
    MASKHole(MASK==1 | H>par.SeaIceThickness)=0;
    MASKHole(MASKHole~=0)=1;
    H1=circshift(MASKHole,[-1 0]); % i+1,j
    H2=circshift(MASKHole,[1 0]); % i-1,j
    H3=circshift(MASKHole,[0 -1]); % i,j+1
    H4=circshift(MASKHole,[0 1]); % i,j-1
    HoleCT=H1==1 | H2==1 | H3==1 | H4==1;
end


function C = conv2fft(varargin)
% C = conv2fft(A, B)
% C = conv2fft(H1, H2, A)
%
%   C = CONV2FFT(A, B) performs the 2-D convolution of matrices A and B.
%   If [ma,na] = size(A), [mb,nb] = size(B), and [mc,nc] = size(C), then
%   mc = max([ma+mb-1,ma,mb]) and nc = max([na+nb-1,na,nb]).
%
%   C = CONV2FFT(H1, H2, A) first convolves each column of A with the vector
%   H1 and then convolves each row of the result with the vector H2.  If
%   n1 = length(H1), n2 = length(H2), and [mc,nc] = size(C) then
%   mc = max([ma+n1-1,ma,n1]) and nc = max([na+n2-1,na,n2]).
%   CONV2(H1, H2, A) is equivalent to CONV2FFT(H1(:)*H2(:).', A) up to
%   round-off.
%
%   C = CONV2FFT(..., SHAPE) returns a subsection of the 2-D
%   convolution with size specified by SHAPE:
%     'full'  - (default) returns the full 2-D convolution,
%     'same'  - returns the central part of the convolution
%               that is the same size as A.
%     'valid' - returns only those parts of the convolution
%               that are computed without the zero-padded edges.
%               size(C) = max([ma-max(0,mb-1),na-max(0,nb-1)],0).
%
%   See also CONV2, CONVN, CONVNFFT
%
%   Author: Bruno Luong <brunoluong@yahoo.com>
%   History:
%       Original: 21-April-2014
if length(varargin)>=3 && isnumeric(varargin{3})
    [H1, H2, A] = deal(varargin{1:3});
    varargin = varargin(4:end);
    C = convnfft(H1(:), A, varargin{:});
    C = convnfft(H2(:).', C, varargin{:});
else
    C = convnfft(varargin{:});
end
end % conv2fft

function A = convnfft(A, B, shape, dims, options)
% CONVNFFT  FFT-BASED N-dimensional convolution.
%   C = CONVNFFT(A, B) performs the N-dimensional convolution of
%   matrices A and B. If nak = size(A,k) and nbk = size(B,k), then
%   size(C,k) = max([nak+nbk-1,nak,nbk]);
% 
%   C = CONVNFFT(A, B, SHAPE) controls the size of the answer C:
%       'full'   - (default) returns the full N-D convolution
%       'same'   - returns the central part of the convolution that
%                  is the same size as A.
%       'valid'  - returns only the part of the result that can be
%                  computed without assuming zero-padded arrays.
%                  size(C,k) = max([nak-max(0,nbk-1)],0).
%
%   C = CONVNFFT(..., SHAPE, DIMS) with DIMS is vector of dimensions where
%       the convolution will be carried out. By default DIMS is
%       [1:max(ndims(A),ndims(B))] (all dimensions). A and B must have the
%       same lengths on other dimensions.
%   C = CONVNFFT(..., SHAPE, DIMS, GPU)
%       GPU is boolean flag, see next
%
%   C = CONVNFFT(..., SHAPE, DIMS, OPTIONS)
%
%   OPTIONS is structure with following optional fields
%       - 'GPU', boolean. If GPU is TRUE Jacket/GPU FFT engine will be used
%       By default GPU is FALSE.
%       - 'Power2Flag', boolean. If it is TRUE, use FFT with length rounded
%       to the next power-two. It is faster but requires more memory.
%       Default value is TRUE.
%
% Class support for inputs A,B:
% float: double, single
%
% METHOD: CONVNFFT uses Fourier transform (FT) convolution theorem, i.e.
%         FT of the convolution is equal to the product of the FTs of the
%         input functions.
%         In 1-D, the complexity is O((na+nb)*log(na+nb)), where na/nb are
%         respectively the lengths of A and B.
%
% Usage recommendation:
%         In 1D, this function is faster than CONV for nA, nB > 1000.
%         In 2D, this function is faster than CONV2 for nA, nB > 20.
%         In 3D, this function is faster than CONVN for nA, nB > 5.
% 
% See also conv, conv2, convn.
% 
%   Author: Bruno Luong <brunoluong@yahoo.com>
%   History:
%       Original: 21-Jun-2009
%       23-Jun-2009: correct bug when ndims(A)<ndims(B)
%       02-Sep-2009: GPU/JACKET option
%       04-Sep-2009: options structure
%       16-Sep-2009: inplace product
if nargin<3 || isempty(shape)
    shape = 'full';
end
if nargin<5 || isempty(options)
    options = struct();
elseif ~isstruct(options) % GPU options
    options = struct('GPU', options);
end
nd = max(ndims(A),ndims(B));
% work on all dimensions by default
if nargin<4 || isempty(dims)
    dims = 1:nd;
end
dims = reshape(dims, 1, []); % row (needed for for-loop index)
% GPU enable flag
GPU = getoption(options, 'GPU', false);
% Check if Jacket is installed
GPU = GPU && ~isempty(which('ginfo'));
% IFUN function will be used later to truncate the result
% M and N are respectively the length of A and B in some dimension
switch lower(shape)
    case 'full',
        ifun = @(m,n) 1:m+n-1;
    case 'same',
        ifun = @(m,n) ceil((n-1)/2)+(1:m);
    case 'valid',
        ifun = @(m,n) n:m;
    otherwise
        error('convnfft: unknown shape %s', shape);
end
classA = class(A);
classB = class(B);
ABreal = isreal(A) && isreal(B);
% Special case, empty convolution, try to follow MATLAB CONVN convention
if any(size(A)==0) || any(size(B)==0)
    szA = zeros(1,nd); szA(1:ndims(A))=size(A);
    szB = zeros(1,nd); szB(1:ndims(B))=size(B);
    % Matlab wants these:
    szA = max(szA,1); szB = max(szB,1);
    szC = szA;
    for dim=dims
        szC(dim) = length(ifun(szA(dim),szB(dim)));
    end
    A = zeros(szC,classA); % empty -> return zeros
    return
end
power2flag = getoption(options, 'Power2Flag', true);
if power2flag
    % faster FFT if the dimension is power of 2
    lfftfun = @(l) 2^nextpow2(l);
else
    % slower, but smaller temporary arrays
    lfftfun = @(l) l;
end
if GPU % GPU/Jacket FFT
    if strcmp(classA,'single')
        A = gsingle(A);
    else
        A = gdouble(A);
    end
    if strcmp(classB,'single')
        B = gsingle(B);
    else
        B = gdouble(B);
    end
    % Do the FFT
    subs(1:ndims(A)) = {':'};
    for dim=dims
        m = size(A,dim);
        n = size(B,dim);
        % compute the FFT length
        l = lfftfun(m+n-1);
        % We need to swap dimensions because GPU FFT works along the
        % first dimension
        if dim~=1 % do the work when only required
            swap = 1:nd;
            swap([1 dim]) = swap([dim 1]);
            A = permute(A, swap);
            B = permute(B, swap);
        end
        A = fft(A,l);
        B = fft(B,l);
        subs{dim} = ifun(m,n);
    end
else % Matlab FFT
    % Do the FFT
    subs(1:ndims(A)) = {':'};
    for dim=dims
        m = size(A,dim);
        n = size(B,dim);
        % compute the FFT length
        l = lfftfun(m+n-1);
        A = fft(A,l,dim);
        B = fft(B,l,dim);
        subs{dim} = ifun(m,n);
    end
end
 
if GPU
    A = A.*B;
    clear B
else
%     % inplace product to save 1/3 of the memory
%     inplaceprod(A,B);
    A = A.*B;
end
% Back to the non-Fourier space
if GPU % GPU/Jacket FFT
    for dim=dims(end:-1:1) % reverse loop
        A = ifft(A,[]);
        % Swap back the dimensions
        if dim~=1 % do the work when only required
            swap = 1:nd;
            swap([1 dim]) = swap([dim 1]);
            A = permute(A, swap);
        end        
    end   
else % Matlab IFFT  
    for dim=dims
        A = ifft(A,[],dim);
    end
end
% Truncate the results
if ABreal
    % Make sure the result is real
    A = real(A(subs{:}));
else
    A = A(subs{:});
end
% GPU/Jacket
if GPU
    % Cast the type back
    if strcmp(class(A),'gsingle')
        A = single(A);
    else
        A = double(A);
    end
end
end % convnfft
% Get defaut option
function value = getoption(options, name, defaultvalue)
% function value = getoption(options, name, defaultvalue)
    value = defaultvalue;
    fields = fieldnames(options);
    found = strcmpi(name,fields);
    if any(found)
        i = find(found,1,'first');
        if ~isempty(options.(fields{i}))
            value = options.(fields{i});
        end
    end
end


